/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -  Elias Papadakis <ep@intelligentmedia.gr>
 */
/* global DeviceManager, Logger, utils, logic, application, XMRManager, AppVersionInformation, defined, md5, b2bapis, ZmqSocket, io, webapis, tizen */

(function () {
  "use strict";

  /**
   * @classdesc Specific to Tizen device.
   * @type DeviceManager
   * @version 1.8.1
   * @since 1.8.1
   */
  //jshint -W057
  window.DeviceManager = new function _DeviceManager() {

    /**
     * Not used in Tizen.
     */
    var v_debug = false;

    /**
     * Configuration for the application.
     */
    this.Config = new function _Config() {

      /* [ Private variables ] **************************************************/
      // <editor-fold defaultstate="collapsed" desc="Private variables">

      //var v_CMSVersion;

      /**
       * Access the current CMS configuration.
       */
      var playerInfo = {
        /* For development only */
//					remoteURL : "http://testcms.imds.gr/",
        // remoteURL : "https://qatest.xibo.co.uk/",
        //remotePsw : "Z8aDtpq8",
//					remoteURL : "https://lgplayer.xibo.cloud/",
//					remotePsw : "m7TYxuft",
        /* For development only */
        remoteURL: "https://",
        remotePsw: null,

        displayName: "Tizen",
        hardwareKey: null, /* This is the md5 of the MAC */
        macAddress: null,
        xmrChannel: null,
        xmrPubKey: null
      };

      /**
       * Screenshoots inverval timer object
       */
      var v_ScreenShotIntervalTmr;

      /* For development only */
      if (AppVersionInformation.isDevVersion) {
//				playerInfo.remoteURL = "https://lgplayer.xibo.cloud/";
//				playerInfo.remotePsw = "m7TYxuft";
      }
      /* For development only */
      // </editor-fold>

      /* [ Private functions ] **************************************************/
      // <editor-fold defaultstate="collapsed" desc="Private functions">

      var getMac = function (cbSuccess, cbFailure) {

        try {
          playerInfo.macAddress = b2bapis.b2bcontrol.getMACAddress();
          playerInfo.hardwareKey = md5(playerInfo.macAddress);
          cbSuccess(true);
        } catch (e) {
          cbFailure(e);
        }
      };

      var createXMRChannel = function () {

        /* New URL means new XMR Channel */
        playerInfo.xmrChannel = md5(playerInfo.remoteURL + playerInfo.remotePsw + playerInfo.macAddress);

      };

      var sendScreenShotRepeatedly = function () {
        if (playerInfo.options.screenShotRequestInterval) {
          v_ScreenShotIntervalTmr = setTimeout(
            function () {
              ScheduleManager.Commands.takeScreenshot();
              sendScreenShotRepeatedly();
            },
            playerInfo.options.screenShotRequestInterval * 60000
          );
        }
      };
      // </editor-fold>

      /* [ Public functions ] ***************************************************/
      // <editor-fold defaultstate="collapsed" desc="Public functions">

      /**
       * Check if configuration file exists.
       * @param {function} cbSuccess Callback to call on success. Provides a boolean parameter with true if configuration exists else false.
       * @param {function} cbFailure Callback to call on error.
       * @returns {undefined} Does not return.
       * @see It will also check configuration parameters for validity!
       * @version 1.8.1
       * @since 1.8.1
       */
      this.configurationExists = function (cbSuccess, cbFailure) {
        getMac(function () {
          if (!defined(localStorage.deviceConfiguration)) {
            /* Create XMR Key and Channel */
            cbSuccess(false);
            return;
          } else {
            cbSuccess(true);
          }

        }, cbFailure);
      };

      /**
       * Load the configuration from file.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.loadConfiguration = function () {
        var tempXMRKey;
        if (playerInfo.hasOwnProperty('xmrPubKey')) {
          if (playerInfo.xmrPubKey !== null) {
            tempXMRKey = playerInfo.xmrPubKey;
          } else {
            XMRManager.createPubKey();
            tempXMRKey = playerInfo.xmrPubKey;
          }
        }
        playerInfo = JSON.parse(localStorage.deviceConfiguration);
        if (defined(tempXMRKey)) {
          playerInfo.xmrPubKey = tempXMRKey;
        }
      };

      /**
       * Save the configuration to file.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.saveConfiguration = function () {
        localStorage.deviceConfiguration = JSON.stringify(playerInfo);
      };

      /**
       * Check if settings are valid.
       * @returns {Boolean} Returns true if there are else false.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.settingsValid = function () {
        /* Check URL */
        if (!defined(playerInfo.remoteURL)) {
          return false;
        }

        /* Check password */
        if (!defined(playerInfo.remotePsw)) {
          return false;
        }

        return true;
      };

      /**
       * Set if CMS is valid
       * @param {type} isValid True if CMS is valid else false.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setCMSValid = function (isValid) {
        localStorage.playerConnectedOnceToCMS = isValid;
      };

      /**
       * Checks for CMS validity. CMS is valid if connect sucessfully at least once!
       * @returns {Boolean} True if valid else false.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.isCMSValid = function () {
        if (!localStorage.hasOwnProperty("playerConnectedOnceToCMS")) {
          return false;
        }

        if (!localStorage.playerConnectedOnceToCMS) {
          return false;
        }
        return true;
      };

      /**
       * Set the CMS url.
       * @param {String} url The remote CMS URL.
       * @returns {Boolean} True if set else false.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setURL = function (url) {
        if (url.length < 11) {
          return false;
        }

        if (url !== playerInfo.remoteURL) {
          Logger.clearLogs(function () {
            Logger.clearStats(function () {
              console.log("Logs and stats cleared.");
            }, function () {
              console.log("Clear stats failed.");
            });
          }, function () {
            console.log("Clear logs failed.");
          });
        }

        playerInfo.remoteURL = url;
        if (url.slice(-1) !== "/") {
          playerInfo.remoteURL += "/";
        }

        createXMRChannel();

        return true;
      };

      /**
       * Get the CMS URL.
       * @returns {String}  The CMS URL.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getURL = function () {
        return playerInfo.remoteURL;
      };

      /**
       * Set the CMS password.
       * @param {String} pass The remote CMS password.
       * @returns {Boolean} True if set else false.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setPassword = function (pass) {
        if (pass.length < 1) {
          return false;
        }
        playerInfo.remotePsw = pass;
        createXMRChannel();
        return true;
      };

      /**
       * Get the password for the CMS server.
       * @returns {String} The password for the CMS server.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getPassword = function () {
        return playerInfo.remotePsw;
      };

      /**
       * Get the monitor's hardware key.
       * @returns {String} Monitor's hardware key.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getHardwareKey = function () {
        return playerInfo.hardwareKey;
      };

      /**
       * Get the display name.
       * @returns {String} The display name.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getDisplayName = function () {
        var dn = "sssp";
        if (!playerInfo.hasOwnProperty("displayName")) {
          //Logger.cmsLog({category:"error", message:"Display name is not available.", method:"getDisplayName", thread:"DeviceManager.Config"});
          // XXX: make a suggestion for a proper error code for this case
          Logger.cmsFault(undefined, undefined,
            {
              code: ERRORS.FaultCodes.RemoteResourceFailed,
              reason: "Display name is not available.",
              method: "getDisplayName",
              thread: "DeviceManager.Config",
              date: new Date(),
              expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000)
            }
          );
          return dn;
        }
        if (playerInfo.displayName.length === 0) {
          //Logger.cmsLog({category:"error", message:"Display name is empty.", method:"getDisplayName", thread:"DeviceManager.Config"});
          // XXX: make a suggestion for a proper error code for this case
          Logger.cmsFault(undefined, undefined,
            {
              code: ERRORS.FaultCodes.RemoteResourceFailed,
              reason: "Display name is empty.",
              method: "getDisplayName",
              thread: "DeviceManager.Config",
              date: new Date(),
              expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000)
            }
          );
          return dn;
        }
        dn = playerInfo.displayName;
        return dn;
      };

      /**
       * Get the monitor's MAC address.
       * @returns {Number} Return the monitor's MAC address.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getMacAddress = function () {
        return playerInfo.macAddress;
      };


      /**
       * Get the Xmr Channel.
       * @returns {String} Return the Xmr Channel.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getXmrChannel = function () {
        return playerInfo.xmrChannel;
      };

      /**
       * Get the Xmr Public Key.
       * @returns {String} Return the Xmr Public Key.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getXmrPubKey = function () {
        return playerInfo.xmrPubKey;
      };

      /**
       * Set the XMR Public Key.
       * @param {String} key The public key (in string representation).
       * @returns None.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setXmrPubKey = function (key) {
        playerInfo.xmrPubKey = key;
      };

      /**
       * Get options provided from CMS.
       * @returns {playerInfo.options} CMS options.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getOptions = function () {
        return playerInfo.options;
      };

      /**
       * Set options provided from CMS.
       * @param {Object} options CMS options.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setOptions = function (options) {
        playerInfo.options = options;
        if (options.hasOwnProperty("displayName")) {
          if (options.displayName.length > 0) {
            playerInfo.displayName = options.displayName;
          } else {
            //Logger.cmsLog({category:"error", message:"Display name is empty.", method:"setOptions", thread:"DeviceManager.Config"});
            // XXX: make a suggestion for a proper error code for this case
            Logger.cmsFault(undefined, undefined,
              {
                code: ERRORS.FaultCodes.RemoteResourceFailed,
                reason: "Display name is empty.",
                method: "setOptions",
                thread: "DeviceManager.Config",
                date: new Date(),
                expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000)
              });
          }
        } else {
          //Logger.cmsLog({category:"error", message:"No display name field in options.", method:"setOptions", thread:"DeviceManager.Config"});
          // XXX: make a suggestion for a proper error code for this case
          Logger.cmsFault(undefined, undefined,
            {
              code: ERRORS.FaultCodes.RemoteResourceFailed,
              reason: "No display name field in options.",
              method: "setOptions",
              thread: "DeviceManager.Config",
              date: new Date(),
              expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000)
            });
        }
        if (options.hasOwnProperty("timers")) {
          DeviceManager.Device.Power.SetTimers(null, logic.nonFatalError, options.timers);
        }

        if (options.hasOwnProperty('pictureOptions')) {
          if (options.pictureOptions.length > 0 && options.pictureOptions !== "{}") {
            try {
              var popts = JSON.parse(options.pictureOptions);
              DeviceManager.Device.setPictureOptions(null, logic.nonFatalError, popts);
            } catch (e) {
              logic.nonFatalError(e);
            }
          }
        }
        ;

        if (options.hasOwnProperty('lockOptions')) {
          var lopts;
          try {
            lopts = JSON.parse(options.lockOptions);

          } catch (e) {
            var nem = "Unable to parse lock options. (" + e.message + ")";
            logic.nonFatalError(new Error(nem));
            return;
          }

          if (lopts.hasOwnProperty('usblock')) {
            DeviceManager.Device.setLockUSB(function () {
              console.log('USB is locked:' + lopts.usblock);
            }, logic.nonFatalError, lopts.usblock);
          }
          if (lopts.hasOwnProperty('osdlock')) {
            DeviceManager.Device.setLockOSD(function () {
              console.log('OSD is locked:' + lopts.osdlock);
            }, logic.nonFatalError, lopts.osdlock);
          }
          if (lopts.hasOwnProperty('keylock')) {
            DeviceManager.Device.setLockKeys(function () {
              console.log('Key locked:', lopts.keylock);
            }, logic.nonFatalError, lopts.keylock);
          }
          if (lopts.hasOwnProperty('safetylock')) {
            DeviceManager.Device.setSafetyLock(function () {
              console.log('Safety lock enabled:', lopts.safetylock);
            }, logic.nonFatalError, lopts.safetylock);
          }
        }
        ;

        if (options.serverPort) {
          if ((options.serverPort != localStorage.internalwsPort) &&
            (options.serverPort != "9696")) {
            localStorage.internalwsPort = options.serverPort;
            setTimeout(function () {
              DeviceManager.Device.powerCommand('reboot');
            }, 3000);

            return;
          }
        } else if (localStorage.internalwsPort) {
          delete localStorage.internalwsPort;
          setTimeout(function () {
            DeviceManager.Device.powerCommand('reboot');
          }, 1000);
          return;
        }

        if (playerInfo.remoteURL.slice(0, 7) === 'http://') {
          if (options.hasOwnProperty('forceHttps')) {
            if ((options.forceHttps * 1) === 1) {
              //var tmpUrl = playerInfo.remoteURL;
              var newURL = 'https://' + playerInfo.remoteURL.slice(7);
              //playerInfo.remoteURL =newURL;
              console.log(newURL)
              CommunicationManager.ConnectCMS(
                function (d) {
                  console.debug('Url changed to https', d);
                  playerInfo.remoteURL = newURL;
                }, function () {
                  //playerInfo.remoteURL = tmpUrl;
                },
                {URL: newURL}
              );
              //playerInfo.remoteURL = tmpUrl;
            }
          }
        }

        if (options.screenShotRequestInterval && options.screenShotRequestInterval != 0 && (!v_ScreenShotIntervalTmr)) {
          sendScreenShotRepeatedly();
        } else if (!options.screenShotRequestInterval && v_ScreenShotIntervalTmr) {
          clearTimeout(v_ScreenShotIntervalTmr);
        }
      };

      /**
       * Get commands provided from CMS.
       * @returns {playerInfo.commands} CMS commands.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getCommands = function () {
        return playerInfo.commands;
      };

      /**
       * Set commands provided from CMS.
       * @param {Object} commands CMS commands.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setCommands = function (commands) {
        playerInfo.commands = commands;
      };


      /**
       * Gets the CMS Version
       * @returns {Number | null} The CMS Version if exists else null.
       * @see If CMS Version is unavailable
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getCMSVersion = function () {
        if (localStorage.hasOwnProperty('CMSVersion')) {
          return localStorage.CMSVersion;
        }

        return null;
      };

      /**
       * Sets the CMS Version
       * @param {Number | String} version The CMS Version
       * @returns {Boolean} True if set else false.
       * @version 3.0.300
       * @since 1.8.1
       */
      this.setCMSVersion = function (version) {
        try {
          version = parseInt(version);
        } catch (e) {
          return false;
        }

        if (version < 4) {
          return false;
        }

        localStorage.CMSVersion = version;
        application.statusPage.supportsFaults(version >= 6);
        return true;
      };

      /**
       * Get the mail address used in Licensing.
       * @returns {String} The mail address.
       */
      this.getEmailAddress = function () {
        if (defined(playerInfo.options)) {
          if (playerInfo.options.hasOwnProperty("emailAddress")) {
            return playerInfo.options.emailAddress;
          }
        }
        return undefined;
      };

      /**
       * Set the mail address used in Licensing.
       * @param {type} email
       * @returns {undefined}
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setEmailAddress = function (email) {
        if (!defined(playerInfo.options)) {
          playerInfo.options = {};
        }

        if (!defined(email) || email === "") {
          email = DeviceManager.Config.getLocalMailAddress();
        }

        playerInfo.options.emailAddress = email;
      };

      /**
       * Get the local mail address. This one is set by user in settings page.
       * It is used as an alternative mail if the mail from CMS options is blank.
       * @returns {String} The local mail.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getLocalMailAddress = function () {
        if (localStorage.hasOwnProperty('localMailAddress')) {
          return localStorage.localMailAddress;
        }
        return "";
      };

      /**
       * Set the local mail address. This one is set by user in settings page.
       * It is used as an alternative mail if the mail from CMS options is blank.
       * @param {String} email The local mail.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setLocalMailAddress = function (email) {
        localStorage.localMailAddress = email;
      };

      /**
       * Get the log level provided from the CMS.
       * @returns {null|String} Return ['off', 'error', 'audit'] log level or null if not set.
       */
      this.getLogLevel = function () {
        if (playerInfo.hasOwnProperty('options')) {
          if (playerInfo.options.hasOwnProperty('logLevel')) {
            return playerInfo.options.logLevel;
          } else {
            return null;
          }
        }

        return null;
      };
      // </editor-fold>

    }();
    // </editor-fold>

    /* Development functions */
    // <editor-fold defaultstate="collapsed" desc="Development functions">

    /**
     * Set debug mode (if cordova system (webOS) is unavailable)
     * @param {Boolean} debug True or False.
     * @returns None
     * @version 1.8.1
     * @since 1.8.1
     * @depricated Not used in Tizen
     */
    this.setDebugMode = function (debug) {
      v_debug = debug;
    };
    // </editor-fold>

    /**
     * Provide access to the filesystem
     * @returns {DeviceManager.FileSystem}
     * @version 1.8.1
     * @since 1.8.1
     */
    this.FileSystem = new function _FileSystem() {

      /**
       * The document root.
       * This points to "downloads" virtual directory.
       */
      var v_docRoot;


      /* +------------------------+ Private functions +------------------------+ */

      /* +------------------------+ Public functions +-------------------------+ */
      // <editor-fold defaultstate="collapsed" desc="Public functions">

      /**
       * Read a file.
       * @param {function} cbSuccess
       * @param {function} cbFailure
       * @param {object} options {file, returnType [string, byteArray, base64]}
       * @returns {undefined}
       * @version 1.8.1
       * @since 1.8.1
       */
      this.readFile = function (cbSuccess, cbFailure, options) {


        var cbStreamOpen = function (fs) {
          var ret;
          switch (options.returnType) {
            case "string":
              ret = fs.read(options.file.fileSize);
              break;
            case "byteArray":
              ret = fs.readBytes(options.file.fileSize);
              break;
            case "base64":
              ret = fs.readBase64(options.file.fileSize);
              break;
            default:
              break;
          }
          fs.close();

          if (cbSuccess) {
            cbSuccess(ret);
          }
        };


        options.file.openStream(
          "r",
          cbStreamOpen,
          cbFailure,
          "UTF-8"
        );


      };

      this.readUSBFile = function (cbSuccess, cbFailure, options) {
        var path = "file:///opt/media/USBDriveA1/";

        tizen.filesystem.resolve(path,
          function (dir) {
            try {
              console.log("Mount point Name is " + dir.path);
              var file;
              if (dir) {
                file = dir.resolve(options.path);
              }

              if (file) {
                file.openStream(
                  'r',
                  function (fileStream) {
                    fileStream.position = 0;
                    var contents = fileStream.read(fileStream.bytesAvailable);
                    fileStream.close();
                    cbSuccess(contents);
                  },
                  function (error) {
                    console.log(JSON.stringify(error));
                    cbFailure(error);
                  }
                );
              }
            } catch (error) {
              cbFailure(error);
            }
          },
          function (e) {
            console.log("Error: " + e.message);
            cbFailure(e);
          }
          , "r");
      };

      /**
       * Save a file into filesystem.
       * @param {type} cbSuccess
       * @param {type} cbFailure
       * @param {Object} options
       * @see options.path
       * @see options.size
       * @see options.data
       * @see options.type
       * @returns {undefined}
       * @version 1.8.1
       * @since 1.8.1
       */
      this.saveFile = function (cbSuccess, cbFailure, options) {


        var cbSave = function () {
          var newfile = v_docRoot.createFile(options.path);
          newfile.openStream(
            "w",
            function (fs) {
              switch (options.type) {
                case "byteArray":
                  fs.writeBytes(options.data);
                  break;
                case "base64":
                  fs.writeBase64(options.data);
                  break;
                default:
                  fs.write(options.data);
                  break;
              }
              fs.close();
              if (cbSuccess) {
                cbSuccess();
              }
            }, function (e) {
              console.log("Error " + e.message);
              if (cbFailure) {
                cbFailure(e);
              }
            }, "UTF-8"
          );
        };

        DeviceManager.FileSystem.deleteFile(cbSave, cbSave, options.path);

      };

      /**
       * Append data to a file.
       * @param {function} cbSuccess Function to call when data is written to the disk.
       * @param {function} cbFailure Function to call when fail to write data to the disk.
       * @param {Object} options Object with properties about filename, size, data etc.
       * @sa {Array of Integer} options.data Array of data.
       * @sa {String} options.path Name of the file.
       * @sa {Integer} options.length Length of the data.
       * @sa {String} options.writeType Type of data to write ["string", "byteArray, "base64"]. Default is "string".
       * @returns {undefined} undefined
       * @see If the file does not exist it will create it.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.appendFile = function (cbSuccess, cbFailure, options) {

        if (!defined(options.writeType)) {
          options.writeType = "string";
        }

        var cbStreamOpen = function (fs) {
          var ret;
          switch (options.writeType) {
            case "string":
              ret = fs.write(options.data);
              break;
            case "byteArray":
              ret = fs.writeBytes(options.data);
              break;
            case "base64":
              ret = fs.writeBase64(options.data);
              break;
            default:
              break;
          }
          fs.close();

          if (cbSuccess) {
            cbSuccess(ret);
          }
        };

        var openStream = function (file) {
          file.openStream(
            "a",
            cbStreamOpen,
            cbFailure,
            "UTF-8"
          );
        };

        DeviceManager.FileSystem.findFile(
          function (f) {
            openStream(f);
          },
          function (e) {
            if (e.code === 8) { //"NotFoundError"
              var newfile = v_docRoot.createFile(options.path);
              openStream(newfile);
            }
          },
          options.path,
          "a"
        );
      };

      /**
       * Check if a file exists.
       * @param {function} cbSuccess Function to call when file exists or not.
       * @param {function} cbFailure Function to call when fail to check if file exists.
       * @param {Object} o Object with properties about the URI.
       * @sa {String} o.path The URI of the target file/directory to check.
       * @returns {undefined} undefined
       * @version 1.8.1
       * @since 1.8.1
       */
      this.fileExists = function (cbSuccess, cbFailure, o) {

        if (o.path === undefined) {
          cbFailure({errorCode: "BAD_PARAMETER", errorText: "Parameter path is undefined"});
          return;
        }

        DeviceManager.FileSystem.findFile(
          function (dir) {
            if (cbSuccess) {
              dir.exists = true;
              cbSuccess(dir);
            }

          },
          function (e) {
            if (e.code === 8) {
              cbSuccess({exists: false});
            } else {
              cbFailure(e);
            }
          },
          o.path);
      };

      /**
       * Get the filesize of a file.
       * @param {type} cbSuccess
       * @param {type} cbFailure
       * @param {type} o Options. Needs path.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.fileSize = function (cbSuccess, cbFailure, file) {
        DeviceManager.FileSystem.findFile(
          function (cbObject) {
            cbSuccess({size: cbObject.fileSize});
          }, cbFailure, file);
      };

      /**
       * Delete a file from filesystem.
       * If filename is a .htz directory should be removed also.
       * @param {function} cbSuccess
       * @param {function} cbFailure
       * @param {string} filename The name of the file.
       * @returns None
       * @version 2.0.200
       * @since 1.8.1
       */
      this.deleteFile = function (cbSuccess, cbFailure, filename) {
        var option = v_docRoot.fullPath + "/" + filename;

        var CBOnSuccess = cbSuccess;
        if (option.endsWith(".htz")) {
          CBOnSuccess = function () {
            v_docRoot.deleteDirectory(
              option.substr(0, option.length - 4),
              true,
              cbSuccess,
              cbFailure);
          }
        }

        v_docRoot.deleteFile(option, CBOnSuccess, cbFailure);

//				/* --- WEBOS CODE --- */
//				
//	            var storage = new Storage();
//
//	            // Delete a directory and its contents
//	            
//	            // Remove directory if an htz file.
//	            function success(){
//	              if (options.path.endsWith(".htz")){
//	                var opts = {
//	                    file: 'downloads/'+options.path.substr(0,options.path.length-4),
//	                    recursive: true
//	                };
//	                storage.removeFile(cbSuccess, cbFailure, opts);
//	              } 
//	            }
//	            
//	            
//	            (new Storage).removeFile(success, cbFailure, fileOps);
//				/* --- WEBOS CODE --- */


      };

      /**
       * List all files in monitor's root folder.
       * @param {Function} cbSuccess Callback function when listing complete. Pass parameter object with files.
       * @param {Function} cbFailure Callback function when failure.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.listFiles = function (cbSuccess, cbFailure) {
//				var fileOps = {
//						path : "file://internal/"
//				};
//
//				if (options){
//					if (options.hasOwnProperty("path"))
//					{
//						fileOps.path += options.path;
//					}
//				}

        var SuccessCb = function (cbObject) {
          //                var files = cbObject.files;
          //                for(var i = 0 ; i < files.length; ++i){
          //                    var fileInfo = files[i];
          //                    console.debug("File Name: " + fileInfo.name + ", Type: " + fileInfo.type+ ", Size: " + fileInfo.size);
          //                }

          if (cbSuccess) {
            cbSuccess(cbObject);
          }
        };
        //(new Storage).listFiles(SuccessCb, cbFailure, fileOps);
        v_docRoot.listFiles(SuccessCb, cbFailure);
      };

      /*
			 * Empty the root directory
			 * @version 1.8.1
			 * @since 1.8.1
			 */
      //jshint unused:true
      this.emptyDirectory = function (cbSuccess, cbFailure) {
        //jshint unused:false
        if (localStorage.emptyInternalMemory === "true") {

          var onSuccessDelete = function () {
            console.log("File/Dir Deleted");
          };

          var onFailDelete = function (e) {
            console.log("File/Dir delete error", e);
          };

          var onsuccess = function (files) {
            for (var i = 0; i < files.length; i++) {
              if (!files[i].isDirectory) {
                v_docRoot.deleteFile(
                  files[i].fullPath,
                  onSuccessDelete,
                  onFailDelete
                );
              } else {
                v_docRoot.deleteDirectory(
                  files[i].fullPath,
                  onSuccessDelete,
                  onFailDelete
                );
              }
            }
            if (cbSuccess) {
              cbSuccess();
            }
          };

          var onerror = function (error) {
            console.log("The error " + error.message + " occurred when listing the files in the selected folder");
          };

          v_docRoot.listFiles(onsuccess, onerror);
        } else {
          if (cbSuccess) {
            cbSuccess();
          }
        }

//				var self = this;
//				var flSuccessCb = function(cbSuccess, cbFailure, fileArray){
//					if (fileArray.length > 0){
//						if (fileArray[0].type === "file")
//						{
//							var delOpt = {
//									path : fileArray[0].name
//							};
//							self.deleteFile(function(){
//								fileArray.splice(0,1);
//
//								if (fileArray.length>0){
//									flSuccessCb(cbSuccess, cbFailure, fileArray);
//								}else{
//									localStorage.emptyInternalMemory = "false";
//									cbSuccess();
//								}
//							}, cbFailure,delOpt);
//						}else{
//							fileArray.splice(0,1);
//							if (fileArray.length>0){
//								flSuccessCb(cbSuccess, cbFailure, fileArray);
//							}else{
//								localStorage.emptyInternalMemory = "false";
//								cbSuccess();
//							}
//						}
//					}else{
//						localStorage.emptyInternalMemory = "false";
//						cbSuccess();
//					}
//
//				};
//
//				if (localStorage.emptyInternalMemory === "true"){
//
//					var rmoptions = {
//							device: 'internal' // This will remove all the files in the local storage.
//					};
//					var storage = new Storage();
//					storage.removeAll(function(cbObject){
//						localStorage.emptyInternalMemory = "false";
//						cbSuccess(cbObject);
//					}, cbFailure, rmoptions);
//
//					//                this.listFiles(function (cbObject){
//					//                    var files = cbObject.files;
//					//                    flSuccessCb(cbSuccess, cbFailure, files);
//					//                }, cbFailure);
//
//				}else{
//					cbSuccess();
//				}
      };

      /*
			 * 
			 * @param {type} cbSuccess
			 * @param {type} cbFailure
			 * @returns {undefined}
			 * @version 1.8.1
			 * @since 1.8.1
			 */
      this.storageInfo = function (cbSuccess, cbFailure) {

        var successCb = function (cbObject) {
          var st_total;
          var st_free;
          var st_used;
          var b_found = false;
          for (var i = 0; i < cbObject.units.length; i++) {
            if (cbObject.units[i].type === "INTERNAL") {
              b_found = true;
              st_total = cbObject.units[i].capacity;
              st_free = cbObject.units[i].availableCapacity;
              st_used = st_total - st_free;
              break;
            }
          }
          if (b_found) {
            DeviceManager.Device.getRealtimeDeviceInformation().storage_free = st_free;
            DeviceManager.Device.getRealtimeDeviceInformation().storage_total = st_total;
            DeviceManager.Device.getRealtimeDeviceInformation().storage_used = st_used;

            application.statusPage.spaceTotal(st_total);
            application.statusPage.spaceFree(st_free);
            application.statusPage.spaceUsed(st_used);
          }
          if (cbSuccess) {
            cbSuccess(cbObject);
          }
        };

        try {
          tizen.systeminfo.getPropertyValue("STORAGE", successCb, cbFailure);
        } catch (e) {
          if (cbFailure) {
            cbFailure(e);
          }
        }
      };

      /**
       * Download a file into the internal storage.
       * @param {type} cbSuccess
       * @param {type} cbFailure
       * @param {Object} options Options for the file to download. See at the bottom for extra arguments
       * @see {String} options.source The source to download from.
       * @see {String} options.destination The location to save.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.downloadFile = function (cbSuccess, cbFailure, options) {
        var listener = {
          onprogress: function (id, receivedSize, totalSize) {
            var percentage = Math.round((((receivedSize / totalSize) * 100 + Number.EPSILON) * 10)) / 10;
            percentage = percentage === 100 ? '' : percentage;
            if (!percentage) {
              application.statusPage.downloadFileId('');
            }
            application.statusPage.downloadStatus(percentage);
            application.statusPage.downloadStatus.notifySubscribers();
            //console.log('Received with id: ' + id + ', ' + receivedSize + '/' + totalSize + ' Percentage: '+percentage);
          },
//				      onpaused: function(id) {
//				          console.log('Paused with id: ' + id);
//				      },
//				      oncanceled: function(id) {
//				          console.log('Canceled with id: ' + id);
//				      },
          //jshint unused:true
          oncompleted: function (id, fileName) {
            //jshint unused:false
            //console.log('Completed with id: ' + id + ', file name: ' + fileName);
            //"This is a string".substr(0,"This is a string".length-5)

            var dlfn = fileName.split('/').pop();
            if (dlfn === options.destination) {
              if (cbSuccess) {
                cbSuccess();
              }
            } else {
              var src = v_docRoot.fullPath + "/" + dlfn;
              var dst = v_docRoot.fullPath + "/" + options.destination;
              v_docRoot.moveTo(
                src,
                dst,
                true,
                function () {
                  if (cbSuccess) {
                    cbSuccess();
                  }
                },
                function (e) {
                  if (cbFailure) {
                    cbFailure(e);
                  }
                }
              );
            }
          },
          onfailed: function (id, error) {
            //console.log('Failed for '+options.destination+' with id: ' + id + ', error: ', error);
            Logger.cmsLog({
              category: "audit",
              message: options.destination + " not found (error: " + JSON.stringify(error) + ").",
              method: "downloadFile",
              thread: "DeviceManager.FileSystem"
            });
            if (cbFailure) {
              cbFailure(error);
            }
          }
        };

        // Start downloading the html file on the web with the corresponding callbacks.
        /* [Constructor(
				   * 	DOMString url, 
				   * 	optional DOMString? destination, 
				   * 	optional DOMString? fileName, 
				   * 	optional DownloadNetworkType? networkType, 
				   * 	optional DownloadHTTPHeaderFields? httpHeader
				   * )] */
        //console.log("Download from "+options.source+" to " + options.destination);
        var downloadRequest = new tizen.DownloadRequest(options.source, v_docRoot.fullPath, options.destination);
        var downloadId;
        try {
          downloadId = tizen.download.start(downloadRequest);
          tizen.download.setListener(downloadId, listener);
        } catch (e) {
          throw e;
//					  if (cbFailure){
//        				  cbFailure(e);
//        			  }
        }
      };

      /**
       * Delete specific files from filesystem.
       * @param {type} cbSuccess Callback function called when all files deleted.
       * @param {type} cbFailure Is never called.
       * @returns None
       * @see Depends on localStorage.removeMediaFiles setting. The format is [ "file1.ext", "file2.ext", ... ]
       * @version 1.8.1
       * @since 1.8.1
       */
      this.deleteFiles = function (cbSuccess, cbFailure) {
        var fls;
        if (localStorage.hasOwnProperty("removeMediaFiles")) {
          try {
            fls = JSON.parse(localStorage.removeMediaFiles);
          } catch (e) {
            console.error("[DeleteFiles]", e);
            if (cbSuccess) {
              cbSuccess();
            }
            return;
          }
          if (Array.isArray(fls)) {
            if (fls.length === 0) {
              cbSuccess();
              return;

            }
          } else {
            cbSuccess();
            return;
          }
        } else {
          cbSuccess();
          return;
        }


        var delFile = function (cbSuccess, cbFailure, filenames) {
          if (filenames.length > 0) {
            var option = v_docRoot.fullPath + "/" + filenames[0];
            v_docRoot.deleteFile(
              option,
              function () {
                filenames.splice(0, 1);
                delFile(cbSuccess, cbFailure, filenames);

              },
              function () {
                filenames.splice(0, 1);
                delFile(cbSuccess, cbFailure, filenames);
              }
            );
          } else {
            localStorage.removeMediaFiles = JSON.stringify([]);
            cbSuccess();
          }
        };

        delFile(cbSuccess, cbFailure, fls);
      };

//			/**
//			 * Calculate the md5 checksum of a file.
//			 * @param {Function} cbSuccess Callback function to return the calculated md5 checksum.
//			 * @param {Function} cbFailure Callback function in case of a failure.
//			 * @param {String} filename Tha name of the file.
//			 * @returns None
//			 * @version 1.8.1
//			 * @since 1.8.1
//			 */
//			this.calcMD5 = function (cbSuccess, cbFailure, filename) {
//
//				function getFileSize(URL, cb) {
//					var successCb = function (cbObject) {
//						cb(cbObject.size);
//					};
//					var failureCb = function (cbObject) {
//						cbFailure(cbObject);
//					};
//					var options = {path: URL};
//					new Storage().statFile(successCb, failureCb, options);
//				}
//
//				var totalRead = 0;
//				var spark = new SparkMD5.ArrayBuffer();
//
//				function readFile(URL, totalFileSize, cb) {
//					var successCb = function (cbObject) {
//						if (cbObject.data.byteLength === 0){
//							cb(spark.end());
//						} else {
//							var binary_data = cbObject.data;    // ArrayBuffer
//							spark.append(binary_data);
//
//							//                        var tempUInt8Array = new Uint8Array(binary_data);
//							//                        for (var i = 0; i < tempUInt8Array.length; i++)
//								//                            readData.push(tempUInt8Array[i]);
//							totalRead += cbObject.data.byteLength;
//							readFile(URL, totalFileSize, cb);
//						}
//					};
//					var failureCb = function (cbObject) {
//						cbFailure(cbObject);
//					};
//					var options = {
//							path: URL,
//							position: totalRead,
//							encoding: 'binary'
//					};
//					if (totalRead >= totalFileSize) {
//						cb(spark.end());
//					} else {
//						new Storage().readFile(successCb, failureCb, options);
//					}
//				}
//
//				getFileSize("file://internal/" + filename, function (fileSize) {
//					readFile("file://internal/" +filename, fileSize, function (md5) {
//						cbSuccess(md5); // Print binary array
//					});
//				});
//
//
//			};

      /**
       * Get a reference to a File object from name.
       * @param {Function} cbSuccess Callback function to return the found file.
       * @param {Function} cbFailure Callback function in case of a failure or if file does not exist.
       * @param {String} strPath Tha name of the file.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.findFile = function (cbSuccess, cbFailure, strPath, mode) {
        //If it contains a / char then it is an absolute file else it is a relative to downloads.
        var validModes = ['r', 'rw', 'a', 'w'];

        if (defined(mode)) {
          if (validModes.indexOf(mode) === -1) {
            mode = 'rw';
          }
        } else {
          mode = 'rw';
        }

        if (strPath.indexOf('/') === -1) {
          strPath = "downloads/" + strPath;
        }
        try {
          tizen.filesystem.resolve(
            strPath,
            function (dir) {
              if (cbSuccess) {
                cbSuccess(dir);
              }
            }, function (e) {
              if (cbFailure) {
                cbFailure(e);
              }
            },
            mode
          );
        } catch (e) {
          if (cbFailure) {
            cbFailure(e);
          }
        }

      };


      /**
       * Unzip a compressed file
       * @param {function} cbSuccess Callback function when file successully unzip.
       * @param {function} cbFailure Callback function in case of a failure.
       * @param {object} options Object that contains information to unzip file.
       * @param {string} options.from Filename to uncompress.
       * @param {string} options.to Path to uncompress the file.
       * @returns {undefined}
       * @version 2.0.0
       * @since 2.0.0
       */
      this.unzipFile = function (cbSuccess, cbFailure, options) {

//	          var storage = new Storage();
//	          function makeDir(success, to){
//	            var mkdopts = {};
//	            mkdopts.path = to;
//	            storage.mkdir(success, failureCb, mkdopts);
//	          };
//	          
//	          var successCb = function () {
//	              if (cbSuccess){
//	                cbSuccess();
//	              }
//	          };
//
//	          var failureCb = function (cbObject) {
//	              console.log(" Error Code [" + cbObject.errorCode + "]: " + cbObject.errorText);
//	              if (cbFailure){
//	                cbFailure();
//	              }
//	          };
//
//	          makeDir(function(){
//	            storage.unzipFile(successCb, failureCb, opts);
//	          },opts.targetPath);

        /* The Tizen way */

        if (!options.hasOwnProperty('from')) {
          if (cbFailure) {
            cbFailure(new ERRORS.ParameterNotDefined("Parameter 'from' not defined"));
          }
        }
        if (!options.hasOwnProperty('to')) {
          if (cbFailure) {
            cbFailure(new ERRORS.ParameterNotDefined("Parameter 'to' not defined"));
          }
        }

        var opts = {
          zipPath: 'downloads/' + options.from,
          targetPath: 'downloads/' + options.to
        };

        //Check if target directory exists and created if not
        try {
          v_docRoot.createDirectory(options.to)
        } catch (e) {
          console.log('Create directory failed but this is normal');
        }

        function errorCallback(error) {
          console.log(error);
          console.log(" Error Code [" + error.errorCode + "]: " + error.errorText);
          if (cbFailure) {
            cbFailure(error);
          }
        }

        function successCallback() {
          console.log("done");
          if (cbSuccess) {
            cbSuccess();
          }
        }

        function progressCallback(opId, val, name) {
          console.log("extracting operation (: " + opId + ") is in progress (" + (val * 100).toFixed(1) + "%)");
        }

        function openSuccess(archive) {
          archive.extractAll(opts.targetPath, successCallback, errorCallback, progressCallback, true);
        }

        tizen.archive.open(opts.zipPath, "r", openSuccess, errorCallback);


      };


      //Prepare directory
      function prepareDirStructure() {
        try {
          tizen.filesystem.resolve(
            'downloads',
            function (dir) {
              v_docRoot = dir;
            }, function (e) {
              console.log("Error" + e.message);
              throw e;
            }, "rw"
          );
        } catch (e) {
          throw e;
        }

      }

      prepareDirStructure();

    }();

    /* +-----------------------------+ Device +-----------------------------+ */
    /**
     * Provide device specific functionality.
     * @returns {DeviceManager.Device}
     * @version 1.8.1
     * @since 1.8.1
     */
    this.Device = new function _Device() {

      /* +---------------------+ Private variables +----------------------+ */

//			var v_deviceSensors = { 
//			"SM5C" : {
//			sensors : { fan: false, lamp: false, screen: false, signal: false, temperature: true }
//			}
//			};

      var v_deviceName = "SSSP5";
      var v_timeZone = "";
      var v_orientation = null;

      var v_clientType = "sssp";
      var v_clientVersion = AppVersionInformation.clientVersion;
      var v_clientCode = AppVersionInformation.clientCode;

      var v_operatingSystem = "tizen";


      /**
       * Holds general realtime device information about the monitor.
       */
      var v_deviceRTInformation = {};

      /**
       * Holds static information about the monitor and the application.
       */
      var v_platformInformation = {
        clientType: v_clientType,
        clientVersion: v_clientVersion,
        clientCode: v_clientCode,
        operatingSystem: v_operatingSystem
      };

      /**
       * Queries for device Capabilities.
       * Remember to add 'http://tizen.org/'
       */
      var deviceCapabilities = [
        "feature/account",
        "feature/archive",
        "feature/badge",
        "feature/bookmark",
        "feature/calendar",
        "feature/contact",
        "feature/datacontrol",
        "feature/datasync",
        "feature/download",
        "feature/network.telephony",
        "feature/network.wifi",
        "feature/systemsetting",
        "feature/systemsetting.home_screen",
        "feature/systemsetting.lock_screen",
        "feature/systemsetting.incoming_call",
        "feature/systemsetting.notification_email",
        "feature/websetting",
        "feature/tv.audio",
        "feature/tv.tuner",
        "feature/tv.display",
        "feature/tv.inputdevice",
        "feature/tv.pip",
        "feature/network.bluetooth.audio.media",
        "feature/email",
        "feature/network.telephony",
        "feature/network.telephony.mms",
        "feature/notification",
        "feature/battery",
        "feature/camera",
        "feature/camera.back",
        "feature/camera.back.flash",
        "feature/camera.front",
        "feature/camera.front.flash",
        "feature/database.encryption",
        "feature/fmradio",
        "feature/humanactivitymonitor",
        "feature/graphics.acceleration",
        "feature/led",
        "feature/input.keyboard",
        "feature/input.keyboard.layout",
        "feature/location",
        "feature/location.batch",
        "feature/location.gps",
        "feature/location.wps",
        "feature/multimedia.transcoder",
        "feature/microphone",
        "feature/multi_point_touch.pinch_zoom",
        "feature/multi_point_touch.point_count",
        "feature/network.bluetooth.always_on",
        "feature/network.bluetooth",
        "feature/network.bluetooth.audio.call",
        "feature/network.bluetooth.audio.media",
        "feature/network.bluetooth.health",
        "feature/network.bluetooth.hid",
        "capability/network.bluetooth.le",
        "capability/network.bluetooth.oop",
        "feature/network.nfc",
        "feature/network.nfc.card_emulation",
        "feature/network.nfc.reserved_push",
        "feature/network.push",
        "feature/network.secure_element",
        "feature/network.telephony",
        "feature/network.telephony.mms",
        "feature/network.wifi",
        "feature/network.wifi.direct",
        "feature/opengles",
        "feature/opengles.version.1_1",
        "feature/opengles.version.2_0",
        "feature/platform.core.api.version",
        "feature/platform.core.cpu.arch",
        "feature/platform.core.fpu.arch",
        "feature/platform.native.api.version",
        "feature/platform.native.osp_compatible",
        "feature/platform.version",
        "feature/platform.web.api.version",
        "feature/platform.version.name",
        "feature/profile",
        "feature/screen",
        "feature/screen.auto_rotation",
        "feature/screen.height",
        "feature/screen.output.hdmi",
        "feature/screen.output.rca",
        "feature/screen.size.all",
        "feature/screen.size.large",
        "feature/screen.size.normal",
        "feature/screen.size.normal.320.320",
        "feature/screen.size.normal.480.800",
        "feature/screen.size.normal.720.1280",
        "feature/screen.width",
        "feature/sensor.accelerometer",
        "feature/sensor.accelerometer.wakeup",
        "feature/sensor.activity_recognition",
        "feature/sensor.barometer",
        "feature/sensor.barometer.wakeup",
        "feature/sensor.gesture_recognition",
        "feature/sensor.gravity",
        "feature/sensor.gyroscope",
        "feature/sensor.gyroscope.wakeup",
        "feature/sensor.heart_rate_monitor",
        "feature/sensor.humidity",
        "feature/sensor.linear_acceleration",
        "feature/sensor.magnetometer",
        "feature/sensor.magnetometer.wakeup",
        "feature/sensor.pedometer",
        "feature/sensor.photometer",
        "feature/sensor.photometer.wakeup",
        "feature/sensor.proximity",
        "feature/sensor.proximity.wakeup",
        "feature/sensor.rotation_vector",
        "feature/sensor.temperature",
        "feature/sensor.tiltmeter",
        "feature/sensor.tiltmeter.wakeup",
        "feature/sensor.ultraviolet",
        "feature/sensor.wrist_up",
        "feature/shell.appwidget",
        "feature/sip.voip",
        "feature/speech.recognition ",
        "feature/speech.synthesis",
        "feature/usb.accessory ",
        "feature/usb.host",
        "feature/web.service",
        "feature/vision.face_recognition",
        "feature/vision.image_recognition",
        "feature/vision.qrcode_generation",
        "feature/vision.qrcode_recognition",
        "feature/build.date",
        "feature/build.string",
        "feature/build.time",
        "feature/manufacturer",
        "system/tizenid",
        "system/model_name",
        "system/platform.communication_processor",
        "system/platform.name",
        "system/platform.processor"
      ];


      /* +---------------------+ Private functions +----------------------+ */

      /**
       * Get the device name!
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @version 1.8.1
       * @since 1.8.1
       */
        //jshint -W098
      var int_getDeviceName = function (cbSuccess, cbFailure) {
          //jshint +W098
          //Not available in Tizen
          if (cbSuccess) {
            cbSuccess();
          }
        };

      //jshint -W098
      var int_getTimeZone = function (cbSuccess, cbFailure) {
        //jshint +W098
        try {
          v_timeZone = tizen.time.getLocalTimezone();
          if (cbSuccess) {
            cbSuccess();
          }
        } catch (e) {
          if (cbFailure) {
            cbFailure(e);
          }
        }
      };

      var int_getOrientation = function (cbSuccess, cbFailure) {
        try {
          //var orient = b2bapis.b2bcontrol.getSourceOrientation("TV");
          var orient = b2bapis.b2bcontrol.getMenuOrientation();
          console.debug('Native portrait settings : ' + orient);
          switch (orient) {
            case "DEGREE_0":
            case "LANDSCAPE":
              //v_orientation="0";
              v_orientation = DeviceManager.Device.SCREENORIENTATION_LANDSCAPE;
              application.statusPage.screenSize("1920 x 1080");
              break;
            case "DEGREE_90":
            case "PORTRAIT":
              //v_orientation="1";
              v_orientation = DeviceManager.Device.SCREENORIENTATION_PORTRAIT;
              application.statusPage.screenSize("1080 x 1920");
              break;
            case "DEGREE_180":
              //v_orientation="8";
              v_orientation = DeviceManager.Device.SCREENORIENTATION_REVERSE_LANDSCAPE;
              application.statusPage.screenSize("1920 x 1080");
              break;
            case "DEGREE_270":
              //v_orientation="9";
              v_orientation = DeviceManager.Device.SCREENORIENTATION_REVERSE_PORTRAIT;
              application.statusPage.screenSize("1080 x 1920");
              break;
            default:
              console.warn("Orientation value is not understood (" + orient + ")");
              break;
          }
          if (cbSuccess) {
            cbSuccess(orient);
          }
        } catch (e) {
          console.error('[getOrientation] ' + e.message);
          if (cbFailure) {
            cbFailure(e);
          }
        }
      };

      /**
       * Sets native portrait rotation.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @param {Number} option
       * One of the following constants:<br>
       * "DEGREE_0" - Landscape<br>
       * "DEGREE_90" - Portrait<br>
       * "DEGREE_180" - Reverse Landscape<br>
       * "DEGREE_270" - Reverse Portrait
       * @returns None
       */
      var int_setNativePortraitMode = function (cbSuccess, cbFailure, option) {
        //b2bapis.b2bcontrol.setSourceOrientation("TV",option,
        b2bapis.b2bcontrol.setMenuOrientation(option,
          function () {
            if (cbSuccess) {
              cbSuccess();
            }
          },
          function (cbError) {
            if (cbFailure) {
              cbFailure(cbError);
            }
          }
        );
      };

      var int_getSystemUsageInfo = function (cbSuccess, cbFailure) {

        var mem_total;
        var mem_free;
        var mem_used;
        try {
          mem_total = tizen.systeminfo.getTotalMemory();
          mem_free = tizen.systeminfo.getAvailableMemory();
          mem_used = mem_total - mem_free;
          application.statusPage.memoryTotal(Number(mem_total * Math.pow(10, -6)).toFixed());
          application.statusPage.memoryUsed(Number(mem_used * Math.pow(10, -6)).toFixed());
          application.statusPage.memoryFree(Number(mem_free * Math.pow(10, -6)).toFixed());
        } catch (e) {
          if (cbFailure) {
            cbFailure(e);
            return;
          }
        }
        v_deviceRTInformation.memory_Total = mem_total;
        v_deviceRTInformation.memory_Free = mem_free;
        v_deviceRTInformation.memory_Used = mem_used;


        function successCb(cbObject) {


          v_deviceRTInformation.cpuLoad = JSON.stringify(cbObject.load);

          if (cbSuccess) {
            cbSuccess(v_deviceRTInformation);
          }
        }

        try {
          tizen.systeminfo.getPropertyValue("CPU", successCb, cbFailure);
        } catch (e) {
          if (cbFailure) {
            cbFailure(e);
          }
        }

      };

      var int_getPlatformInfo = function (cbSuccess, cbFailure) {

        //v_platformInformation
        v_platformInformation.capabilities = {};
        var i, t, cat, cap, res;
        for (i = 0; i < deviceCapabilities.length; i++) {
          t = deviceCapabilities[i].split('/');
          cat = t[0];
          cap = t[1];

          try {
            res = tizen.systeminfo.getCapability('http://tizen.org/' + deviceCapabilities[i]);
            if (!v_platformInformation.capabilities[cat]) {
              v_platformInformation.capabilities[cat] = {};
            }
            v_platformInformation.capabilities[cat][cap] = res;
          } catch (error) {
            console.log("Error name : " + error.name + ", message : " + error.message + " on cap:" + deviceCapabilities[i]);
          }
        }

        v_platformInformation.firmwareVersion = webapis.productinfo.getFirmware();
        v_platformInformation.hardwareVersion = "Not Supported";
        v_platformInformation.UHDPanel = webapis.productinfo.isUdPanelSupported();
        v_platformInformation.modelName = webapis.productinfo.getRealModel();
        v_platformInformation.modelSeries = webapis.productinfo.getModel();
        v_platformInformation.sdkVersion = "Not Supported";
        v_deviceName = v_platformInformation.modelName;

        try {
          application.statusPage.tizenVersion(v_platformInformation.capabilities.feature["platform.version"]);
        } catch (e) {
          application.statusPage.tizenVersion("");
        }

        try {
          application.statusPage.deviceModel(v_platformInformation.modelName);
        } catch (e) {
          application.statusPage.deviceModel("Unavailable!");
        }

        try {
          application.statusPage.firmwareVersion(v_platformInformation.firmwareVersion);
        } catch (e) {
          application.statusPage.firmwareVersion("Unavailable!");
        }

        if (cbSuccess) {
          cbSuccess();
        }
      };

      var int_getUsageData = function (cbSuccess, cbFailure) {

        var t = b2bapis.b2bcontrol.getSystemUpTime();
        var ups = t % 60;
        t = (t - ups) / 60;
        var upm = t % 60;
        t = (t - upm) / 60;
        var uph = t;

        if (ups < 10) {
          ups = "0" + ups;
        }
        if (upm < 10) {
          upm = "0" + upm;
        }
        if (uph < 10) {
          uph = "0" + uph;
        }
        v_deviceRTInformation.uptime = uph + ":" + upm + ":" + ups;
        if (cbSuccess) {
          cbSuccess();
        }
      };

      /**
       * This is handled by the monitor on Tizen.
       * @depricated
       */
      var upgradeIpkApplication = function (cbSuccess, cbFailure) {
        //console.warn("Not yet implemented");
        if (cbSuccess) {
          cbSuccess();
        }
//				var successCb = function () {
//					cbSuccess();
//				};
//
//				var options = {
//						type: "ipk",
//						to: Storage.AppMode.LOCAL,
//						recovery: false
//				};
//
//				var storage = new Storage();
//				storage.upgradeApplication(successCb, cbFailure, options);
      };

      var setNTPServerAddress = function (cbSuccess, cbFailure) {
        var ntpAddress = "pool.ntp.org";
        var onSuccess = function (val) {
          cbSuccess("[setNTPServerAddress] success : " + ntpAddress)
        }
        var onError = function (error) {
          cbFailure(error);
        }
        b2bapis.b2bcontrol.setNTPServerAddress(ntpAddress, onSuccess, onError);
      };

      var getNTPServerAddress = function (cbSuccess, cbFailure) {
        var NTPServerAddress = null;
        try {
          NTPServerAddress = b2bapis.b2bcontrol.getNTPServerAddress();
        } catch (e) {
          console.log("[getNTPServerAddress] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
        }
        if (NTPServerAddress) {
          cbSuccess("NTP address is already set as : " + NTPServerAddress)
        } else {
          setNTPServerAddress(cbSuccess, cbFailure);
        }
      };

      var getNTPUse = function (cbSuccess, cbFailure) {
        var NTPUse = null;
        try {
          NTPUse = b2bapis.b2bcontrol.getNTPUse();
        } catch (e) {
          console.log("[getNTPUse] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
        }
        if (NTPUse && NTPUse === 'USE') {
          getNTPServerAddress(cbSuccess, cbFailure)
        } else {
          cbSuccess("NTP is set as NOT USE");
        }
      };

      this.setNTP = function (cbSuccess, cbFailure) {
        getNTPUse(cbSuccess, cbFailure);
      }

      var setServerProperty = function (cbSuccess, cbFailure, url) {

        var onSuccess = function (val) {
          if (cbSuccess) {
            cbSuccess();
          }
        };
        var onError = function (error) {
          if (cbFailure) {
            cbFailure({code: error.code, name: error.name, messag: error.message});
          }
        };

        b2bapis.b2bcontrol.setURLLauncherAddress(url, onSuccess, onError);
      };


      /*
			 * Proxy for setting monitor settings that has common signature.
			 */
      var v_proxyPictureSettings = function (setting, opts) {
        var r, c;
        try {
          c = webapis.avinfo["get" + setting]();
          if (c !== opts) {
            r = webapis.avinfo["set" + setting](opts);
            if (r === true) {
              return r;
            } else {
              return {type: "OutOfRangeError", message: "Out of range!"};
            }
          } else {
            console.log("No need to change " + setting + ".", opts);
            return true;
          }
        } catch (e) {
          return {type: e.name, message: e.message};
        }
      };

      /**
       * Sets monitor settings to the monitor.
       */
      var v_pictureSettings = {
        backlight: function (successCb, failureCb, opts) {
          var r = v_proxyPictureSettings("Backlight", opts);
          if (r === true) {
            successCb();
          } else {
            failureCb(r);
          }
        },
        contrast: function (successCb, failureCb, opts) {
          var r = v_proxyPictureSettings("Contrast", opts);
          if (r === true) {
            successCb();
          } else {
            failureCb(r);
          }
        },
        brightness: function (successCb, failureCb, opts) {
          var r = v_proxyPictureSettings("Brightness", opts);
          if (r === true) {
            successCb();
          } else {
            failureCb(r);
          }
        },
        sharpness: function (successCb, failureCb, opts) {
          var r = v_proxyPictureSettings("Sharpness", opts);
          if (r === true) {
            successCb();
          } else {
            failureCb(r);
          }
        },
        color: function (successCb, failureCb, opts) {
          var r = v_proxyPictureSettings("ColorStrength", opts);
          if (r === true) {
            successCb();
          } else {
            failureCb(r);
          }
        },
        tint: function (successCb, failureCb, opts) {
          var r = v_proxyPictureSettings("ColorTint", opts);
          if (r === true) {
            successCb();
          } else {
            failureCb(r);
          }
        },
        speakerselection: function (successCb, failureCb, opts) {
          var r = v_proxyPictureSettings("SpeakerSelection", opts);
          if (r === true) {
            successCb();
          } else {
            failureCb(r);
          }
        },
        colorspacergb: function (successCb, failureCb, opts) {
          //console.info("colorspacergb:",opts);
          for (var color in opts) {
            var hasChanged = false;
            var oc = opts[color];
            var c = webapis.avinfo.getColorSpaceCustomRGB(color);
            for (var cs in oc) {
              if (c.hasOwnProperty(cs)) {
                if (c[cs] !== oc[cs]) {
                  c[cs] = oc[cs];
                  hasChanged = true;
                }
              } else {
                failureCb({type: "InvalidSettingError", message: "Setting " + cs + " is not valid!"});
                return;
              }
            }
            var r;
            if (hasChanged) {
              try {
                r = webapis.avinfo.setColorSpaceCustomRGB(color, c.red, c.green, c.blue);
                if (r === true) {
                  console.log("ColorSpaceCustomRGB: Setting for " + color + " applied.", c);
                } else {
                  failureCb({type: "OutOfRangeError", message: "Out of range!"});
                  return;
                }
              } catch (e) {
                failureCb({type: e.name, message: e.message});
                return;
              }
            }
          }
          successCb();
        },
        whitebalance: function (successCb, failureCb, opts) {
          //console.info("whitebalance:",opts);
          var hasChanged = false;
          var c = webapis.avinfo.getWhiteBalance();
          for (var cs in opts) {
            if (c.hasOwnProperty(cs)) {
              if (c[cs] !== opts[cs]) {
                c[cs] = opts[cs];
                hasChanged = true;
              }
            } else {
              failureCb({type: "InvalidSettingError", message: "Setting " + cs + " is not valid!"});
              return;
            }
          }

          var r;
          if (hasChanged) {
            try {
              r = webapis.avinfo.setWhiteBalance(c.offset_r, c.offset_g, c.offset_b, c.gain_r, c.gain_g, c.gain_b);
              if (r === true) {
                console.log("WhiteBalance: Settings applied.", c);
              } else {
                failureCb({type: "OutOfRangeError", message: "Out of range!"});
                return;
              }
            } catch (e) {
              failureCb({type: e.name, message: e.message});
              return;
            }
          }
          successCb();
        }
      };

      var int_getPictureSettings = function (cbSuccess, cbFailure) {
        var customRGBColors = ["red", "green", "blue", "magenta", "cyan", "yellow"];
        try {

          v_deviceRTInformation.picturesettings = {};
          v_deviceRTInformation.picturesettings.backlight = webapis.avinfo.getBacklight();
          v_deviceRTInformation.picturesettings.brightness = webapis.avinfo.getBrightness();
          v_deviceRTInformation.picturesettings.color = webapis.avinfo.getColorStrength();
          v_deviceRTInformation.picturesettings.tint = webapis.avinfo.getColorTint();
          v_deviceRTInformation.picturesettings.contrast = webapis.avinfo.getContrast();
          v_deviceRTInformation.picturesettings.sharpness = webapis.avinfo.getSharpness();
          v_deviceRTInformation.picturesettings.speakerselection = webapis.avinfo.getSpeakerSelection();
          v_deviceRTInformation.picturesettings.colorspacergb = {};
          for (var i = 0; i < customRGBColors.length; i++) {
            v_deviceRTInformation.picturesettings.colorspacergb[customRGBColors[i]] = webapis.avinfo.getColorSpaceCustomRGB(customRGBColors[i]);
          }
          v_deviceRTInformation.picturesettings.whitebalance = webapis.avinfo.getWhiteBalance();

          if (cbSuccess) {
            cbSuccess();
          }
        } catch (e) {
          if (cbFailure) {
            cbFailure({type: e.name, message: e.message});
          }
        }

      };

      /* +----------------------+ Public constants +----------------------+ */
      utils.const(this, "SCREENORIENTATION_LANDSCAPE", "0");
      utils.const(this, "SCREENORIENTATION_PORTRAIT", "1");
      utils.const(this, "SCREENORIENTATION_REVERSE_LANDSCAPE", "8");
      utils.const(this, "SCREENORIENTATION_REVERSE_PORTRAIT", "9");

      // </editor-fold>

      /* +----------------------+ Public functions +----------------------+ */
      // <editor-fold defaultstate="collapsed" desc="Public functions">

      /**
       * Batch run gathering device specific information!
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.gatherDeviceInfo = function (cbSuccess, cbFailure) {
        int_getPlatformInfo(function (cbObject) {
          int_getDeviceName(function (cbObject) {
            int_getTimeZone(function (cbObject) {
              int_getOrientation(function (cbObject) {
                cbSuccess(cbObject);
              }, cbFailure);
            }, cbFailure);
          }, cbFailure);
        }, cbFailure);
      };

      /**
       * Collect realtime device information.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.collectRealTimeDeviceInfo = function (cbSuccess, cbFailure) {
        DeviceManager.FileSystem.storageInfo(function () {
          int_getSystemUsageInfo(function () {
            int_getUsageData(function () {
              int_getPictureSettings(function () {
                if (cbSuccess) {
                  cbSuccess();
                }
              }, cbFailure);
            }, cbFailure);
          }, cbFailure);
        }, cbFailure);
      };

      /**
       * Display a message on screen for a predefined time.
       * @param {Function} cbSuccess
       * @param {Function} cbFailure
       * @param {String} message Message to show on screen.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       * @depricated Not compatible with Tizen
       */
      this.showMessage = function (cbSuccess, cbFailure, message) {
        if (cbSuccess) {
          cbSuccess();
        } else if (cbFailure) {
          cbFailure();
        } else {
          Logger.cmsLog({
            category: Logger.LOGLEVEL_AUDIT,
            message: "Not compatible with Tizen. LayoutManager should handle this",
            method: "showMessage",
            thread: "DeviceManager.Device"
          });
        }
      };

      /**
       * Take a screenshot.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.screenShot = function (cbSuccess, cbFailure) {

        //This will return the realpath of the image.
        var successCallback = function (cbObject) {

          DeviceManager.FileSystem.findFile(
            function (file) {
              DeviceManager.FileSystem.readFile(
                function (data) {
                  if (cbSuccess) {
                    cbSuccess({data: data});
                  }
                },
                cbFailure,
                {file: file, returnType: "base64"}
              );
            },
            cbFailure,
            cbObject
          );
        };

        var errorCallback = function (cbObject) {
          if (cbFailure) {
            cbFailure(cbObject);
          }
        };

        b2bapis.b2bcontrol.captureScreen(successCallback, errorCallback);
      };

      /**
       * Set the picture options.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @param {object} options Object with the picture options to set.
       * @returns undefined
       * @version 1.8.11
       * @since 1.8.11
       */
      this.setPictureOptions = function (cbSuccess, cbFailure, options) {
        function failureCb(e) {
          if (cbFailure) {
            cbFailure({
              category: 'error',
              message: e.errorText,
              method: 'setPictureOptions',
              thread: 'DeviceManager.Device',
              onlyOnce: true
            });
          }
        };

        var popts = {};
        var hasOpts = false;

        for (var opt in options) {
          if (v_pictureSettings.hasOwnProperty(opt)) {
            popts[opt] = options[opt];
            hasOpts = true;
          } else {
            // Logger.cmsLog({
            //     category: 'error',
            //     message: opt+' is invalid picture setting.',
            //     method: 'setPictureOptions',
            //     thread: 'DeviceManager.Device',
            //     onlyOnce: true
            // });
            // XXX: make a suggestion for a proper error code for this case
            Logger.cmsFault(undefined, undefined,
              {
                code: ERRORS.FaultCodes.RemoteResourceFailed,
                reason: opt + ' is invalid picture setting.',
                method: "setPictureOptions",
                thread: "DeviceManager.Device",
                date: new Date(),
                expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000),
                onlyOnce: true
              }
            );
          }
        }

        var applyMonitorSetting = function () {
          var k = (Object.keys(popts));
          if (k.length > 0) {
            v_pictureSettings[k[0]](
              function () {
                delete popts[k[0]];
                applyMonitorSetting();
              },
              function (e) {
                e.setting = k[0];
                Logger.cmsLog({
                  category: 'error',
                  message: JSON.stringify(e),
                  method: 'setPictureOptions',
                  thread: 'DeviceManager.Device',
                  onlyOnce: true
                });
                delete popts[k[0]];
                applyMonitorSetting();
              },
              popts[k[0]]
            );
          }
        }

        if (hasOpts) {
          applyMonitorSetting();
          //console.log("Set picture options:", popts);
        } else {
          if (cbFailure) {
            cbFailure(ERRORS.ErrorWithSettings("No setting(s) provided or setting(s) was wrong!"));
          }
        }
      };

      /**
       * Get the device name.
       * @returns {String} The name of the device.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getDeviceName = function () {
        return v_deviceName;
      };

      /**
       * Get the device timezone setting.
       * @returns {String} The timezone setting of the device.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getTimeZone = function () {
        return v_timeZone;
      };

      /**
       * Get the monitor's orientation
       * @returns {v_orientation} The orientation of the monitor in CMS values<br>{0 - Landscape, 1 - Portrait, 8 - Reverse Landscape, 9 - Reverse Portrait}
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getOrientation = function () {
        return v_orientation;
      };

      var listenerID;
      var wifiInterval;
      var getConnectionType = function () {
        clearInterval(wifiInterval);
        console.log('-----CLEARING WIFI SIGNAL CHECK INTERVALS IF ANY --------');
        try {
          var isOnWifi = webapis.network.getActiveConnectionType() === webapis.network.NetworkActiveConnectionType.WIFI;
          console.log('---------CONNECTION STATUS IS WIFI ? :   ' + isOnWifi + '   ------------');
          if (isOnWifi) {
            application.statusPage.hasWifi('True');
            console.log('-----SETTING NEW WIFI SIGNAL CHECK INTERVAL --------');
            wifiInterval = setInterval(checkWifiSignal, 60 * 1000);
          } else {
            application.statusPage.hasWifi('False');
            application.statusPage.wifiSignal('');
          }
        } catch (e) {
          console.error("Error code: [" + e.code + "] message: " + e.message);
        }
      }


      var checkWifiSignal = function () {
        var WiFiSignalStrengthLevel = '';
        try {
          WiFiSignalStrengthLevel = webapis.network.getWiFiSignalStrengthLevel();
        } catch (e) {
          console.log("getWiFiSignalStrengthLevel exception [" + e.code + "] name: " + e.name + " message: " + e.message);
        }

        switch (WiFiSignalStrengthLevel) {
          case 1:
            WiFiSignalStrengthLevel = ' (Signal strength: Bad)';
            break;
          case 2:
            WiFiSignalStrengthLevel = ' (Signal strength: Poor)';
            break;
          case 3:
            WiFiSignalStrengthLevel = ' (Signal strength: Fair)';
            break;
          case 4:
            WiFiSignalStrengthLevel = ' (Signal strength: Good)';
            break;
          case 5:
            WiFiSignalStrengthLevel = ' (Signal strength: Excellent)';
            break;
          default:
            WiFiSignalStrengthLevel = '';
            break;
        }
        application.statusPage.wifiSignal(WiFiSignalStrengthLevel);
      }

      this.registerNetworkChangeListener = function () {
        getConnectionType();
        console.log('-----REGISTERING NETWORK CHANGE LISTENER --------');
        var onChange = function (data) {
          console.log("[NetworkStateChangedCallback] data :" + data + " changed");
          getConnectionType();
        }

        try {
          listenerID = webapis.network.addNetworkStateChangeListener(onChange);
        } catch (e) {
          console.log("addNetworkStateChangeListener exception [" + e.code + "] name: " + e.name + " message: " + e.message);
        }

        if (listenerID > -1) {
          console.log("addNetworkStateChangeListener success listener ID [" + listenerID + "] ");
        }
      }

      /**
       * Execute a reboot or poweroff command on device.
       * @param {"reboot"|"powerOff"} command Either "reboot" or "powerOff" command.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.powerCommand = function (command) {
        logic.status.setLastCommandStatus(true);
        console.debug("Command " + command + " executed successfully!");

        /* No time to run this! */
        function successCb() {
        }

        function failureCb(e) {
          logic.status.setLastCommandStatus(false);
          console.error("Error", e);
        }

        switch (command) {
          case 'reboot':
            b2bapis.b2bcontrol.rebootDevice(successCb, failureCb);
            break;
          case 'powerOff':
            b2bapis.b2bcontrol.setPowerOff(successCb, failureCb);
            break;
          default:
            Logger.cmsLog({
              category: "error",
              message: "Command " + command + " is not valid!",
              method: "powerCommand",
              thread: "DeviceManager.Device"
            });
            logic.status.setLastCommandStatus(false);
            break;
        }
      };

      /**
       * Turns on/off the display panel. This will only affect the panel, not the main power.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @param {type} command Command is either DeviceManager.Power.DISPLAY_OFF or Power.DisplayMode.DISPLAY_ON.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setDisplayMode = function (cbSuccess, cbFailure, command) {

        function successCb(cbObject) {
          cbSuccess(cbObject);
        }

        function failureCb(cbObject) {
          cbFailure(cbObject);
        }

        b2bapis.b2bcontrol.setPanelMute(
          command,
          successCb,
          failureCb
        );
      };

      /**
       * Sets native portrait rotation.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @param {"0" | "1" | "8" | "9"} option <br>
       * One of the following value:<br>
       * "0" - Landscape<br>
       * "1" - Portrait<br>
       * "8" - Reverse Landscape<br>
       * "9" - Reverse Portrait<br>
       * This setting is alligned with the CMS orientation option value.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setNativePortraitMode = function (cbSuccess, cbFailure, option) {
        var orient;
        var bdRemoveCSS;
        var bdAddCSS;
        var screenSize;
        var knownOrientation = false;

        switch (option) {
          //case "1": /* Portrait */
          case DeviceManager.Device.SCREENORIENTATION_PORTRAIT:
            //orient = "DEGREE_90";
            orient = "PORTRAIT";
            bdRemoveCSS = "body-horizontal";
            bdAddCSS = "body-vertical";
            screenSize = "1080 x 1920";
            v_orientation = "1";
            break;
          //case "8": /* Reverse Landscape */
          case DeviceManager.Device.SCREENORIENTATION_REVERSE_LANDSCAPE:
            //orient = "DEGREE_180";
            orient = "LANDSCAPE";
            bdRemoveCSS = "body-vertical";
            bdAddCSS = "body-horizontal";
            screenSize = "1920 x 1080";
            v_orientation = "8";
            break;
          //case "9": /* Reverse Portrait */
          case DeviceManager.Device.SCREENORIENTATION_REVERSE_PORTRAIT:
            //orient = "DEGREE_270";
            v_orientation = "9";
            orient = "PORTRAIT";
            bdRemoveCSS = "body-horizontal";
            bdAddCSS = "body-vertical";
            screenSize = "1080 x 1920";
            break;
          //case "0": /* Landscape */
          case DeviceManager.Device.SCREENORIENTATION_LANDSCAPE:
            knownOrientation = true;
          /* falls through */
          default:
            //orient = "DEGREE_0";
            v_orientation = "0";
            orient = "LANDSCAPE";
            bdRemoveCSS = "body-vertical";
            bdAddCSS = "body-horizontal";
            screenSize = "1920 x 1080";
            if (!knownOrientation) {
              console.warn("Unknown orientation value (" + option + ")");
            }
            //Logger.cmsLog({category:"error", message:"Unknown orientation value ("+option+")", method:"setOptions"});
            break;
        }

        var sucessCb = function () {
          LayoutManager.setLayouts([]);
          if (cbSuccess) {
            cbSuccess();
          }
        };

//				var e = document.getElementById("webOS_Body");
//				e.classList.remove(bdRemoveCSS);
//				e.classList.add(bdAddCSS);
        application.statusPage.screenSize(screenSize);
        int_setNativePortraitMode(sucessCb, cbFailure, orient);
      };

      /**
       * Get the collected realtime device information.
       * @returns {Object} Realtime device specific information.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getRealtimeDeviceInformation = function () {
        return v_deviceRTInformation;
      };

      /**
       * Register monitoring of system specific events.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @returns None
       * @see Require getPlatformInfo() have run before.
       * @version 1.8.1
       * @since 1.8.1
       * @depricated Tizen not compatible
       */
      this.registerSystemMonitor = function (cbSuccess, cbFailure) {
        cbSuccess();

      };

      /**
       * Get monitor model.
       * @returns {String} The model.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getModel = function () {
        return v_platformInformation.modelName;
      };

      /**
       * Get the platform information.
       * @returns {Object} The platform information.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getPlatformInfo = function () {
        return v_platformInformation;
      };

      /**
       * Upgrade the application.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @param {type} filename
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.upgradeApplication = function (cbSuccess, cbFailure, filename) {
        setServerProperty(function () {
          cbSuccess();
        }, cbFailure, filename);
//				setServerProperty(function(){
//					upgradeIpkApplication(
//							function(){
//								cbSuccess();
//							},
//							function(cbObject){
//								cbFailure(cbObject);
//							});
//
//				},cbFailure, filename);
      };

      /**
       * Get the server properties.
       * @param {Function} cbSuccess Function to call on success. Return fqdnAddr if available.
       * @param {Function} cbFailure Function to call on failure. Return error object.
       * @returns None
       * @version 1.8.1
       * @since 1.8.1
       */
      this.getServerProperty = function (cbSuccess, cbFailure) {

        var URLLauncherAddress = null;
        try {
          URLLauncherAddress = b2bapis.b2bcontrol.getURLLauncherAddress();
          var index = URLLauncherAddress.indexOf('playersoftware')
          if (index > 0) {
            URLLauncherAddress = URLLauncherAddress.substr(0, index)
          }

          //   var partsArray = URLLauncherAddress.split('/')
          //   var URLlnch = '';
          //   for(var i=0; i<partsArray.length; i++){
          // 	  if(partsArray[i] == 'playersoftware'){
          // 		  i=partsArray.length; // Or break;
          // 	  }else{
          // 		URLlnch=URLlnch + partsArray[i] + '/';
          // 	  }
          //   }
          //   URLLauncherAddress = URLlnch
        } catch (e) {
          if (cbFailure) {
            cbFailure({
              code: e.code,
              name: e.name,
              message: e.message
            });
          }
        }

        var obj = {};

        if (null !== URLLauncherAddress) {
          obj.fqdnAddr = URLLauncherAddress;
        }

        v_platformInformation.server = obj;
        if (cbSuccess) {
          cbSuccess(obj);
        }

//				function successCb(cbObject) {
//				//                console.debug("cbObject : " + JSON.stringify(cbObject));
//				//                console.debug("server IP : " + cbObject.serverIp);
//				//                console.debug("server Port : " + cbObject.serverPort);
//				//                console.debug("secure Connection : " + cbObject.secureConnection);
//				//                console.debug("applicationLaunchMode : " + cbObject.appLaunchMode);
//				//                console.debug("fully Qualified Domain Name Mode : " + cbObject.fqdnMode);
//				//                console.debug("fully Qualified Domain Name Address: " + cbObject.fqdnAddr);

//				v_platformInformation.server = cbObject;

//				cbSuccess(cbObject);
//				}

//				function failureCb(cbObject) {
//				cbFailure(cbObject);
//				}

//				var configuration = new Configuration();
//				configuration.getServerProperty(successCb, failureCb);  
      };

      var showMemoryUsage = function (cbSuccess, cbFailure) {

        var str = "f:";
        var mem;
        try {
          mem = tizen.systeminfo.getAvailableMemory();
        } catch (e) {
          if (cbFailure) {
            cbFailure(e);
          }
        }
        mem = mem.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ".");
        str += mem;
        application.memoryStatus.memoryFree(str);

        if (cbSuccess) {
          cbSuccess(v_deviceRTInformation);
        }
      };

      /**********************************************************************************************************/
      /**
       * Set the USB Lock.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @param {Boolean} lock Lock the USB if false else unlock.
       * @returns {undefined}
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setLockUSB = function (cbSuccess, cbFailure, lock) {
        console.error('setLockUSB is not implemented');
        //Logger.cmsLog({category:"audit", message:"setLockUSB is not supported. Please use the device menu.", onlyOnce:true});
        if (cbSuccess) {
          cbSuccess();
        }

//	            var options = {
//	                enabled: JSON.parse(lock)
//	            };
//	            
//	            var configuration = new Configuration();
//	            configuration.setUSBLock(cbSuccess, cbFailure, options);
      };

      /**
       * Set the OSD Lock.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @param {Boolean} lock Lock the OSD if true else unlock.
       * @returns {undefined}
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setLockOSD = function (cbSuccess, cbFailure, lock) {
        var val = JSON.parse(lock);
        var setting = "OFF";
        if (val === true) {
          setting = "ON";
        }
        b2bapis.b2bcontrol.setOsdMute(setting, cbSuccess, cbFailure);
      };

      /**
       * Lock the monitor keys.
       * @param {Function} cbSuccess
       * @param {Function} cbFailure
       * @param {String} lock One of "allowall", "blockall", "poweronly", true, false.<br>
       * "allowall" or false All keys will be operational.<br>
       * "blockall" or true No key will be operational.<br>
       * "poweronly" Only power key will be operational.
       * @returns {undefined}
       * @version 2.0.1
       * @since 1.8.1
       */

      this.setLockKeys = function (cbSuccess, cbFailure, lock) {

        var localSetting; //Local key
        var remoteSetting; //IR

        if (lock.hasOwnProperty('local')) {
          switch (lock.local) {
            case false:
            case "allowall":
              localSetting = "OFF";
              break;
            case true:
            case "blockall":
            case "poweronly":
              localSetting = "ON";
              break;
            default:
              cbFailure(new Error("Unknown local key lock value. Value:" + lock.local));
              return;
              break;
          }
        }

        if (lock.hasOwnProperty('remote')) {
          switch (lock.remote) {
            case false:
            case "allowall":
              remoteSetting = "OFF";
              break;
            case true:
            case "blockall":
            case "poweronly":
              remoteSetting = "ON";
              break;
            default:
              cbFailure(new Error("Unknown remote key lock value. Value:" + lock.remote));
              return;
              break;
          }
        }

        var localLock = function () {
          if (localSetting) {
            b2bapis.b2bcontrol.setButtonLock(localSetting, cbSuccess, cbFailure);
          } else {
            if (cbSuccess) {
              cbSuccess();
            }
          }
        };

        if (remoteSetting) {
          b2bapis.b2bcontrol.setIRLockOnOff(remoteSetting, localLock, cbFailure);
        } else {
          localLock();
        }
      };

      /**
       * Set the Safety Lock.
       * @param {Function} cbSuccess Function to call on success.
       * @param {Function} cbFailure Function to call on failure.
       * @param {Boolean} lock Enable safety lock if true else disable.
       * @returns {undefined}
       * @version 1.8.1
       * @since 1.8.1
       */
      this.setSafetyLock = function (cbSuccess, cbFailure, lock) {
        var val = JSON.parse(lock);
        var setting = "OFF";
        if (val === true) {
          setting = "ON";
        }
        b2bapis.b2bcontrol.setSafetyLock(setting, cbSuccess, cbFailure);
      };

      /**********************************************************************************************************/


      if (AppVersionInformation.displayMemoryUsage === true) {
        setTimeout(function () {
          setInterval(function () {
            showMemoryUsage(null, null);
          }, 2000);
        }, 5000);
      }

      /**
       * Provide device specific functionality on power.
       * @returns {DeviceManager.Device.Power}
       * @version 1.8.1
       * @since 1.8.1
       */
      this.Power = new function _Power() {
        /*
				 * On Tizen days are matched against the timer index for both the on and off timers.
				 * +------------*
				 * |Timer | Day |
				 * +------------*
				 * |    1 | MON |
				 * |    2 | TUE |
				 * |    3 | WED |
				 * |    4 | THR |
				 * |    5 | FRI |
				 * |    6 | SAT |
				 * |    7 | SUN |
				 * +------------*
				 */

        /* +-----------------------+ Constants +------------------------+ */
        utils.const(this, "DISPLAY_ON", "OFF");
        utils.const(this, "DISPLAY_OFF", "ON");
        utils.const(this, "REBOOT", "reboot");
        utils.const(this, "SHUTDOWN", "powerOff");

        /* +-------------------+ Private variables +--------------------+ */
        /*
				 *  Contains constant values that represent the days of a week. Use in display timer functions.
				 */
        var v_timerDays = {
          monday: "MON",
          tuesday: "TUE",
          wednesday: "WED",
          thursday: "THU",
          friday: "FRI",
          saturday: "SAT",
          sunday: "SUN"
        };

        /**
         * Day literals
         */
        var v_DayLiteral = ["MON", "TUE", "WED", "THU", "FRI", "SAT", "SUN"];

        /**
         * Getting day names with b2bapis.b2bcontrol.get[On/Off]TimerRepeat functions return wrong day names.
         * Use this map to change with correct names.
         */
        var v_DayBugMap = {
          'MON ': 'FRI',
          'TUE ': 'THU',
          'WED ': 'WED',
          'THU ': 'TUE',
          'FRI ': 'MON',
          'SAT ': 'SUN',
          'SUN ': 'SAT'
        };

        /**
         * List of on timers.
         */
        var v_timerList_On;

        /**
         * List of off timers.
         */
        var v_timerList_Off;

        /**
         * Show the decision logic.
         */
        var v_showLogic = true;

        /* +-------------------+ Private functions +--------------------+ */

        var splitTime = function (strTime) {
          if (Number.isFinite(strTime)) {
            strTime = strTime.toString();
          }
          var m = strTime.substr(-2);
          var h = strTime.substr(0, strTime.length - 2);
          return {hour: h, minute: m};
        };

        var getOnTimerList = function (cbSuccess, cbFailure) {
          v_timerList_On = [];
          var i, t, r, v;

          for (i = 1; i < 8; i++) {
            r = b2bapis.b2bcontrol.getOnTimerRepeat("TIMER" + i);
            v_timerList_On[i - 1] = {};
            if (r !== "") {
              t = splitTime(b2bapis.b2bcontrol.getOnTimer("TIMER" + i));
              v = b2bapis.b2bcontrol.getOnTimerVolume("TIMER" + i);
              v_timerList_On[i - 1].hour = t.hour;
              v_timerList_On[i - 1].minute = t.minute;
              v_timerList_On[i - 1].week = v_DayBugMap[r];
              v_timerList_On[i - 1].volume = v;
            }
          }
          if (v_showLogic) {
            console.log('[Timers] On timers are', v_timerList_On);
          }
          if (cbSuccess) {
            cbSuccess(v_timerList_On);
          }
        };

        var getOffTimerList = function (cbSuccess, cbFailure) {
          v_timerList_Off = [];
          var i, t, r;

          for (i = 1; i < 8; i++) {
            t = splitTime(b2bapis.b2bcontrol.getOffTimer("TIMER" + i));
            r = b2bapis.b2bcontrol.getOffTimerRepeat("TIMER" + i);
            v_timerList_Off[i - 1] = {};
            if (r !== "") {
              v_timerList_Off[i - 1].hour = t.hour;
              v_timerList_Off[i - 1].minute = t.minute;
              v_timerList_Off[i - 1].week = v_DayBugMap[r];
            }
          }
          if (v_showLogic) {
            console.log('[Timers] Off timers are', v_timerList_Off);
          }
          if (cbSuccess) {
            cbSuccess(v_timerList_Off);
          }

        };

        var int_setOnTimers = function (cbSuccess, cbFailure, timers) {
          if (timers.length === 0) {
            cbSuccess();
            return;
          }

          var i;

          for (i = 0; i < 7; i++) {
            if (timers[i].update || timers[i].update === undefined) {
              b2bapis.b2bcontrol.setOnTimerRepeat(
                "TIMER" + (i + 1),
                "",
                "TIMER_OFF",
                "",
                function () {
                },
                cbFailure
              );
            }
          }

          for (i = 0; i < 7; i++) {
            if (timers[i].update) {

              b2bapis.b2bcontrol.setOnTimerRepeat(
                'TIMER' + (i + 1),
                timers[i].hour + ":" + timers[i].minute,
                "TIMER_MANUAL",
                timers[i].week,
                function () {
                },
                cbFailure
              );
              b2bapis.b2bcontrol.setOnTimerVolume(
                'TIMER' + (i + 1),
                timers[i].vol,
                function () {
                },
                cbFailure
              );
            }
          }

          if (cbSuccess) {
            cbSuccess();
          }
        };

        var int_setOffTimers = function (cbSuccess, cbFailure, timers) {
          if (timers.length === 0) {
            cbSuccess();
            return;
          }

          var i;

          for (i = 0; i < 7; i++) {
            if (timers[i].update || timers[i].update === undefined) {
              b2bapis.b2bcontrol.setOffTimerRepeat(
                "TIMER" + (i + 1),
                "",
                "TIMER_OFF",
                "",
                function () {
                },
                cbFailure
              );
            }
          }

          for (i = 0; i < 7; i++) {
            if (timers[i].update) {
              b2bapis.b2bcontrol.setOffTimerRepeat(
                'TIMER' + (i + 1),
                timers[i].hour + ":" + timers[i].minute,
                "TIMER_MANUAL",
                timers[i].week,
                function () {
                },
                cbFailure
              );
            }
          }

          if (cbSuccess) {
            cbSuccess();
          }

        };

        /* +--------------------+ Public functions +--------------------+ */
        /**
         * Syncs internal tv clock to keep up with CMS and JS time
         */
        this.setInternalClock = function () {
          var today = new Date();
          var h = today.getHours();
          var m = today.getMinutes();
          var s = today.getSeconds();
          var y = today.getFullYear();
          var mo = today.getMonth() + 1;
          var d = today.getDate();
          var time = y + ':' + mo + ':' + d + ':' + h + ':' + m + ':' + s;
          var onSuccess = function () {
            console.log("[setCurrentTime] Sync internal tv clock: successfull");
          }
          var onError = function (error) {
            console.log("[setCurrentTime] code :" + error.code + " error name: " + error.name + " message " + error.message);
          }
          //2015= year, 12= month, 25= day, 15=hour, 56= minute, 40= second
          if (y !== 1970) {
            b2bapis.b2bcontrol.setCurrentTime(time, onSuccess, onError);
          }
        }

        /**
         * Get the server properties.
         * @param {Function} cbSuccess Function to call on success.
         * @param {Function} cbFailure Function to call on failure.
         * @param {Object} options The JSON object from the CMS.
         * @returns None
         * @version 1.8.1
         * @since 1.8.1
         */
        this.SetTimers = function (cbSuccess, cbFailure, options) {
          function successCb(cbObject) {
            console.debug("Done with timers!");
            if (cbSuccess) {
              cbSuccess(cbObject);
            }
          }

          var parseTimerTime = function (time) {
            var t = time.split(":");
            var hour = Number.parseInt(t[0]) + "";
            var minute = t[1];

            return {hour: hour, minute: minute};
          };

          var compareTimers = function () {
            var intTimers = {
              setOnTimers: [],
              setOffTimers: []
            };

            var cmsTimers;
            var i, t, item, tmrIndex;
            if (v_showLogic) {
              console.log('[Timers] Parse timers from cms!');
            }
            for (i = 0; i < 7; i++) {
              intTimers.setOnTimers[i] = {};
              intTimers.setOffTimers[i] = {};
            }
            try {
              cmsTimers = JSON.parse(options);
              /* Transform and validate */
              for (item in cmsTimers) {
                if (!v_timerDays.hasOwnProperty(item)) {
                  // Logger.cmsLog({category:"error", message: "Timer "+item+" is not a day.", type:"ERROR", method:"compareTimers", thread:"DeviceMananager.Device.Power"});
                  // XXX: make a suggestion for a proper error code for this case
                  Logger.cmsFault(undefined, undefined,
                    {
                      code: ERRORS.FaultCodes.FaultRemoteResourceFailed,
                      reason: "Timer " + item + " is not a day.",
                      method: "compareTimers",
                      thread: "DeviceManager.Device.Power",
                      date: new Date(),
                      expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000),
                      type: "ERROR"
                    }
                  );
                  return;
                }
                if (cmsTimers[item].hasOwnProperty("on")) {
                  t = parseTimerTime(cmsTimers[item].on);
                  if (t.hasOwnProperty("hour") && t.hasOwnProperty("minute")) {
                    cmsTimers[item].on = t;
                  } else {
                    //Logger.cmsLog({category:"error", message: "Timer "+item+" has invalid on time ("+cmsTimers[item].on+").", type:"ERROR", method:"compareTimers", thread:"DeviceMananager.Device.Power"});
                    Logger.cmsFault(undefined, undefined,
                      {
                        code: ERRORS.FaultCodes.FaultRemoteResourceFailed,
                        reason: "Timer " + item + " has invalid on time (" + cmsTimers[item].on + ").",
                        method: "compareTimers",
                        thread: "DeviceManager.Device.Power",
                        date: new Date(),
                        expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000),
                        type: "ERROR"
                      }
                    );
                    return;
                  }
                }

                if (cmsTimers[item].hasOwnProperty("off")) {
                  t = parseTimerTime(cmsTimers[item].off);
                  if (t.hasOwnProperty("hour") && t.hasOwnProperty("minute")) {
                    cmsTimers[item].off = t;
                  } else {
                    //Logger.cmsLog({category:"error", message: "Timer "+item+" has invalid off time ("+cmsTimers[item].off+").", type:"ERROR", method:"compareTimers", thread:"DeviceMananager.Device.Power"});
                    // XXX: make a suggestion for a proper error code for this case
                    Logger.cmsFault(undefined, undefined,
                      {
                        code: ERRORS.FaultCodes.FaultRemoteResourceFailed,
                        reason: "Timer " + item + " has invalid off time (" + cmsTimers[item].off + ").",
                        method: "compareTimers",
                        thread: "DeviceManager.Device.Power",
                        date: new Date(),
                        expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000),
                        type: "ERROR"
                      }
                    );
                    return;
                  }
                }

                if (cmsTimers[item].hasOwnProperty("vol")) {
                  cmsTimers[item].vol = Number.parseInt(cmsTimers[item].vol);
                } else {
                  cmsTimers[item].vol = tizen.tvaudiocontrol.getVolume();
                }


                if (!(cmsTimers[item].hasOwnProperty("on") || cmsTimers[item].hasOwnProperty("off"))) {
                  //Logger.cmsLog({category:"error", message: "Timer "+item+" has no time.", type:"ERROR", method:"compareTimers", thread:"DeviceMananager.Device.Power"});
                  // XXX: make a suggestion for a proper error code for this case
                  Logger.cmsFault(undefined, undefined,
                    {
                      code: ERRORS.FaultCodes.FaultRemoteResourceFailed,
                      reason: "Timer " + item + " has no time.",
                      method: "compareTimers",
                      thread: "DeviceManager.Device.Power",
                      date: new Date(),
                      expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000),
                      type: "ERROR"
                    }
                  );
                  return;
                }
              }
            } catch (e) {
              if (cbFailure) {
                cbFailure(e);
              }
              return;
            }


            /* Check for addition */
            if (v_showLogic) {
              console.log('[Timers] Check for on timers!');
            }


            for (item in cmsTimers) {
              /* On timers */
              tmrIndex = v_DayLiteral.indexOf(v_timerDays[item]);
              //Check for changes
              if (cmsTimers[item].hasOwnProperty("on")) {
                intTimers.setOnTimers[tmrIndex] = {
                  hour: cmsTimers[item].on.hour,
                  minute: cmsTimers[item].on.minute,
                  week: v_timerDays[item],
                  inputSource: "ext://hdmi:1",
                  vol: cmsTimers[item].vol
                };
              }

              if (v_timerList_On[tmrIndex].week !== intTimers.setOnTimers[tmrIndex].week ||
                v_timerList_On[tmrIndex].hour !== intTimers.setOnTimers[tmrIndex].hour ||
                v_timerList_On[tmrIndex].minute !== intTimers.setOnTimers[tmrIndex].minute) {
                if (v_showLogic) {
                  console.log('[Timers] On Timer' + tmrIndex + ' needs update!');
                }
                intTimers.setOnTimers[tmrIndex].update = true;
              } else {
                intTimers.setOnTimers[tmrIndex].update = false;
              }

              /* Off timers */
              if (cmsTimers[item].hasOwnProperty("off")) {
                intTimers.setOffTimers[tmrIndex] = {
                  hour: cmsTimers[item].off.hour,
                  minute: cmsTimers[item].off.minute,
                  week: v_timerDays[item],
                  inputSource: "ext://hdmi:1"
                };
              }
              if (v_timerList_Off[tmrIndex].week !== intTimers.setOffTimers[tmrIndex].week ||
                v_timerList_Off[tmrIndex].hour !== intTimers.setOffTimers[tmrIndex].hour ||
                v_timerList_Off[tmrIndex].minute !== intTimers.setOffTimers[tmrIndex].minute) {
                if (v_showLogic) {
                  console.log('[Timers] Off Timer' + tmrIndex + ' needs update!');
                }
                intTimers.setOffTimers[tmrIndex].update = true;
              } else {
                intTimers.setOffTimers[tmrIndex].update = false;
              }
            }

            /* Apply changes*/

            if (v_showLogic) {
              console.log('[Timers] Add on timers!');
            }
            int_setOnTimers(function () {
              if (v_showLogic) {
                console.log('[Timers] Add off timers!');
              }

              int_setOffTimers(
                function () {
                  //Gather again the timers for next comparison if something went wrong!
                  getOnTimerList(function () {
                    getOffTimerList(cbSuccess, cbFailure);
                  }, cbFailure);
                },
                cbFailure,
                intTimers.setOffTimers
              );

            }, cbFailure, intTimers.setOnTimers);
          };

          var getOffTimers = function () {
            if (!defined(v_timerList_Off)) {
              if (v_showLogic) {
                console.log('[Timers] getOffTimers');
              }
              getOffTimerList(function () {
                compareTimers();
              }, function () {
                compareTimers();
              });
            } else {
              compareTimers();
            }
          };

          var getOnTimers = function () {
            if (!defined(v_timerList_On)) {
              if (v_showLogic) {
                console.log('[Timers] getOnTimers');
              }
              getOnTimerList(function () {
                getOffTimers();
              }, function () {
                getOffTimers();
              });
            } else {
              getOffTimers();
            }
          };

          if (true) {
            getOnTimers();
          } else {
            console.warn("Setting timers is disabled due to a firmware error.");
            if (cbSuccess) {
              cbSuccess();
            }
          }
        };
      }();

      /**
       * Provide access to monitor inputs
       * @returns {DeviceManager.Device.Inputs}
       */
      this.Inputs = new function _Inputs() {

        /**
         * Plays an external input on an area of the screen
         * @param {Function} cbSuccess Function to call when external input starts plays.
         * @param {Function} cbFailure Function to call on failure.
         * @param {Object} options<br>
         * options contains <b>divId</b> and <b>sourceName</b>.<br>
         * divId - The id of the div where a new video tag named extvid_{divid} will be created.<br>
         * sourceName - One of the following {RGB, HDMI, DVI, DP, OPS} input<br>
         * @version 1.8.1
         * @since 1.8.1
         * @returns None
         */
        this.playExternalInput = function _playExternalInput(cbSuccess, cbFailure, options) {
          // CMS options are [RGB, HDMI, DVI, DP, OPS]
          function showWindow(videoSourceString) {
            try {
              var tgt = document.getElementById(options.divId);
              var rect = tgt.getClientRects()[0];
              //jshint -W016
              var recleft = rect.left | 0;
              var rectop = rect.top | 0;
              var recwidth = rect.width | 0;
              var recheight = rect.height | 0;
              //jshint +W016
              var orientation = DeviceManager.Device.getOrientation();
              var rectangle = recleft + ":" + rectop + ":" + recwidth + ":" + recheight + ":" + orientation;
              b2bapis.b2bcontrol.setNetPIP(1, videoSourceString, rectangle, cbSuccess, cbFailure); // ON
            } catch (e) {
              cbFailure(e);
            }
          }


          function systemInfoSuccessCB(videoSource) {

            var i, bFound = false;
            for (i = 0; i < videoSource.connected.length; i++) {
              if (videoSource.connected[i].type === options.sourceName &&
                videoSource.connected[i].number === 1) {
                bFound = true;
                var videoSourceString = videoSource.connected[i].type + videoSource.connected[i].number;
                showWindow(videoSourceString);
                break;
              }
            }

            if (!bFound) {
              for (i = 0; i < videoSource.disconnected.length; i++) {
                if (videoSource.disconnected[i].type === options.sourceName &&
                  videoSource.disconnected[i].number === 1) {
                  if (cbFailure) {
                    cbFailure({message: "Video source " + options.sourceName + " is not connected!"});
                    return;
                  }
                }
              }
              if (cbFailure) {
                cbFailure({message: "Video source " + options.sourceName + " is not available!"});
                return;
              }
            }
          }

          try {
            tizen.systeminfo.getPropertyValue("VIDEOSOURCE", systemInfoSuccessCB, cbFailure);
          } catch (error) {
            cbFailure(error);
          }

        };

        /**
         * Hides external input of the screen
         * @param {Function} cbSuccess Function to call when external input stop plays.
         * @param {Function} cbFailure Function to call on failure.
         * @version 1.8.1
         * @since 1.8.1
         */
        this.hideExternalInput = function _hideExternalInput(cbSuccess, cbFailure) {
          b2bapis.b2bcontrol.setNetPIP(0, "", "", cbSuccess, cbFailure); // OFF
        };
      }();
    }();

    /**
     * Handle the node.js server.
     */
    this.NodeServer = new function _NodeServer() {

      var serverPath = '../lib/starter.js';
      switch (tizen.systeminfo.getCapability('http://tizen.org/feature/platform.version')) {
        case "4.0":
          serverPath = '../lib/starter_sssp6.js';
          break;
        case "5.0":
          serverPath = '../lib/starter_sssp7.js';
          break;
        case "6.5":
          serverPath = '../lib/starter_sssp10.js';
          break;
        default:
          break;
      }

      var serverName = 'XFTServer';
      var socket;
      /**
       * Initialize Node server
       */
      this.init = function () {
        function onSuccess(r) {
          console.log("Node server started", r);
          setTimeout(function () {
            console.log("Connect to IPC!");
            socket = io('http://127.0.0.1:8888'/*,{transports: ['websocket']}*/);
            socket.on('connect', function () {
              console.log('IPC Connected!');
              //DeviceManager.NodeServer.socket = socket;
              setTimeout(function () {
                ZmqSocket.__onFlashReady(socket);
              }, 0);
              var port = localStorage.internalwsPort || 9696;
              DeviceManager.NodeServer.internalws.listen(port, function () {
                console.log('Internal ws listening!');
              });
              setTimeout(function () {
                socket.emit('ping', 'alive');
              }, 5000);
            });
            socket.on('ping', function (ret) {
              console.log('Node.js is alive!');
            });
          }, 500);
        }

        function onError(e) {
          console.warn("Node server error", e);
        }

        try {
          setTimeout(function () {
            b2bapis.b2bcontrol.startNodeServer(serverPath, serverName, onSuccess, onError);
          }, 0);
        } catch (e) {
          console.error(e);
        }
      };

      /**
       * Manager Internal web server
       * @returns {undefined}
       */
      this.internalws = new function _internalws() {
        /**
         * Start internal web server at port
         * @param {type} port The port to listen for incoming connections
         * @param {type} cb The callback function to call after start listening.
         * @returns {undefined}
         */
        this.listen = function (port, cb) {
          socket.emit('intws_listen', port, cb);
        };
        /**
         * Close the internal web server.
         * @param {function} cb
         * @returns {undefined}
         */
        this.close = function (cb) {
          socket.emit('intws_close', cb);
        };
      };

      //this.socket = null;
    }();
  }();
  //jshint +W057
}());
