/* global ERRORS, defined */
(function () {
  "use strict";

  /**
   * @classdesc Provides errors for the entire application
   * @type ERRORS
   * @version 1.8.1
   * @since 1.8.1
   */
  //jshint -W057
  window.ERRORS = new function _ERRORS() {
    /**
     * Connection error constant value (errorCode = 10001)
     */
    this.CONNECTIONERRORCODE = 10001;
    /**
     * Wrong server key constant value (errorCode = 10002)
     */
    this.WRONGSERVERKEYCODE = 10002;
    /**
     * Client not licensed constant value (errorCode = 1000).
     * Same as FaultNotLicensed
     */
    this.CLIENTNOTLICENSED = 1000;

    /**
     * Wrong image format constant value (errorCode = 3000)
     * Same as FaultImageUnknown
     */
    this.WRONGIMAGEFORMAT = 3000;

    /**
     * Error with settings constant value (errorCode = 10005)
     */
    this.ERRORWITHSETTINGS = 10005;
    /**
     * File not found constant value (errorCode = 10006)
     */
    this.FILENOTFOUND = 10006;
    /**
     * Parameter not defined constant value (errorCode = 10007)
     */
    this.PARAMETERNOTDEFINED = 10007;
    /**
     * Error in parameter (errorCode = 10008)
     */
    this.PARAMETERERROR = 10008;


    this.FaultNotLicensed = 1000;
    this.FaultMemoryRunningLow = 1001;
    this.FaultMemoryCritical = 1002;
    this.FaultPowerPointNotAvailable = 1003;
    this.FaultGeneralError = 1004;

    this.FaultVideoSource = 2001;
    this.FaultVideoUnexpected = 2099;

    this.FaultImageUnknown = 3000;
    this.FaultImageDecode = 3001;
    this.FaultImageOutOfMemory = 3002;

    this.FaultXMRUnknownCommand = 4000;
    this.FaultFileAddFailed = 4400;
    this.FaultFileDeleteFailed = 4401;
    this.FaultRemoteResourceFailed = 4404;

    this.FaultXlfNoContent = 5000;

    this.FaultSettingNotAvailable = 6000;
    this.FaultTimerInvalidDay = 6001;
    this.FaultTimerInvalidTime = 6002;
    this.FaultPicturePropertySetFailed = 6003;
    this.FaultPicturePropertyInvalidValue = 6004;
    this.FaultPicturePropertyValueOutOfRange = 6005;

    this.FaultCodes = {
      /**
       * Not licensed.
       */
      NotLicensed: this.FaultNotLicensed,
      /**
       * Memory running low.
       */
      MemoryRunningLow: this.FaultMemoryRunningLow,
      /**
       * Memory critical.
       */
      MemoryCritical: this.FaultMemoryCritical,
      /**
       * PowerPoint not available.
       */
      PowerPointNotAvailable: this.FaultPowerPointNotAvailable,
      /**
       * Video source.
       */
      VideoSource: this.FaultVideoSource,
      /**
       * Video unexpected.
       */
      VideoUnexpected: this.FaultVideoUnexpected,
      /**
       * Image unknown.
       */
      ImageUnknown: this.FaultImageUnknown,
      /**
       * Image decode.
       */
      ImageDecode: this.FaultImageDecode,
      /**
       * Image out of memory.
       */
      ImageOutOfMemory: this.FaultImageOutOfMemory,
      /**
       * Remote Resource Failed.
       */
      RemoteResourceFailed: this.FaultRemoteResourceFailed,
      /**
       * XLF no content.
       */
      XlfNoContent: this.FaultXlfNoContent,
      /**
       * General error.
       */
      GeneralError: this.FaultGeneralError,
      /**
       * Setting not available.
       */
      SettingNotAvailable: this.FaultSettingNotAvailable,
      /**
       * File add failed.
       */
      FileAddFailed: this.FaultFileAddFailed,
      /**
       * File delete failed.
       */
      FileDeleteFailed: this.FaultFileDeleteFailed,
      /**
       * Timer invalid day.
       */
      TimerInvalidDay: this.FaultTimerInvalidDay,
      /**
       * Timer invalid time.
       */
      TimerInvalidTime: this.FaultTimerInvalidTime,
      /**
       * XMR unknown command.
       */
      XMRUnknownCommand: this.FaultXMRUnknownCommand,
      /**
       * Picture property set failed.
       */
      PicturePropertySetFailed: this.FaultPicturePropertySetFailed,
      /**
       * Picture property invalid value.
       */
      PicturePropertyInvalidValue: this.FaultPicturePropertyInvalidValue,
      /**
       * Picture property value out of range.
       */
      PicturePropertyValueOutOfRange: this.FaultPicturePropertyValueOutOfRange
    };


    /* Abstract error*/
    var absError = function (defaultMessage, defaultCode, customMessage) {
      var msg = defaultMessage;
      var code = defaultCode;
      if (defined(msg)) {
        msg = customMessage;
      }

      return {errorCode: code, errorText: msg};
    };


    /**
     * Create a new error object with errorCode(=10001) and errorText (equal to <b>customMessage</b> or <b>"Problem with connection"</b>).
     * @param {String} customMessage A custom message instead of <b>"Problem with connection"</b>.
     * @returns {Object} The new error object.
     */
    this.ConnectionError = function (customMessage) {
      return new absError("Problem with connection", ERRORS.CONNECTIONERRORCODE, customMessage);
    };

    /**
     * Create a new error object with errorCode(=10002) and errorText (equal to <b>customMessage</b> or <b>"The Server key you entered does not match with the server key at this address"</b>).
     * @param {String} customMessage A custom message instead of <b>"The Server key you entered does not match with the server key at this address"</b>.
     * @returns {Object} The new error object.
     */
    this.WrongServerKey = function (customMessage) {
      return new absError("The Server key you entered does not match with the server key at this address", ERRORS.WRONGSERVERKEYCODE, customMessage);
    };

    /**
     * Create a new error object with errorCode(=10003) and errorText (equal to <b>customMessage</b> or <b>"The image format is wrong"</b>).
     * @param {String} customMessage A custom message instead of <b>"The image format is wrong"</b>.
     * @returns {Object} The new error object.
     */
    this.ClientIsNotLicensed = function (customMessage) {
      return new absError("The image format is wrong", ERRORS.CLIENTNOTLICENSED, customMessage);
    };

    /**
     * Create a new error object with errorCode(=10004) and errorText (equal to <b>customMessage</b> or <b>"The image format is wrong"</b>).
     * @param {String} customMessage A custom message instead of <b>"The image format is wrong"</b>.
     * @returns {Object} The new error object.
     */
    this.WrongImageFormat = function (customMessage) {
      return new absError("The image format is wrong", ERRORS.WRONGIMAGEFORMAT, customMessage);
    };

    /**
     * Create a new error object with errorCode(=10005) and errorText (equal to <b>customMessage</b> or <b>"Error with settings"</b>).
     * @param {String} customMessage A custom message instead of <b>"Error with settings"</b>.
     * @returns {Object} The new error object.
     */
    this.ErrorWithSettings = function (customMessage) {
      return new absError("Error with settings", ERRORS.ERRORWITHSETTINGS, customMessage);
    };

    /**
     * Create a new error object with errorCode(=10006) and errorText (equal to <b>customMessage</b> or <b>"File not found"</b>).
     * @param {String} customMessage A custom message instead of <b>"File not found"</b>.
     * @returns {Object} The new error object.
     */
    this.FileNotFound = function (customMessage) {
      return new absError("File not found", window.ERRORS.FILENOTFOUND, customMessage);
    };

    /**
     * Create a new error object with errorCode(=10007) and errorText (equal to <b>customMessage</b> or <b>"Parameter is not defined"</b>).
     * @param {String} customMessage A custom message instead of <b>"Parameter is not defined"</b>.
     * @returns {Object} The new error object.
     */
    this.ParameterNotDefined = function (customMessage) {
      return new absError("Parameter is not defined", ERRORS.PARAMETERNOTDEFINED, customMessage);
    };

    /**
     * Create a new error object with errorCode(=10008) and errorText (equal to <b>customMessage</b> or <b>"Parameter error"</b>).
     * @param {String} customMessage A custom message instead of <b>"Parameter error"</b>.
     * @returns {Object} The new error object.
     */
    this.ParameterError = function (customMessage) {
      return new absError("Parameter error", ERRORS.PARAMETERERROR, customMessage);
    };

  }();
}());
