/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -
 */
/*
 * Library Manager
 * 
 * V0.0.1
 */
/* global LibraryManager, DeviceManager, CommunicationManager, defined, ERRORS, Logger, utils */

(function () {
  "use strict";

  /**
   * @namespace LibraryManager
   * @classdesc Provide Library handling.
   * @type LibraryManager
   * @version 1.8.1
   * @since 1.8.1
   */
  //jshint -W057
  window.LibraryManager = new function _LibraryManager() {

    /**
     * Library file.
     * @param {String} id The id of the file.
     * @param {String} filename The filename of the file.
     * @param {String} size The size of the file.
     * @param {String} type One of [media, layout, resource] as type of the file.
     * @returns {LibraryManager.libFile} A new library file.
     */
    this.libFile = function (id, filename, size, type) {
      var _self = this;
      this.id = id;
      this.filename = filename;
      this.size = size;
      this.type = type;
      this.md5 = null;
      this.isCorrect = false;
      /**
       * Last time file checked. This is updated when add a new file in library and when download it.
       */
      this.lastChecked = Date.now() / 1000 | 0;
      /**
       * Manually invalidate a file.
       * This is usually used for resources that has not updated since lastChecked + (updateInterval | 5 mins).
       * @property {Boolean} LibraryManager.manuallyInvalidated Manually invalidate file.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.manuallyInvalidated = false;

      /**
       * Check if file is valid.
       * @param {Function} cbSuccess Function to call if check is success. Doesn't mean it is valid.
       * @param {Function} cbFailure Function to call if check fails.
       * @param {Object} options Object with md5, size and updated fields to check against.
       * @version 1.8.1
       * @since 1.8.1
       */
      this.validateFile = function (cbSuccess, cbFailure, options) {
        //var options.isCorrect = false;
        var fileOptions = {
          path: filename
        };

        var checkMD5Cb = function (cbObject) {
          if (cbObject === options.md5) {
            _self.isCorrect = true;
            cbSuccess({isValid: true});
          } else {
            _self.isCorrect = false;
            cbSuccess({isValid: false, reason: "Wrong MD5!"});
          }
        };

        var checkSizeCb = function (cbObject) {
          if (type === "resource") {
            if (_self.updated === options.updated) {
              _self.isCorrect = true;
              _self.times = {
                atime: new Date(cbObject.atime),
                mtime: new Date(cbObject.mtime),
                ctime: new Date(cbObject.ctime)
              };
              cbSuccess({isValid: true});
            } else {
              _self.isCorrect = false;
              cbSuccess({isValid: false, reason: "Wrong update time!"});
            }
          } else {
            if (cbObject.size === parseInt(options.size)) {
              checkMD5Cb(_self.md5);
            } else {
              /* File exists but has wrong size. Delete it here so it can be downloaded again. */
              var fileOptions = {
                path: filename
              };
              DeviceManager.FileSystem.deleteFile(function () {
                _self.isCorrect = false;
                cbSuccess({
                  isValid: false,
                  reason: "Wrong file size (expected: " + options.size + ", got: " + cbObject.size + ")!"
                });
              }, cbFailure, fileOptions.path);
            }
          }
        };

        var checkExistCb = function (cbObject) {
          if (cbObject.exists === true) {
            DeviceManager.FileSystem.fileSize.call(_self, checkSizeCb, cbFailure, fileOptions.path);
          } else {
            _self.isCorrect = false;
            cbSuccess({isValid: false, reason: "File does not exist!"});
          }
        };

        if (_self.manuallyInvalidated) {
          _self.isCorrect = false;
          cbSuccess({isValid: false, reason: "Manually invalidated!"});
        } else {
          DeviceManager.FileSystem.fileExists.call(_self, checkExistCb, cbFailure, fileOptions);
        }
      };

      this.downloader = null;


      try {
        if (defined(window.LibraryManager)) {
          if (defined(window.LibraryManager.blacklisted_widgets)) {
            if (window.LibraryManager.blacklisted_widgets.indexOf(id) > -1) {
              this.isBlacklisted = true;
            }
          }
        }
      } catch (e) {
        console.error('LibraryManager.libFile: New file error', e.message, e.stack);
      }

    };

    this.libFile.prototype.download = function (cbSuccess, cbFailure) {
      //console.debug("Start downloading file! ", _self);
      if (this.downloader) {
        this.downloader(cbSuccess, cbFailure);
      } else {
        console.warn("File has no downloader! ", this);
      }
    };

    /*
     * File should contain
     * + filename
     * + size
     * + type (from v_Downloadable_Types)
     * + isCorrect
     * + lastChecked
     */
    /* +------------------------+ Private Variables +------------------------+ */
    // <editor-fold defaultstate="collapsed" desc="Private variables">
    //var _self = this;
    var v_Downloadable_Types = ["media", "layout", "resource"];
    var v_RequiredFiles = [];
    var v_TempRequiredFiles = [];
    var v_blacklisted_widgets = undefined;
    //var v_blacklisted_layouts = undefined;
    // </editor-fold>

    /* +------------------------+ Private functions +------------------------+ */
    // <editor-fold defaultstate="collapsed" desc="Private functions">


    var f_findIndexOfFileByProperty = function (prop, match) {
      var retIndex = null;
      for (var i = 0, max = v_RequiredFiles.length; i < max; i++) {
        if (v_RequiredFiles[i].hasOwnProperty(prop)) {
          if (v_RequiredFiles[i][prop] === match) {
            retIndex = v_RequiredFiles[i];
            break;
          }
        }
      }
      return retIndex;
    };

    /**
     * @deprecated This will be replaced by checkFileValid.
     * @param {Function} cbSuccess
     * @param {Function} cbFailure
     * @param {object} options
     */
    var checkFile = function (cbSuccess, cbFailure, options) {
      options.isCorrect = false;
      var successSizeCb = function (cbObject) {
        if (cbObject.size === options.size) {
          options.isCorrect = true;
          cbSuccess(true);
        } else {
          /* File exists but has wrong size. Delete it here so it can be downloaded again. */
          var fileOptions = {
            path: options.saveAs
          };
          DeviceManager.FileSystem.deleteFile(function () {
            cbSuccess(false);
          }, cbFailure, fileOptions);
        }
      };

      var successExistsCb = function (cbObject) {
        if (cbObject.exists === true) {
          if (options.type === "resource") {
            options.isCorrect = true;
            cbSuccess(true);
          } else {
            var fileOptions = {
              path: options.saveAs
            };
            DeviceManager.FileSystem.fileSize(successSizeCb, cbFailure, fileOptions);
          }
        } else {
          cbSuccess(false);
        }
      };

      var fileOptions = {
        path: options.saveAs
      };

      DeviceManager.FileSystem.fileExists(successExistsCb, cbFailure, fileOptions);
    };

    var checkFileValid = function (cbSuccess, cbFailure, options) {

      var f = LibraryManager.getFileByName(options.filename);
      f.validateFile(cbSuccess, cbFailure, options);

    };

    var checkFileRecursive = function (cbSuccess, cbFailure, options) {
      options.v_fileIndex++;

      var checkFileSuccessCb = function (cbObject) {
        if (cbObject === false) {
          options.v_needsUpdate = true;
        }
        checkFileRecursive(cbSuccess, cbFailure, options);
      };

      if (options.v_fileIndex >= v_RequiredFiles.length) {
        /* Finish going through files */
        cbSuccess({needsUpdate: options.v_needsUpdate});

      } else {
        //if (v_RequiredFiles[options.v_fileIndex].hasOwnProperty("path")){
        if (v_RequiredFiles[options.v_fileIndex].hasOwnProperty("type")) {
          if (LibraryManager.isDownloadable(v_RequiredFiles[options.v_fileIndex])) {
            /* Add time stamp of check */
            v_RequiredFiles[options.v_fileIndex].lastChecked = utils.getTimeStamp();

            if (v_RequiredFiles[options.v_fileIndex].type === "resource") {
              v_RequiredFiles[options.v_fileIndex].path = /*"resource_" + */ v_RequiredFiles[options.v_fileIndex].id;
              v_RequiredFiles[options.v_fileIndex].saveAs = v_RequiredFiles[options.v_fileIndex].path;
            }

            checkFile(checkFileSuccessCb, cbFailure, v_RequiredFiles[options.v_fileIndex]);
          } else {
            checkFileRecursive(cbSuccess, cbFailure, options);
          }
        } else {
          checkFileRecursive(cbSuccess, cbFailure, options);
        }
      }
    };

    // </editor-fold>

    /* +------------------------+ Public functions +------------------------+ */
    // <editor-fold defaultstate="collapsed" desc="Public functions">

    this.setRequiredFiles = function (files) {
      v_TempRequiredFiles = files;
    };

    this.getRequiredFiles = function () {
      //return (JSON.parse(JSON.stringify(v_RequiredFiles))); /* Return a copy */
      //return v_RequiredFiles;
      return v_TempRequiredFiles;
    };


    this.checkFilesOK = function () {
      for (var i = 0, max = v_RequiredFiles.length; i < max; i++) {
        if (v_RequiredFiles[i].isCorrect === false) {
          return false;
        }
      }

      return true;

    };

    /**
     * Check a file if exists and has the correct size in filesystem.
     * @param {type} cbSuccess Callback function when check completes.
     * @param {type} cbFailure Callback function in case of an error.
     * @param {type} file The file to examine.
     * @returns None
     * @see It will update its isCorrect field appropriately.
     */
    this.checkFileOK = function (cbSuccess, cbFailure, file) {
      checkFileValid(cbSuccess, cbFailure, file);
    };

    /**
     * Add a new file in library.
     * @param {type} cbSuccess Callback function with a file Object checked for validity.
     * @param {type} cbFailure Callback function in case of a failure.
     * @param {Object} file File should contain filename, size and type fields.
     * @returns None
     * @see Adding a file it will automatically detect if file is in file system and if it is correct!
     */
    this.addFile = function (cbSuccess, cbFailure, file) {
      var fe, tf;
      switch (file.download) {
        case "http":
          tf = {
            id: file.id,
            filename: file.saveAs,
            size: file.size,
            type: file.type,
            md5: file.md5
          };

          if (!defined(tf.filename)) {
            switch (tf.type) {
              case "layout":
                tf.filename = file.id + ".xlf";
                break;
              default:
                throw ERRORS.ParameterNotDefined();
              //break;
            }
          }

          fe = this.getFileByName(tf.filename);
          if (fe === null) {
            fe = new LibraryManager.libFile(tf.id, tf.filename, tf.size, tf.type);
            fe.md5 = tf.md5;
            file.code && (fe.code = file.code);
            v_RequiredFiles.push(fe);

          } else {
            fe.id = tf.id;
          }
          fe.downloader = function (successCb, failureCb) {

            var dlInfo = {
              source: file.path,
              destination: tf.filename
            };
            DeviceManager.FileSystem.downloadFile(function () {
              fe.isCorrect = true;
              fe.md5 = tf.md5;
              fe.size = tf.size;
              fe.validateFile(function (cbobject) {
                fe.lastChecked = utils.getTimeStamp();
                if (cbobject.hasOwnProperty('isValid')) {
                  if (!cbobject.isValid) {
                    Logger.cmsLog({
                      category: "audit",
                      message: "Validating media " + fe.filename + " failed. Reason: " + cbobject.reason,
                      mediaID: fe.id,
                      method: "validateFile",
                      thread: "LibraryManager.addFile.downloader"
                    });
                  }
                }
                if (fe.filename.endsWith(".htz")) {
                  var from = fe.filename;
                  var to = fe.filename.substr(0, fe.filename.length - 4);
                  DeviceManager.FileSystem.unzipFile(
                    function () {
                      successCb(cbobject);
                    },
                    function () {
                    },
                    {from: from, to: to});
                } else {
                  successCb(cbobject);
                }
              }, function (cbObj) {
                Logger.cmsLog({
                  category: "audit",
                  message: "Download file failed (not valid). Reason: " + JSON.stringify(cbObj),
                  method: "validateFile",
                  thread: "LibraryManager.addFile.downloader"
                });
                failureCb(cbObj);
              }, tf);
            }, function (cbObject) {
              var message = JSON.stringify(cbObject);
              if (cbObject.httpStatus && cbObject.httpStatus === 404) {
                message = "File not found. Widget id: " + fe.id;
              }
              ;
              Logger.cmsLog({
                category: "audit",
                message: "Monitor download file failed. Reason: " + message,
                method: "validateFile",
                thread: "LibraryManager.addFile.downloader"
              });
              failureCb(cbObject);
            }, dlInfo);
          };
          break;
        case "xmds":
          tf = {
            id: file.id,
            size: file.size,
            type: file.type,
            updated: file.updated,
            md5: file.md5
          };

          switch (file.type) {
            case "layout":
              tf.filename = file.id + ".xlf";
              fe = this.getFileByName(tf.filename);
              if (fe === null) {
                fe = new LibraryManager.libFile(tf.id, tf.filename, tf.size, tf.type);
                fe.md5 = tf.md5;
                v_RequiredFiles.push(fe);
              } else {
                fe.id = tf.id;
              }

              fe.downloader = function (successCb, failureCb) {

                var dlInfo = {
                  path: fe.filename,
                  id: fe.id,
                  size: fe.size,
                  type: fe.type
                };

                CommunicationManager.downloadFile(
                  function (cbobject) {
                    fe.isCorrect = true;
                    fe.md5 = tf.md5;
                    fe.size = tf.size;
                    fe.lastChecked = utils.getTimeStamp();
                    successCb(cbobject);

                  }, function (cbObject) {
                    console.debug("Monitor download file failed!", cbObject);
                    failureCb(cbObject);
                  }, dlInfo);
              };
              break;
            case "media":
              tf.filename = file.path;
              fe = this.getFileByName(tf.filename);
              if (fe === null) {
                fe = new LibraryManager.libFile(tf.id, tf.filename, tf.size, tf.type);
                fe.md5 = tf.md5;
                v_RequiredFiles.push(fe);
              } else {
                fe.id = tf.id;
              }

              fe.downloader = function (successCb, failureCb) {

                var dlInfo = {
                  path: fe.filename,
                  id: fe.id,
                  size: fe.size,
                  type: fe.type
                };

                CommunicationManager.downloadFile(
                  function (cbobject) {
                    fe.isCorrect = true;
                    fe.md5 = tf.md5;
                    fe.size = tf.size;
                    successCb(cbobject);
                  }, function (cbObject) {
                    console.debug("Monitor download file failed!", cbObject);
                    failureCb(cbObject);
                  }, dlInfo);
              };
              break;
            default:
              console.warn("Could not download " + file.type + " from XMDS.");
              break;
          }
          break;
        default: //All other cases (Resources)
          tf = {
            filename: file.id + ".html",
            id: file.id,
            layoutid: file.layoutid,
            mediaid: file.mediaid,
            regionid: file.regionid,
            type: file.type,
            updated: file.updated
          };

          fe = this.getFileByName(tf.filename);
          if (fe === null) {
            fe = new LibraryManager.libFile(tf.id, tf.filename, tf.size, tf.type);
            fe.updated = tf.updated;
            v_RequiredFiles.push(fe);
          } else {
            fe.id = tf.id;
          }
          fe.downloader = function (successCb, failureCb) {

            var dlInfo = {
              layoutid: tf.layoutid,
              regionid: tf.regionid,
              mediaid: tf.mediaid,
              saveAs: tf.filename

            };
            CommunicationManager.downloadResource(function (cbobject) {
              fe.isCorrect = true;
              fe.updated = tf.updated;
              fe.manuallyInvalidated = false;

              /*****************************************************************************************************/
              fe.validateFile(function (cbobject) {
                if (cbobject.hasOwnProperty('isValid')) {
                  if (!cbobject.isValid) {
                    Logger.cmsLog({
                      category: "audit",
                      message: "Validating resource " + fe.filename + " failed. Reason: " + cbobject.reason,
                      mediaID: fe.id,
                      method: "validateFile",
                      thread: "LibraryManager.addFile.downloader"
                    });
                  }
                }
                successCb(cbobject);
              }, function (cbObj) {
                Logger.cmsLog({
                  category: "audit",
                  message: "Download resource failed (not valid). Reason: " + JSON.stringify(cbObj),
                  method: "validateFile",
                  thread: "LibraryManager.addFile.downloader"
                });
                failureCb(cbObj);
              }, tf);
              /*****************************************************************************************************/

            }, function (cbObject) {
              console.debug("Monitor download file failed!", cbObject);
              failureCb(cbObject);
            }, dlInfo);
          };


          break;
      }

      if (defined(fe)) {
        fe.validateFile(function (cbObj) {
          //console.debug("File validated : " + f.filename, cbObj);
          if (!fe.isCorrect) {
            console.debug("File " + fe.filename + " not valid. Reason: " + cbObj.reason);
          }
          cbSuccess(fe);
        }, function (e) {
          Logger.cmsFault(undefined, undefined,
            {
              code: ERRORS.FaultCodes.RemoteResourceFailed,
              reason: "Invalid file : " + fe.filename,
              type: "ERROR",
              method: "addFile",
              thread: "LibraryManager",
              date: new Date(),
              expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000)
            }
          );
          //console.debug("File validation failed :" + fe.filename,cbObj);
        }, tf);
      } else {
        cbSuccess(null);
      }
    };

    /**
     * Get file by name (path)
     * @param {String} name The name of the file (actually the path).
     * @returns {libFile} The file. Null if file not exists.
     */
    this.getFileByName = function (name) {
      return f_findIndexOfFileByProperty("filename", name);
    };

    /**
     * Get file by index.
     * @param {type} index The index of the file.
     * @returns {Object} The file.
     */
    this.getFileByIndex = function (index) {
      if (index >= v_RequiredFiles.length) {
        return null;
      }

      return v_RequiredFiles[index];
    };

    /**
     * Get file by id.
     * @param {String} id The id number.
     * @returns {Object} The file.
     */
    this.getFileById = function (id) {
      var idx = f_findIndexOfFileByProperty("id", id);

      return idx;
    };

    /**
     * Get file by action code.
     * @param {String} code The code.
     * @returns {Object} The file.
     */
    this.getFileByActionCode = function (code) {
      var idx = f_findIndexOfFileByProperty("code", code);
      return idx;
    };

    /*
     * Remove a file from library.
     * @param {Function} cbSuccess Callback function called after file deleted succesfull.
     * @param {Function} cbFailure Callback function called in case of a failure.
     * @param {String} filename The filename of the file.
     * @returns None
     * @see It will also remove the file from the filesystem.
     */
    this.removeFile = function (cbSuccess, cbFailure, filename) {
      //var f = LibraryManager.getFileByName(filename);

      var idx = null;
      for (var i = 0, max = v_RequiredFiles.length; i < max; i++) {
        if (v_RequiredFiles[i].filename === filename) {
          idx = i;
          break;
        }
      }
      if (idx !== null) {
        v_RequiredFiles.splice(idx, 1);
      }
      DeviceManager.FileSystem.deleteFile(
        cbSuccess,
        function (cbObject) {
          console.debug("Error", cbObject);
          //if (cbObject.errorText.substring(0,20) === "File does not exists")
          if (cbObject.code === 8) //File is not avalaible
          {
            cbSuccess();
          } else {
            cbFailure(cbObject);
          }
        },
        filename
      );

    };

    /**
     * Empty the files list.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.clearAllFiles = function () {
      v_RequiredFiles = [];
    };

    /**
     * Returns the count of files.
     * @returns {Number} Count of files.
     */
    this.length = function () {
      return v_RequiredFiles.length;
    };

    /**
     * Check if a file can be downloaded from CMS.
     * @param {type} filename The name of a file.
     * @returns {Boolean} True if it can else false.
     */
    this.isDownloadable = function (filename) {
      if (!defined(filename)) {
        return false;
      }
      var f = this.getFileByName(filename);
      return v_Downloadable_Types.indexOf(f.type) !== -1;
    };

    /**
     * Check if a file is valid.
     * @param {String} filename The name of a file.
     * @returns {Boolean} True if it is valid else false.
     */
    this.isFileOk = function (filename) {
      if (!defined(filename)) {
        return false;
      }
      var f = this.getFileByName(filename);

      if (f === null) {
        console.debug('LibraryManager.libFile.isFileOk: ' + filename + ' not in v_RequiredFiles!');
        return false;
      }

      var r = f.isCorrect;

      r = r && (typeof f.isBlacklisted !== 'undefined' ? !f.isBlacklisted : true);

      return r;
    };

    /**
     * Saves the library.
     * @returns None
     */
    this.saveLibrary = function () {
      localStorage.library = JSON.stringify(v_RequiredFiles);
    };

    /**
     * Loads the library.
     * @returns {Boolean} True if library loaded successfully else false.
     */
    this.loadLibrary = function () {
      if (localStorage.hasOwnProperty("library")) {
        var tmplibfiles = [];
        try {
          tmplibfiles = JSON.parse(localStorage.library);
        } catch (e) {
          console.warn('LibraryManager.loadLibrary: Parse error', e.message, e.stack);
          return false;
        }


        try {
          for (var i = 0, max = tmplibfiles.length; i < max; i++) {
            var f = new this.libFile(tmplibfiles[i].id, tmplibfiles[i].filename, tmplibfiles[i].size, tmplibfiles[i].type, tmplibfiles[i].download);
            f.md5 = tmplibfiles[i].md5;
            f.isCorrect = tmplibfiles[i].isCorrect;
            tmplibfiles[i].code && (f.code = tmplibfiles[i].code);
            if (tmplibfiles[i].type === "resource") {
              f.updated = tmplibfiles[i].updated;
            }
            v_RequiredFiles.push(f);
          }
        } catch (e) {
          console.warn('LibraryManager.loadLibrary: Error', e.message, e.stack);
          return false;
        }
      } else {
        console.debug('LibraryManager.loadLibrary: No library in localStorage');
      }
      return true;
    };

    /**
     * Clean up the library from old files
     * @param {Function} cbSuccess
     * @param {Function} cbFailure
     * @returns None
     */
    this.cleanUpLibrary = function (cbSuccess, cbFailure) {
      var i, j, maxi, maxj;
      console.debug("Clean up library");
      var markRemove = [];
      for (i = 0; i < v_RequiredFiles.length; i++) {
        var found = false;
        for (j = 0; j < v_TempRequiredFiles.length; j++) {
          if (v_TempRequiredFiles[j].hasOwnProperty("type")) {
            if (v_TempRequiredFiles[j].type === "media" ||
              v_TempRequiredFiles[j].type === "layout") {
              if (v_TempRequiredFiles[j].download === "xmds") {
                if (v_TempRequiredFiles[j].type === "layout") {
                  v_TempRequiredFiles[j].saveAs = v_TempRequiredFiles[j].id + ".xlf";
                } else if (v_TempRequiredFiles[j].type === "media") {
                  v_TempRequiredFiles[j].saveAs = v_TempRequiredFiles[j].path;
                }
              }
              if (v_RequiredFiles[i].filename === v_TempRequiredFiles[j].saveAs) {
                found = true;
                break;
              }
            } else if (v_TempRequiredFiles[j].type === "resource") {
              if (v_RequiredFiles[i].filename === v_TempRequiredFiles[j].id + ".html") {
                found = true;
                break;
              }

            }
          }
        }
        if (!found) {
          markRemove.push(v_RequiredFiles[i].filename);
        }
      }


      var itemsRemoved = false;

      var removeEach = function () {
        if (markRemove.length > 0) {
          itemsRemoved = true;
          LibraryManager.removeFile(removeEach, cbFailure, markRemove.pop());
        } else {

          cbSuccess(itemsRemoved);
        }
      };

      /* Issue #21 */
      DeviceManager.FileSystem.listFiles(
        function (files) {
          for (i = 0, maxi = files.length; i < maxi; i++) {
            if (files[i].isFile) {
              var found = false;
              for (j = 0, maxj = v_RequiredFiles.length; j < maxj; j++) {
                if (files[i].name === v_RequiredFiles[j].filename) {
                  found = true;
                  break;
                }
              }
              if (!found) {
                console.debug("Delete:" + files[i].name);
                markRemove.push(files[i].name);
              }
            }
          }
          removeEach();
        },
        function (cbObject) {
          console.debug("Could not list files!", cbObject);
          removeEach();
        }, null
      );

    };


    /**
     * Update file metadata (after download)
     * @param {Object} file The file downloaded
     * @returns {undefined}
     * @throws {ERRORS.ParameterNotDefined} If file has no filename.
     * @throws {ERRORS.FileNotFound} If file is not found in library.
     */
    this.updateFile = function (file) {
      if (!defined(file.saveAs)) {
        throw ERRORS.ParameterNotDefined("Filename is not defined in file.");
      }

      var fe = this.getFileByName(file.saveAs);

      if (fe === null) {
        throw ERRORS.FileNotFound();
      }

      if (fe.type !== "resource") {
        fe.size = file.size;
        fe.type = file.type;
        fe.md5 = file.md5;

      } else {
        fe.type = file.type;
        fe.updated = file.updated;
      }

      LibraryManager.saveLibrary();
    };


    this.getLibraryFiles = function () {
      return v_RequiredFiles;
    };

    /**
     * Array of blacklisted widgets
     * @property {Boolean} LibraryManager.blacklisted_widgets .
     * @version 1.8.18
     * @since 1.8.1
     */
    Object.defineProperty(this, 'blacklisted_widgets', {
      get: function () {
        if (!v_blacklisted_widgets) {
//	                if (localStorage.hasOwnProperty('blacklist_widgets')){
//	                    v_blacklisted_widgets = JSON.parse(localStorage.blacklist_widgets);
//	                    application.statusPage.blacklist(v_blacklisted_widgets);
//	                } else {
          v_blacklisted_widgets = [];
//	                }
          var localPush = v_blacklisted_widgets.push;
          v_blacklisted_widgets.push = function (value) {
            if (v_blacklisted_widgets.indexOf(value) === -1) {
              localPush.call(v_blacklisted_widgets, value);
//	                        localStorage.blacklist_widgets = JSON.stringify(v_blacklisted_widgets);
              application.statusPage.blacklist(v_blacklisted_widgets);
              var lfile = LibraryManager.getFileById(value);
              if (lfile !== null) {
                lfile.isBlacklisted = true;
                ScheduleManager.revalidateContent();
              }
            }
          };
        }

        return v_blacklisted_widgets;
      }
    });

    this.loadLibrary();

    // </editor-fold>
  }();
}());
