/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -
 */
/* global XMRManager, Logger, DeviceManager, logic, ScheduleManager, application, utils, ZmqSocket, RSAKey, byte2Hex, hex2b64, linebrk, b64tohex, b64toBA, defined */
(function () {
  "use strict";

  /**
   * @namespace XMRManager
   */

  //jshint -W057
  /**
   * @classdesc Manages XMR messages.
   * Provides XMR connection with the CMS.
   * @type XMRManager
   * @version 1.8.1
   * @since 1.8.1
   */
  window.XMRManager = new function _XMRManager() {

    /* +------------------------+ Private Variables +------------------------+ */
    // <editor-fold defaultstate="collapsed" desc="Private variables">
    //Minutes until reconnection if no message or error
    utils.const(this, 'XMR_RECONNECTIME', 70); //For testing I'm going to set timeout every second else time is 70 seconds (70). 

    var rsa;
    var v_socket = new ZmqSocket();
    var ready = false;
    var v_reconnectTimer = null;
    var v_reconnectTimeout = this.XMR_RECONNECTIME;
    var v_currentAddress = "";
    // </editor-fold>

    /* +------------------------+ Private functions +------------------------+ */
    // <editor-fold defaultstate="collapsed" desc="Private functions">

    // <editor-fold defaultstate="collapsed" desc="RSA functions">
    var loadPrivateKey = function () {
      // TODO : For testing we will use the predefined key
      var pk = JSON.parse(localStorage.privateKey);

      rsa = new RSAKey();
      rsa.setPrivateEx(pk.n, pk.e, pk.d, pk.p, pk.q, pk.dmp1, pk.dmq1, pk.coeff);
      if (rsa.n === null) {
        XMRManager.createKey();
      }
    };

    var savePrivateKey = function () {
      var savekey = {
        n: rsa.n.toString(16),
        e: rsa.e.toString(16),
        d: rsa.d.toString(16),
        p: rsa.p.toString(16),
        q: rsa.q.toString(16),
        dmp1: rsa.dmp1.toString(16),
        dmq1: rsa.dmq1.toString(16),
        coeff: rsa.coeff.toString(16)
      };
      localStorage.privateKey = JSON.stringify(savekey);

    };

    /**
     * Calculate the length header for PKCS.
     * @param {Integer} len The length value.
     * @return {String} The string representatino of length header for PKCS.
     */
    var calcLenHdr4PKCS = function (len) {
      if (len <= 127) {
        return byte2Hex(len);
      }

      var bsz = len.toString(16).length / 2;
      //var hdzero = false;
      var bn = Math.ceil(bsz);
      var hdrLen = byte2Hex(bn + 128);
      var combHdr = hdrLen;
      //jshint -W016
      if (bsz !== (bsz | 0)) {
        //jshint +W016
        combHdr += "0";
      }
      combHdr += len.toString(16);
//			if (isInteger === true){
//			combHdr +=  "00"; /* TODO: This is not always the case. If the first data byte is >128 then it needs the 00's */
//			}
      return combHdr;

      /* hdrLen is +128 */
    };

    /**
     * Create a sequence node which contains nodes.
     * @param {Array} nodes Array with hex representation of child nodes to add to sequence
     * @returns {Object} Returns a Sequence node with childs.
     */
    var createDerNodeSequence = function (nodes) {
      var node = "30";

      var nodesString = "";
      /* Calculate nodes length */
      var nodesLength = 0;
      for (var i = 0; i < nodes.length; i++) {
        nodesLength += nodes[i].length / 2;
        nodesString += nodes[i];
      }
      node += calcLenHdr4PKCS(nodesLength);
      node += nodesString;
      return node;
    };

    /**
     * Create a Integer node.
     * @param {String} data Hex representation of Integer node data.
     * @returns {Object} Returns a Integer node.
     */
    var createDerNodeInteger = function (data) {
      var pat = "01234567890ABCDEF";
      var node = "02";

      var extralen = 0;
      var posPrefix = false;
      var hexlen = data.length / 2;

      //jshint -W016
      if (hexlen !== (hexlen | 0))
        //jshint +W016
      {
        data = "0" + data;
      } else if (pat.indexOf(data.charAt(0).toUpperCase()) > 7) {
        posPrefix = true;
        extralen += 2;
      }

      node += calcLenHdr4PKCS(Math.ceil((data.length + extralen) / 2));
      if (posPrefix) {
        node += "00";
      }

      node += data;

      return node;
    };

    /**
     * Create a Bit String node.
     * @param {String} data Hex representation of Bit String node data.
     * @returns {Object} Returns a Bit String node.
     */
    var createDerNodeBitString = function (data) {
      var node = "03";
      node += calcLenHdr4PKCS(data.length / 2 + 1);
      node += "00";
      node += data;
      return node;
    };

    var constructPKCS = function () {

      var pk = rsa.n.toString(16);
      var exp = rsa.e.toString(16);

      /* Fix lengths */
      var hexlen = pk.length / 2;

      //jshint -W016
      if (hexlen !== (hexlen | 0))
        //jshint +W016
      {
        pk = "0" + pk;
      }

      hexlen = exp.length / 2;
      //jshint -W016
      if (hexlen !== (hexlen | 0))
        //jshint +W016
      {
        exp = "0" + exp;
      }

      var pkNode = createDerNodeInteger(pk);
      var expNode = createDerNodeInteger(exp);

      var pkexpNode = createDerNodeSequence([pkNode, expNode]);

      var bitString = createDerNodeBitString(pkexpNode);

      var seqNode = createDerNodeSequence(["300D06092A864886F70D0101010500", bitString]);

      var pem = hex2b64(seqNode);

      var lbpem = "-----BEGIN PUBLIC KEY-----\n" + linebrk(pem, 64) + "\n-----END PUBLIC KEY-----";


      DeviceManager.Config.setXmrPubKey(lbpem);
    };

    var decryptMessage = function (env_key, chipher) {

      var encRndkeyHex = b64tohex(env_key);
      var rndkeyStr = rsa.decrypt(encRndkeyHex);
      if (rndkeyStr === null) {
        console.error("Could not decrypt message");
        return;
      }

      var rndKeyBA = utils.conversions.hex2BA(rndkeyStr);

      var chipherBA = b64toBA(chipher);

      var encdata = rc4(rndKeyBA, chipherBA);

      console.info("XMR Data", encdata);

      return encdata;
    };

    /**
     * RC4 symmetric encryption/decryption .
     * @param {Array} key Byte array of key
     * @param {Array} str Byte array of encrypted data
     * @returns {String} String of decrypted data
     */
    var rc4 = function (key, str) {
      if (key === null || str === null) {
        return null;
      }
      var s = [], j = 0, k, x, res = '', z = [];
      for (var i = 0; i < 256; i++) {
        s[i] = i;
      }
      for (i = 0; i < 256; i++) {
        j = (j + s[i] + key[i % key.length]) % 256;
        x = s[i];
        s[i] = s[j];
        s[j] = x;
      }
      i = 0;
      j = 0;

      var myStr = '';

      for (var y = 0; y < str.length; y++) {
        i = (i + 1) % 256;
        j = (j + s[i]) % 256;
        x = s[i];
        s[i] = s[j];
        s[j] = x;
        k = s[(s[i] + s[j]) % 256];
        //jshint -W016
        //res += String.fromCharCode(str[y] ^ k);
        res += byte2Hex(str[y] ^ k);
        z.push(str[y] ^ k);
        myStr += String.fromCharCode(str[y] ^ k);
        //jshint +W016				
      }
      return myStr;
    };
    // </editor-fold>


    var socketReconnectCheck = function () {
      --v_reconnectTimeout;
      if (v_reconnectTimeout <= 0) {
        v_reconnectTimeout = XMRManager.XMR_RECONNECTIME;
        reconnect_socket();
      }
    };

    var reconnect_socket = function () {
      console.debug('XMR:Reconnect Attemp reconnect');
      ready = false;
      if (v_socket.state !== ZmqSocket.CLOSING) {
        v_socket.close();
      }

//            // detach event from previous socket;
//			v_socket.onopen = null;
//	        v_socket.onmessage = null;
//	        v_socket.onclose = null;
//	        v_socket.onerror = null;
//            v_socket = new ZmqSocket();
      XMRManager.connect();
    };

    // <editor-fold defaultstate="collapsed" desc="Socket functions">
    v_socket.onopen = function () {
      ready = true;
      console.info("Connected with XMR");
      v_socket.subscribe("H");
      v_socket.subscribe(DeviceManager.Config.getXmrChannel());

      /* FIX: XLG-84: XMR failed to reconnect on disconnect*/
      if (v_reconnectTimer !== null) {
        clearInterval(v_reconnectTimer);
      }
      v_reconnectTimer = setInterval(socketReconnectCheck, 1000);
    };
    v_socket.onmessage = function (data) {

      // If we receive data from CMS then we are connected.
      v_reconnectTimeout = XMRManager.XMR_RECONNECTIME;

      if (data[0] !== 'H') {
        console.log('IPC data', data);
        var dm = decryptMessage(data[1], data[2]);

        if (dm !== null) {
          XMRManager.routeCommand(dm);
        }
      } else {
        console.debug('Got heartbeat from XMR.');
        //TODO: Add XMR Heartbeat info to status page
        application.statusPage.lastXMRTimestamp(utils.getLocalDate());
      }
      //} else {
      //    console.warn('Assertion failure!');
    };
    v_socket.onclose = function () {
      console.info('close');
    };
    v_socket.onerror = function (msg) {
      console.warn(msg);

      if (v_socket.state !== ZmqSocket.CLOSING) {
        v_socket.close();
      }

//			v_socket.onopen = null;
//	        v_socket.onmessage = null;
//	        v_socket.onclose = null;
//	        v_socket.onerror = null;

      /* FIX: XLG-84: XMR failed to reconnect on disconnect*/
      if (v_reconnectTimer !== null) {
        clearInterval(v_reconnectTimer);
      }
      v_reconnectTimer = setInterval(socketReconnectCheck, 1000);
    };


    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="XMR Commands">


    // XMR Commands </editor-fold> 

    // </editor-fold>


    /* +------------------------+ Public functions +------------------------+ */
    // <editor-fold defaultstate="collapsed" desc="Public functions">

    /**
     * Initialization of XMRManager
     */
    this.init = function () {

//			//TODO : FOR TESTING ONLY
//			var pk = {"n":"88da3dc80de78546032ae925b34ea752c83a6fb43b6f5240e4c51d709e2d8e0e8b672abbb1c3305c88b1e2bd93048bb40bd39cefc840b2b10ef5d8708a28521c27a1211b1c545e0ed1cd01e2a710e5bd71e00e59b58f6b70d95f949262c22b999e47f9dbd6adc3a39c16bee4a07b541155f705e5d7813667562579c634063e29",
//					  "e":"10001",
//					  "d":"5e37e2d118c18cbfe6ee6415d09d1615a844932342c821990be7a99d1f49bdc3c5b60175b7acea5ecabdad9d56da8a542ad045fbcfa2bb63ab29171a6403de8300a6230d98f5a2d0851e728c3be6b7ea91430d6212c598067f27b83a69da8b262c135e473b840e9c5d9d8246acf97f1f982754f02aca1545383f5c01c288b501",
//					  "p":"cfac09538f8600843c7d2efe7708f532df29cf613a5fbb0d1fafc033b9884afb98c538085eb187551b6f5160c6cb3d6ddd6168b33fc675b411f1c46907ea3547",
//					  "q":"a8b32a5e4486dac873466e1096814a7f43a7e91043c4535cb1cc90a2bbfd4c75c3d295bccf11f7213984632471d59a4f18517465116852f37649617a0c75690f",
//					  "dmp1":"a0ab694bcceecfd77f351af6a501b932f295566cd2ccdddaf978f02b148fedf359eebbbb3e07fbbc4ccd83186cb531819e958e7ba00bdb64c5bd57482b67787",
//					  "dmq1":"679c5aaa3916462b14b9ae01dc1e1e5e1974782d3821cd3d422a2caef61cb528231424492938947ac073b1dc8159ba8357a1378697716679072e3c831441cc3b",
//					  "coeff":"a6e1146e2954e2df68d0a0b95f04adf1b177ec0fbc5e4d759ebc3f2199a1a6d14c61e9a3732919a6ce985d00c167264a986de110381d236366fa2bf4963c48fd"}
//			localStorage.privateKey = JSON.stringify(pk);
//			//TODO : FOR TESTING ONLY - END

      /* Load or create a new RSA key. */
      if (localStorage.hasOwnProperty("privateKey")) {
        loadPrivateKey();
      } else {
        this.createKey();
      }

      /* FOR DEVELOPMENT ONLY */
      //constructPrivatePKCS();
      /* FOR DEVELOPMENT ONLY */
//			decryptMessage("ABxMpGP7n5quUtT06LnzXeWWGUW0LBGxO2g6Qojm5WtJnRk22Ny+1xgfRZmfde1jgwy8eeL/YKFJWBVCiWMwa4DK/F9V34F43ZmL3AhjrnS/EHcXKWd5+Z28OgvSoBZUK8Einnpt1cm8zx4M6hSCQwP5/yPRwdsOo00kTBWj0Z4=",
//			"Ve2bzKqdlWqLRDiz0S7m7g0=");

      DeviceManager.NodeServer.init();
    };

    /**
     * Create a new RSA Key.
     * @method XMRManager.createKey
     * @returns None
     */
    this.createKey = function () {
      //constructPKCS(); //Only for test before rsa.generate!
      console.debug("Create new key!");
      rsa = new RSAKey();
      rsa.generate(1024, "10001");
      console.debug("RSA Key generated" /*, JSON.stringify(rsa)*/);
      savePrivateKey();
      constructPKCS();
    };

    /**
     * Create a XMR public key from private!
     * @method XMRManager.createPubKey
     * @returns None
     */
    this.createPubKey = function () {
      if (defined(rsa)) {
        if (defined(rsa.n)) {
          constructPKCS();
        }
      }
    };


    /**
     * Connect to the XMR address and port.
     * @returns None
     * If "xmrNetworkAddress" is not set it will return without connecting.<br>
     * xmrNetworkAddress format is tcp://IP:PORT
     */
    this.connect = function () {
      var v_addr = "";
      var v_port = 9505;

      if (DeviceManager.Config.getOptions().hasOwnProperty("xmrNetworkAddress")) {
        var tempAddr = DeviceManager.Config.getOptions().xmrNetworkAddress;
        if (tempAddr === 'DISABLED') {
          if (v_socket) {
            if (v_socket.state === ZmqSocket.OPEN) {
              v_socket.close();
            }
          }
          return;
        }

        if (v_socket) {
          if (v_socket.state === ZmqSocket.OPEN) {
            return;
          }
        }
        /*Inform status Page*/
        application.statusPage.xmrAddr(tempAddr);
        v_currentAddress = tempAddr;
        /* Normaly tcp is in front. */
        //jshint -W018
        if (!(tempAddr.substr(0, 3) === 'tcp'))
          //jshint +W018
        {
          return;
        }

        tempAddr = "http" + tempAddr.substr(3);


        var link = document.createElement('a');
        //  set href to any path
        link.setAttribute('href', tempAddr);

        v_addr = link.hostname;
        if (link.port !== undefined) {
          v_port = link.port;
        }
      } else {
        return;
      }

      console.log('XMR.connect called for fd:' + v_socket.fd);
      v_socket.connect(v_addr, v_port);
    };

    this.getAddress = function () {
      return v_currentAddress;
    }

    this.close = function () {
      v_socket.close();
    }

    /**
     * Route the command from XMR.
     * @param {String} msg The command received.
     * @returns None
     */
    this.routeCommand = function (msg) {
      logic.status.setLastCommandStatus(false);
      var cmd;
      try {
        cmd = JSON.parse(msg);
      } catch (e) {
        console.warn("XMR command not understood!", msg);
        //Logger.cmsLog({category:"error",message:"XMR command not understood!", method:"routeCommand", thread:"XMRManager"});
        // XXX: make a suggestion for a proper error code for this case
        Logger.cmsFault(undefined, undefined, {
          code: ERRORS.FaultCodes.RemoteResourceFailed,
          reason: "XMR command not understood!",
          method: "routeCommand",
          thread: "XMRManager",
          date: new Date(),
          expires: new Date(new Date().valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000)
        });
        return;
      }

      if (cmd.hasOwnProperty('createdDt')) {
        if (cmd.hasOwnProperty('ttl')) {
          var displayTime = Date.now();
          if (Math.abs((displayTime - Date.parse(cmd.createdDt)) / 1000) > cmd.ttl) {
            /* Command outside our timeframe. Don't run! */
            Logger.cmsLog({
              category: "audit",
              message: "XMR command outside timeframe (Display time: " + displayTime + ", Command time: " + Date.parse(cmd.createdDt) + ", TTL: " + cmd.ttl + ")!",
              method: "routeCommand",
              thread: "XMRManager"
            });
            return;
          }
        }
      }


      if (cmd.hasOwnProperty('action')) {
        switch (cmd.action) {
          case 'changeLayout':
            ScheduleManager.Commands.changeLayout(cmd.layoutId, cmd.duration, cmd.downloadRequired, cmd.changeMode);
            break;
          case 'collectNow':
            ScheduleManager.Commands.collectNow();
            break;
          case 'commandAction':
            if (cmd.hasOwnProperty("commandCode")) {
              ScheduleManager.Commands.commandAction(cmd.commandCode);
            }
            break;
          case 'overlayLayout':
            ScheduleManager.Commands.overlayLayout(cmd.layoutId, cmd.duration, cmd.downloadRequired);
            break;
          case 'rekeyAction':
            ScheduleManager.Commands.rekeyAction();
            break;
          case 'licenceCheck':
            //Collect now to get the profile settings
            //Check for Licence (LicenceMgr / checkLicenseServer)
            CommunicationManager.ConnectCMS(
              function (d) {
                //console.debug('Licence Check Connect to CMS');
                LicenseManager.checkLicenseServer(function () {
                    var licRes = 'Not licenced';
                    if (LicenseManager.isLicenseValid()) {
                      licRes = 'Licensed fully';
                    } else if (LicenseManager.inTrial()) {
                      licRes = "Trial";
                    }
                    CommunicationManager.emergencyNotify(null, function () {
                    }, {licenceResult: licRes});
                  }, function () {
                    //TODO: What if fails!
                    console.debug('Licence Check Connect to CMS failed!');
                    var licRes = 'Not licenced';
                    if (LicenseManager.isLicenseValid()) {
                      licRes = 'Licensed fully';
                    } else if (LicenseManager.inTrial()) {
                      licRes = "Trial";
                    }
                    CommunicationManager.emergencyNotify(null, function () {
                    }, {licenceResult: licRes});
                  },
                  true);
              }, function () {
                //TODO: What if fails!
                console.debug('licenceCheck - CommunicationManager.ConnectCMS failed!');
              });
            break;
          case 'revertToSchedule':
            ScheduleManager.Commands.revertToSchedule();
            break;
          case 'screenShot':
            ScheduleManager.Commands.takeScreenshot();
            break;
          case 'triggerWebhook':
            InteractiveManager.handleWebHookEvent(cmd.triggerCode);
            break;
          default:
            console.warn("XMR command " + cmd.action + " not supported!");
            break;
        }
      }
      /*
       * {"action":"screenShot","createdDt":"2017-04-12T15:35:44+03:00","ttl":120}
       */


    };

    // </editor-fold>  
  }();
  //jshint +W057
}());
