/* global utils, CommunicationManager, WebOSManager, application, IDBKeyRange, ERRORS */

(function () {
  "use strict";
  /**
   * @classdesc Log facility for the application.
   * @type Logger
   * @version 1.8.1
   * @since 1.8.1
   */
  //jshint -W057
  window.Logger = new function _Logger() {

    // <editor-fold defaultstate="collapsed" desc="Private variables">
    /* +------------------------+ Private Variables +------------------------+ */

    /**
     * Instance of IndexDB.
     * @version 1.8.1
     * @since 1.8.1
     */
    var v_db;

    /**
     * Name of Logs db object store.
     * @type {String}
     * @version 1.8.1
     * @since 1.8.1
     */
    var v_nameObjStoreLogs = "logs";

    /**
     * Name of Statistics db object store.
     * @type {String}
     * @version 1.8.1
     * @since 1.8.1
     */
    var v_nameObjStoreStats = "stats";

    var v_cmsLogCategory = Object.create(null);

    Object.defineProperty(v_cmsLogCategory, 'AUDIT', {
      enumerable: false,
      configurable: false,
      writable: false,
      value: 'audit'
    });

    Object.defineProperty(v_cmsLogCategory, 'ERROR', {
      enumerable: false,
      configurable: false,
      writable: false,
      value: 'error'
    });

    /**
     * Is sending logs as backlog.
     * @type {Boolean}
     * @version 2.0.201
     * @since 2.0.201
     */
    var v_BacklogLogs = false;

    /**
     * Is sending stats as backlog.
     * @type {Boolean}
     * @version 2.0.200
     * @since 2.0.200
     */
    var v_BacklogStats = false;

    /**
     * Count of logs in db
     * @type Number
     */
    var v_BacklogLogsCount = 0;

    /**
     * Count of stats in db
     * @type Number
     */
    var v_BacklogStatsCount = 0;

    /**
     * How much time should we wait before we send again!
     * @type Number
     */
    var v_FaultsRetryTime = 0;

    /**
     * List of faults that are to be reported to the CMS.
     * @type {Array}
     * @version 3.0.300
     * @since 3.0.300
     */
    var v_FaultsStorage = [];

    /* +------------------------+ Private Function  +------------------------+ */
    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="Private functions">
    /**
     * Add log entry to CMS log.
     * @param {Object} entry An object with the folowing fields.
     * <table>
     * <tr><th>Type</th><th>Field</th><th>Description</th><th>Optional</th></tr>
     * <tr><td>"error" | "audit"</td><td>category</td><td>Either error or audit.</td><td>No</td></tr>
     * <tr><td></td><td></td><td>Audit messages are discarded unless auditing<br> is switched on globally and on the display.</td><td></td></tr>
     * <tr><td>String</td><td>message</td><td>The log message.</td><td>No</td>
     * <tr><td>String</td><td>scheduleID</td><td>The assosiated schedule id.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>layoutID</td><td>The assosiated layout id.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>mediaID</td><td>The assosiated mediaID id.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>type</td><td>The type.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>method</td><td>The method.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>thread</td><td>The Thread that the log message executed on.</td><td>Yes</td></tr>
     * <tr><td>Boolean</td><td>onlyOnce</td><td>Don't put the same message more than once if already exists in the log to send.</td><td>Yes</td></tr>
     * </table>
     * @returns None
     * @example cmsLog({category:"error",message:"Error message"});
     */

    /**
     * Create a new log entry.
     * @param {[Logger.cmsLog.CATEGORY.ERROR | Logger.cmsLog.CATEGORY.AUDIT]} category Either error or audit. Audit messages are discarded unless auditing<br> is switched on globally and on the display.
     * @param {String} message The log message.
     * @returns {Logger.logEntry} Return a new log entry.
     */
    function logEntry(category, message) {

      if (category !== v_cmsLogCategory.ERROR && category !== v_cmsLogCategory.AUDIT) {
        throw new TypeError("Category must be 'error' or 'audit'");
      }

      if (!defined(message)) {
        throw new TypeError("Message must be set!");
      }
      this.date = utils.getISODate();
      this.category = category;
      this.message = message;
      this.onlyOnce = false;

      /**
       * Get the DOM representation of the log entry.
       * @returns {Object} Return the DOM equivelant to the log entry.
       */
      this.constructor.prototype.toElement = function () {
        var doc = document.implementation.createDocument(null, "l", null);

        var de = doc.createElement("log");
        de.setAttribute("date", this.date);
        de.setAttribute("category", this.category);

        var me = doc.createElement("message");
        me.textContent = this.message;
        de.appendChild(me);

        if (this.scheduleID !== undefined) {
          var si = doc.createElement("scheduleID");
          si.textContent = this.scheduleID;
          de.appendChild(si);
        }

        if (this.layoutID !== undefined) {
          var li = doc.createElement("layoutID");
          li.textContent = this.layoutID;
          de.appendChild(li);
        }

        if (this.mediaID !== undefined) {
          var mi = doc.createElement("mediaID");
          mi.textContent = this.mediaID;
          de.appendChild(mi);
        }

        if (this.type !== undefined) {
          var ty = doc.createElement("type");
          ty.textContent = this.type;
          de.appendChild(ty);
        }

        if (this.method !== undefined) {
          var met = doc.createElement("method");
          met.textContent = this.method;
          de.appendChild(met);
        }

        if (this.thread !== undefined) {
          var th = doc.createElement("thread");
          th.textContent = this.thread;
          de.appendChild(th);
        }

        return de;
      };

      /**
       * Compare if two entry logs are equal.
       * @param {type} compareObj The entry log to compare to.
       * @returns {Boolean} True if equal else false.
       */
      this.constructor.prototype.equals = function (compareObj) {
        var cmpAttribs = ["category", "message", "method", "thread", "type", "scheduleID", "layoutID", "mediaID"];

        for (var i = 0, max = cmpAttribs.length; i < max; i++) {
          if (this[cmpAttribs[i]] !== compareObj[cmpAttribs[i]]) {
            return false;
          }
        }
        return true;
      };

      /**
       * Search into indexDB if this log entry exists.
       * @param {function} successCb Callback function with result. True if exist else false.
       * @param {function} failureCb Callback function in caes of an error.
       * @returns {undefined} undefined.
       */
      this.constructor.prototype.exists = function (successCb, failureCb) {
        var transaction = v_db.transaction([v_nameObjStoreLogs], "readonly");
        var objectStore = transaction.objectStore(v_nameObjStoreLogs);
        var request = objectStore.openCursor();
        var current = this;

        request.onerror = function (event) {
          console.error(event);
          if (failureCb) {
            failureCb(event);
          }
        };

        request.onsuccess = function (event) {
          var cursor = event.target.result;
          if (cursor) {
            var log = cursor.value;
            if (current.equals(log)) {
              successCb(true);
            } else {
              cursor.continue();
            }
          } else {
            if (successCb) {
              successCb(false);
              return;
            }
          }
        };
      };
    }

    /**
     * Create a fault entry. Fault entries can fallback on log entry if XMDS is less than 6.
     * @param code {Error.FaultCodes} The error code
     * @param reason {String} The reason of the fault
     * @param date {Date} The date the incident occured (dateTime string, Format Y-m-d H:i:s)
     * @param expires {Date} The date the fault expires (dateTime string, Format Y-m-d H:i:s)
     * @param layoutId {Number=} The ID of the affected Layout (integer)
     * @param regionId {Number=} The ID of the affected Region (integer)
     * @param widgetId {Number=} The ID of the affected Widget (integer)
     * @param scheduleId {Number=} The ID of the affected Schedule event (integer)
     * @param mediaId {Number=} The ID of the affected Media (integer)
     * @returns {faultEntry} Return a new fault entry
     * @throws {TypeError} If code is not set.
     * @throws {TypeError} If reason is not set.
     * @throws {TypeError} If date is not set.
     * @throws {TypeError} If expire is not set.
     * @version 3.0.300
     * @since 3.0.300
     */
    function faultEntry(code, reason, date, expires, layoutId, regionId, widgetId, scheduleId, mediaId) {

      if (!defined(code)) {
        throw new TypeError("Code must be set!");
      }
      if (!defined(reason)) {
        throw new TypeError("Reason must be set!");
      }
      if (!defined(date)) {
        throw new TypeError("Date must be set!");
      }
      if (!defined(expires)) {
        throw new TypeError("Expires must be set!");
      }

      this.code = code;
      this.reason = reason;
      this.date = date;
      this.expires = expires;

      this.category = v_cmsLogCategory.ERROR; // For combatibility with logEntry
      this.message = reason; // For combatibility with status page and log messages array
      this.onlyOnce = false;

      layoutId && (this.layoutId = layoutId);
      regionId && (this.regionId = regionId);
      widgetId && (this.widgetId = widgetId);
      scheduleId && (this.scheduleId = scheduleId);
      mediaId && (this.mediaId = mediaId);

      /**
       * Get the correct JSON notation
       * @returns {String} The faultEntry as JSON text.
       */
      this.constructor.prototype.toJSON = function () {
        //this.toJSON = function(){
        var o = Object.assign({}, this);
        delete o.category;
        delete o.message;
        delete o.onlyOnce;
        o.date = utils.getISODate(this.date);
        o.expires = utils.getISODate(this.expires);
        return o;
      };

      /**
       * Compare if two entry logs are equal.
       * @param {type} compareObj The entry log to compare to.
       * @returns {Boolean} True if equal else false.
       */
      this.constructor.prototype.equals = function (compareObj) {
        // "date", "expires" are calculated in case a similar fault exists
        var cmpAttribs = ["code", "reason", "layoutId", "regionId", "widgetId", "scheduleId", "mediaID"];

        for (var i = 0, max = cmpAttribs.length; i < max; i++) {
          if (this[cmpAttribs[i]] !== compareObj[cmpAttribs[i]]) {
            return false;
          }
        }
        return true;
      };

      /**
       * Search into indexDB if this log entry exists.
       * @returns {boolean} True if exist else false.
       */
      this.constructor.prototype.exists = function () {
        for (var i = 0; i < v_FaultsStorage.length; i++) {
          if (this.equals(v_FaultsStorage[i])) {
            return true;
          }
        }
        return false;
      };
    };

    /**
     * Prepare the indexDB for storing Logs and Stats.
     * @param {Function} cbSuccess Call on success and pass the db instance back.
     * @param {Function} cbFailure Call on initialization failure.
     * @returns {undefined} undefined
     * @version 3.0.300
     * @since 1.8.14
     */

    function prepareDB(cbSuccess, cbFailure) {
      var schemaVer = 1;
      var dbName = 'xibo';
      var req = window.indexedDB.open(dbName, schemaVer);


      req.onerror = function (ev) {
        if (ev.target.error.name === "VersionError") {
          var req = indexedDB.deleteDatabase(dbName);
          req.onsuccess = function () {
            console.log("Deleted database successfully");
            LayoutManager.toast(undefined, undefined, "Deleted database successfully");
            //setTimeout(function(){DeviceManager.Device.powerCommand('reboot');}, 5000);
            setTimeout(function () {
              prepareDB(cbSuccess, cbFailure);
            }, 1000);
          };
          req.onerror = function () {
            console.log("Couldn't delete database");
            LayoutManager.toast(undefined, undefined, "Couldn't delete database");
            setTimeout(function () {
              DeviceManager.Device.powerCommand('reboot');
            }, 5000);
          };
          req.onblocked = function () {
            console.log("Couldn't delete database due to the operation being blocked");
            LayoutManager.toast(undefined, undefined, "Couldn't delete database due to the operation being blocked");
            setTimeout(function () {
              DeviceManager.Device.powerCommand('reboot');
            }, 5000);
          };
        } else {
          console.error(ev);
          LayoutManager.toast(undefined, undefined, ev.target.error.message);
          //          if (cbFailure) {
          //            cbFailure(ev);
          //          }
        }
      };

      req.onsuccess = function (ev) {
        console.debug('Database initialized!');
        v_db = ev.target.result;
        if (cbSuccess) {
          cbSuccess(v_db);
        }
        Logger.countLogs(
          function (cl) {
            console.debug('Logs in db', cl);
            Logger.countStats(
              function (cs) {
                console.debug('Stats in db', cs);
              },
              function (e) {
                console.error(e);
              }
            );
          },
          function (e) {
            console.error(e);
          }
        );
      };

      req.onupgradeneeded = function (ev) {
        //console.info(ev);
        var db = ev.target.result;

        db.onerror = function (ug_ev) {
          console.error(ug_ev);
        };

        if (ev.oldVersion === 0) {
          db.createObjectStore(v_nameObjStoreLogs, {autoIncrement: true});
          db.createObjectStore(v_nameObjStoreStats, {autoIncrement: true});
        }
      };
    }

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="Public functions">
    /* +------------------------+ Public Functions  +------------------------+ */

    /**
     * Add log entry to CMS log.
     * @param {Object} entry An object with the folowing fields.
     * <table>
     * <tr><th>Type</th><th>Field</th><th>Description</th><th>Optional</th></tr>
     * <tr><td>"error" | "audit"</td><td>category</td><td>Either error or audit.</td><td>No</td></tr>
     * <tr><td></td><td></td><td>Audit messages are discarded unless auditing<br> is switched on globally and on the display.</td><td></td></tr>
     * <tr><td>String</td><td>message</td><td>The log message.</td><td>No</td>
     * <tr><td>String</td><td>scheduleID</td><td>The assosiated schedule id.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>layoutID</td><td>The assosiated layout id.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>mediaID</td><td>The assosiated mediaID id.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>type</td><td>The type.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>method</td><td>The method.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>thread</td><td>The Thread that the log message executed on.</td><td>Yes</td></tr>
     * <tr><td>Boolean</td><td>onlyOnce</td><td>Don't put the same message more than once if already exists in the log to send.</td><td>Yes</td></tr>
     * </table>
     * @param {function} cbSuccess Callback function after add log successfuly.
     * @param {function} cbFailure Callback function on error.
     * @returns None
     * @example cmsLog({category:"error",message:"Error message"}, cbSuccess, cbFailure);
     */
    this.cmsLog = function (entry, cbSuccess, cbFailure) {

      if (!defined(entry)) {
        console.error("CMS log entry not defined!");
      }

      if (!entry.hasOwnProperty("category")) {
        console.error("CMS log entry has no category defined!");
      }
      if (!entry.hasOwnProperty("message")) {
        console.error("CMS log entry has no message defined!");
      }

      var v_logEntry = new logEntry(entry.category, entry.message);

      if (entry.hasOwnProperty("scheduleID")) {
        v_logEntry.scheduleID = entry.scheduleID;
      }

      if (entry.hasOwnProperty("layoutID")) {
        v_logEntry.layoutID = entry.layoutID;
      }

      if (entry.hasOwnProperty("mediaID")) {
        v_logEntry.mediaID = entry.mediaID;
      }

      if (entry.hasOwnProperty("type")) {
        v_logEntry.type = entry.type;
      }

      if (entry.hasOwnProperty("method")) {
        v_logEntry.method = entry.method;
      }

      if (entry.hasOwnProperty("thread")) {
        v_logEntry.thread = entry.thread;
      }

      if (entry.hasOwnProperty("onlyOnce")) {
        v_logEntry.onlyOnce = entry.onlyOnce;
      }

      /*
      if (v_logEntry.category === "audit") {
        console.info("Audit", v_logEntry);
      } else {
        console.warn("Error", v_logEntry);
      }
      */


      var addLogEntry = function (le) {
        var transaction = v_db.transaction([v_nameObjStoreLogs], "readwrite");

        transaction.onerror = function () {
          console.error('Error adding log', transaction.error);
        };

        var objectStore = transaction.objectStore(v_nameObjStoreLogs);
        var objectStoreRequest = objectStore.add(le);

        objectStoreRequest.onsuccess = function () {
//          console.debug('Log added', le);
          v_BacklogLogsCount += 1;
          application.statusPage.backlogLogsCount(v_BacklogLogsCount);
          if (cbSuccess) {
            cbSuccess();
          }
        };

        objectStoreRequest.onerror = function (event) {
          console.debug('Log add failed', event);
          if (cbFailure) {
            cbFailure();
          }
        };

      };

      var checkOnlyOnce = function () {
        //Find if onlyOnce is set
        if (v_logEntry.onlyOnce) {
          v_logEntry.exists(
            function (result) {
              if (!result) {
                addLogEntry(v_logEntry);
              }
            }, function (err) {
              throw err;
            });
        } else {
          addLogEntry(v_logEntry);
        }
      };


      var v_logLevel = DeviceManager.Config.getLogLevel();

      switch (v_logLevel) {
        case 'error':
          if (v_logEntry.category === v_cmsLogCategory.ERROR) {
            checkOnlyOnce();
            application.statusPage.addLogMessage(v_logEntry);
          }
          break;
        case 'audit':
          checkOnlyOnce();
          application.statusPage.addLogMessage(v_logEntry);
          break;
        default:
          //Do nothing
          break;
      }
    };

    Object.defineProperty(this.cmsLog, 'CATEGORY', {
      enumerable: false,
      configurable: false,
      writable: false,
      value: v_cmsLogCategory
    });

    /**
     * Submit the log to the cms.
     * @param {Function} successCb Function to call on success.
     * @param {Function} failureCb Function to call on failure.
     * @returns None
     */
    this.submitLog = function (successCb, failureCb) {
      var v_callbackCalled = false;

      var sendLogs = function () {
        var doc = document.implementation.createDocument(null, "logs", null);


        var sendBatchCount = 50;

        Logger.countLogs(
          function (count) {

            if (count === 0) {
              v_BacklogLogs = false;
              if (!v_callbackCalled && successCb) {
                successCb();
              }
              return;
            } else if (count > 500) {
              v_BacklogLogs = true;
            }

            if (v_BacklogLogs) {
              sendBatchCount = 300;
            }

            Logger.getAllLogKeys(function (keys) {
                var keyMin = keys[0];
                var keyMax = keys[keys.length - 1];

                Logger.getAllLogs(
                  function (values) {

                    //If we support reportFault use it else fallback to default

                    for (var i = 0; i < values.length; i++) {
                      var logObj = Object.create(logEntry.prototype);
                      var log = Object.assign(logObj, values[i]);

                      doc.firstChild.appendChild(log.toElement());
                    }
                    var l = (new XMLSerializer()).serializeToString(doc);
                    CommunicationManager.sendLog(
                      function () {
                        application.statusPage.backlogLogsDate(utils.getISODate());

                        // Backlog should run on its own timing regardless of sendStats.
                        Logger.removeSpecificLogs(
                          function () {
                            v_BacklogLogsCount -= values.length;
                            if (v_BacklogLogsCount < 0) {
                              v_BacklogLogsCount = 0;
                            }
                            application.statusPage.backlogLogsCount(v_BacklogLogsCount);
                            if (v_BacklogLogs) {
                              setTimeout(sendLogs, 30000);
                            }
                            if (!v_callbackCalled) {
                              v_callbackCalled = true;
                              if (successCb) {
                                successCb();
                              }
                            }
                          },
                          failureCb,
                          IDBKeyRange.bound(keyMin, keyMax)
                        );
                      },
                      failureCb,
                      l
                    );
                  },
                  function (event) {
                    console.error(event);
                    if (failureCb) {
                      failureCb(event);
                    }
                  },
                  sendBatchCount
                );
              },
              failureCb,
              sendBatchCount);
          },
          failureCb
        );
      };

      sendLogs();
    };

    /**
     * Add stat entry.
     * @param {Object} stat An object with the folowing fields.
     * <table>
     * <tr><th>Type</th><th>Field</th><th>Description</th><th>Optional</th></tr>
     * <tr><td>String</td><td>fromdt</td><td>The ISO date that the layout/media started playing.</td><td>No</td></tr>
     * <tr><td>String</td><td>todt</td><td>The ISO date that the layout/media finished playing.</td><td>No</td>
     * <tr><td>String</td><td>scheduleid</td><td>The ID of the schedule that caused the Layout/Media to be shown.</td><td>No</td></tr>
     * <tr><td>String</td><td>layoutid</td><td>The assosiated layout id.</td><td>Yes</td></tr>
     * <tr><td>String</td><td>mediaid</td><td>The assosiated mediaID id.</td><td>Yes</td></tr>
     * <tr><td colspan="4"><i>Either <b>layoutid</b> or <b>mediaid</b> must be set.</i></td></tr>
     * </table>
     * @param {function} cbSuccess Callback function after add log successfuly.
     * @param {function} cbFailure Callback function on error.
     * @returns None
     * @see For ISO date see utils.getISODate()
     * @example Logger.logStat({fromdt:utils.getISODate(),
     todt:utils.getISODate(),
     scheduleid:"5", 
     layoutid:"1"
     });<br>
     Logger.logStat({fromdt:utils.getISODate(),
     todt:utils.getISODate(),
     scheduleid:"5", 
     mediaid:"1"
     });
     */
    this.logStat = function (stat, cbSuccess, cbFailure) {

      if (!DeviceManager.Config.getOptions().hasOwnProperty("statsEnabled")) {
        return;
      }

      if (DeviceManager.Config.getOptions().statsEnabled == false) {
        return;
      }

      if (!defined(stat)) {
        console.error("Statistic not defined!");
      }

      if (!stat.hasOwnProperty("layoutid")) {
        console.error("No layout id defined!");
      }

      if (!stat.hasOwnProperty("fromdt")) {
        console.error("No from date defined!");
      }

      if (!stat.hasOwnProperty("todt")) {
        console.error("No to date defined!");
      }

      if (!stat.hasOwnProperty("scheduleid")) {
        console.error("No schedule id defined!");
      }

      if (stat.hasOwnProperty("mediaid")) {
        stat.type = "media";
      } else {
        stat.type = "layout";
      }

      var transaction = v_db.transaction([v_nameObjStoreStats], "readwrite");

      transaction.onerror = function (event) {
        console.error('Error adding stat', transaction.error);
      };


      var objectStore = transaction.objectStore(v_nameObjStoreStats);
      var objectStoreRequest = objectStore.add(stat);

      objectStoreRequest.onsuccess = function (event) {
        //console.debug('Stat added', stat);
        v_BacklogStatsCount += 1;
        application.statusPage.backlogStatsCount(v_BacklogStatsCount);

        if (cbSuccess) {
          cbSuccess();
        }
      };

      objectStoreRequest.onerror = function (event) {
        console.debug('Stat add failed', event);
        if (cbFailure) {
          cbFailure();
        }
      };

    };

    /**
     * Submit the log to the cms.
     * @param {Function} successCb Function to call on success.
     * @param {Function} failureCb Function to call on failure.
     * @returns None
     */
    this.submitStats = function (successCb, failureCb) {

      var v_callbackCalled = false;
      /**
       * Delete a stat or an array of stats
       *
       * @param {type} sucCb  Callback when a key or keys get deleted.
       * @param {type} failCb Callback in case of failure.
       * @param {type} sid  Index or array of key indexes to delete.
       * @returns {undefined} Does not return. Use the callback for results.
       */
      var deleteStat = function (sucCb, failCb, sid) {
//    	      var transaction = v_db.transaction([v_nameObjStoreStats], "readwrite");
//    	      var objectStore = transaction.objectStore(v_nameObjStoreStats);

        //sid should always be an array internally
        if (!Array.isArray(sid)) {
          sid = [sid];
        }

        var transaction = v_db.transaction([v_nameObjStoreStats], "readwrite");
        var objectStore = transaction.objectStore(v_nameObjStoreStats);

        function intDelStat() {
          var k = sid.pop();
          var objectStoreRequest = objectStore.delete(k);
          objectStoreRequest.onsuccess = function (event) {
            if (sid.length > 0) {
              intDelStat();
            } else {
              if (sucCb) {
                sucCb();
              }
            }
          };
          objectStoreRequest.onerror = function (event) {
            if (failCb) {
              failCb(event);
            }
          };
        }
        ;

        intDelStat();

      };

      var sendStats = function () {
        var doc = document.implementation.createDocument(null, "stats", null);

        var sendBatchCount = 50;

        Logger.countStats(
          function (count) {

            if (count === 0) {
              v_BacklogStats = false;
              if (!v_callbackCalled && successCb) {
                successCb();
              }
              return;
            } else if (count > 500) {
              v_BacklogStats = true;
            }

            if (v_BacklogStats) {
              sendBatchCount = 300;
            }

            Logger.getAllStatKeys(function (keys) {
                var keyMin = keys[0];
                var keyMax = keys[keys.length - 1];

                Logger.getAllStats(
                  function (values) {
                    for (var i = 0; i < values.length; i++) {
                      var de = doc.createElement("stat");
                      for (var prop in values[i]) {
                        de.setAttribute(prop, values[i][prop]);
                      }
                      doc.firstChild.appendChild(de);
                    }
                    var l = (new XMLSerializer()).serializeToString(doc);
                    CommunicationManager.sendStats(
                      function () {
                        application.statusPage.backlogStatsDate(utils.getISODate());

                        // Backlog should run on its own timing regardless of sendStats.
                        Logger.removeSpecificStats(
                          function () {
                            v_BacklogStatsCount -= values.length;
                            if (v_BacklogStatsCount < 0) {
                              v_BacklogStatsCount = 0;
                            }
                            application.statusPage.backlogStatsCount(v_BacklogStatsCount);
                            if (v_BacklogStats) {
                              setTimeout(sendStats, 30000);
                            }
                            if (!v_callbackCalled) {
                              v_callbackCalled = true;
                              if (successCb) {
                                successCb();
                              }
                            }
                          },
                          failureCb,
                          IDBKeyRange.bound(keyMin, keyMax)
                        );
                      },
                      failureCb,
                      l
                    );
                  },
                  function (event) {
                    console.error(event);
                    if (failureCb) {
                      failureCb(event);
                    }
                  },
                  sendBatchCount
                );
              },
              failureCb,
              sendBatchCount);
          },
          failureCb
        );
      };

//      console.time();
      var sendHourlyStats = function () {
        var sendBatchCount = 50;
        Logger.countStats(
          function (count) {
            if (count === 0) {
              v_BacklogStats = false;
              if (!v_callbackCalled && successCb) {
                successCb();
              }
              return;
            } else if (count >= 3600) {
              v_BacklogStats = true;
            }

            if (v_BacklogStats) {
              sendBatchCount = 300;
            }

            var doc = document.implementation.createDocument(null, "stats", null);

            var transaction = v_db.transaction([v_nameObjStoreStats], "readwrite");
            var request = transaction.objectStore(v_nameObjStoreStats).openCursor();
            var hStats = {};
            var cTime = new Date(utils.getISODate());
            cTime.setMinutes(0);
            cTime.setSeconds(0);

            var keysToDelete = [];


            function addHourlyStat(stat, dbkey) {
              if (cTime >= new Date(stat.todt)) {
                keysToDelete.push(dbkey);

                //XLG-142: Send proof of play stats for Splash screen
                if (stat.hasOwnProperty('layoutid') && stat.layoutid !== 'splash') {
                  //var key = stat.fromdt.substr(0,13)+" - "+stat.todt.substr(0,13)+' '+stat.mediaid+' '+stat.layoutid+' '+stat.scheduleid;
                  var key = stat.todt.substr(0, 13) + ' ' + stat.mediaid + ' ' + stat.layoutid + ' ' + stat.scheduleid;
                  var fdt = stat.fromdt;
                  var tdt = stat.todt;
                  //var dur = (new Date(tdt)-new Date(fdt))/1000;
                  var dur = stat.duration;

                  if (hStats[key]) {
                    if ((new Date(hStats[key].fromdt)) > (new Date(fdt))) {
                      hStats[key].fromdt = fdt;
                    }
                    if ((new Date(hStats[key].todt)) < (new Date(tdt))) {
                      hStats[key].todt = tdt;
                    }
                    hStats[key].count++;
                    hStats[key].duration += dur;
                  } else {
                    hStats[key] = {
                      fromdt: fdt,
                      todt: tdt,
                      layoutid: stat.layoutid,
                      scheduleid: stat.scheduleid,
                      type: stat.type,
                      count: 1,
                      duration: dur
                    };
                    if (stat.mediaid) {
                      hStats[key].mediaid = stat.mediaid;
                    }
                  }
                }
              }
            }

            request.onerror = function (event) {
              console.error(event);
              if (failureCb) {
                failureCb(event);
              }
            };
            request.onsuccess = function (event) {
              var cursor = event.target.result;
              if (cursor && Object.keys(hStats).length < sendBatchCount) {
                var stat = cursor.value;
                //	                  keysToDelete.push(cursor.key);

                var gStart = new Date(stat.fromdt.slice(0, 13) + ":00:00");
                var gEnd = new Date(gStart * 1 + 60 * 60 * 1000);

                var fstat = Object.assign({}, stat);
                var iter = 1000;

                while (true) {
                  // Calculate the duration beforehand!
                  if (new Date(stat.fromdt) < gStart) {
                    fstat.fromdt = utils.getISODate(gStart);
                  }
                  if (new Date(stat.todt) > gEnd) {
                    fstat.todt = utils.getISODate(gEnd);
                  }
                  fstat.duration = (new Date(fstat.todt) - new Date(fstat.fromdt)) / 1000;

                  //Always set statistics to boundaries
                  fstat.fromdt = utils.getISODate(gStart);
                  fstat.todt = utils.getISODate(gEnd);

                  //Stats ending on boundary have zero duration. Do not add them.
                  if (fstat.duration > 0) {
                    addHourlyStat(fstat, cursor.key);
                  }

                  if (gEnd > new Date(stat.todt)) {
                    break;
                  }

                  gStart = gEnd;
                  gEnd = new Date(gStart * 1 + 60 * 60 * 1000);

                  fstat = Object.assign({}, stat);

                  if (iter === 0) {
                    throw new Error("Endless loop detected");
                  } else {
                    iter--;
                  }
                }

                cursor.continue();
              } else {
                if (Object.keys(hStats).length > 0) {
                  for (var stat in hStats) {
                    var de = doc.createElement("stat");
                    for (var prop in hStats[stat]) {
                      de.setAttribute(prop, hStats[stat][prop]);
                    }

                    doc.firstChild.appendChild(de);
                  }
                  var l = (new XMLSerializer()).serializeToString(doc);
                  //console.debug("Submit stats:",l);
                  CommunicationManager.sendStats(
                    function () {
                      application.statusPage.backlogStatsDate(utils.getISODate());
                      var ktdCount = keysToDelete.length;
                      var keyToDeleteRange = IDBKeyRange.bound(keysToDelete[0], keysToDelete[keysToDelete.length - 1]);
                      Logger.removeSpecificStats(function () {
                          v_BacklogStatsCount -= ktdCount;
                          if (v_BacklogStatsCount < 0) {
                            v_BacklogStatsCount = 0;
                          }
                          application.statusPage.backlogStatsCount(v_BacklogStatsCount);

                          if (v_BacklogStats) {
//                          console.timeLog();
                            setTimeout(sendHourlyStats, 30000);
                          }
                          if (!v_callbackCalled) {
                            v_callbackCalled = true;
                            if (successCb) {
                              successCb();
                            }
                          }
                        },
                        failureCb,
                        keyToDeleteRange);
                    },
                    failureCb,
                    l);
                } else {
                  v_BacklogStats = false;
                  if (successCb) {
                    successCb();
                    return;
                  }
                }
              }
            };

          },
          function () {
            if (failureCb) {
              failureCb(event);
            }
          }
        );
      };

      var sendDailyStats = function () {

        var sendBatchCount = 50;
        Logger.countStats(
          function (count) {
            if (count === 0) {
              v_BacklogStats = false;
              if (!v_callbackCalled && successCb) {
                successCb();
              }
              return;
            } else if (count >= 86400) {
              //} else if (count >= 3000) { // THIS LINE IS FOR TESTING
              v_BacklogStats = true;
            }

            if (v_BacklogStats) {
              sendBatchCount = 300;
            }

            var doc = document.implementation.createDocument(null, "stats", null);

            var transaction = v_db.transaction([v_nameObjStoreStats], "readonly");
            var objectStore = transaction.objectStore(v_nameObjStoreStats);
            var request = objectStore.openCursor();
            var hStats = {};
            var cTime = new Date(utils.getISODate());
            cTime.setHours(0);
            cTime.setMinutes(0);
            cTime.setSeconds(0);

            var keysToDelete = [];

            function addDailyStat(stat, dbkey) {
              if (cTime >= new Date(stat.todt)) {
                keysToDelete.push(dbkey);

                //XLG-142: Send proof of play stats for Splash screen
                if (stat.hasOwnProperty('layoutid') && stat.layoutid !== 'splash') {
                  //var key = stat.fromdt.substr(0,10)+" - "+stat.todt.substr(0,10)+' '+stat.mediaid+' '+stat.layoutid+' '+stat.scheduleid;
                  var key = stat.todt.substr(0, 10) + ' ' + stat.mediaid + ' ' + stat.layoutid + ' ' + stat.scheduleid;
                  var fdt = stat.fromdt;
                  var tdt = stat.todt;
                  //var dur = (new Date(tdt)-new Date(fdt))/1000;
                  var dur = stat.duration;

                  if (hStats[key]) {
                    if ((new Date(hStats[key].fromdt)) > (new Date(fdt))) {
                      hStats[key].fromdt = fdt;
                    }
                    if ((new Date(hStats[key].todt)) < (new Date(tdt))) {
                      hStats[key].todt = tdt;
                    }
                    hStats[key].count++;
                    hStats[key].duration += dur;
                  } else {
                    hStats[key] = {
                      fromdt: fdt,
                      todt: tdt,
                      layoutid: stat.layoutid,
                      scheduleid: stat.scheduleid,
                      type: stat.type,
                      count: 1,
                      duration: dur
                    };
                    if (stat.mediaid) {
                      hStats[key].mediaid = stat.mediaid;
                    }
                  }
                }
              }
            }

            request.onerror = function (event) {
              console.error(event);
              if (failureCb) {
                failureCb(event);
              }
            };
            request.onsuccess = function (event) {
              var cursor = event.target.result;
              if (cursor && Object.keys(hStats).length < sendBatchCount) {
                var stat = cursor.value;
                //		            keysToDelete.push(cursor.key);

                var gStart = new Date(stat.fromdt.slice(0, 10) + " 00:00:00");
                var gEnd = new Date(gStart * 1 + 24 * 60 * 60 * 1000);

                var fstat = Object.assign({}, stat);
                var iter = 1000;

                while (true) {
                  // Calculate the duration beforehand!
                  if (new Date(stat.fromdt) < gStart) {
                    fstat.fromdt = utils.getISODate(gStart);
                  }
                  if (new Date(stat.todt) > gEnd) {
                    fstat.todt = utils.getISODate(gEnd);
                  }
                  fstat.duration = (new Date(fstat.todt) - new Date(fstat.fromdt)) / 1000;

                  //Always set statistics to boundaries
                  fstat.fromdt = utils.getISODate(gStart);
                  fstat.todt = utils.getISODate(gEnd);

                  //Stats ending on boundary have zero duration. Do not add them.
                  if (fstat.duration > 0) {
                    addDailyStat(fstat, cursor.key);
                  }

                  if (gEnd > new Date(stat.todt)) {
                    break;
                  }

                  gStart = gEnd;
                  gEnd = new Date(gStart * 1 + 24 * 60 * 60 * 1000);

                  fstat = Object.assign({}, stat);

                  if (iter === 0) {
                    throw new Error("Endless loop detected");
                  } else {
                    iter--;
                  }
                }

                cursor.continue();
              } else {
                if (Object.keys(hStats).length > 0) {
                  for (var stat in hStats) {
                    var de = doc.createElement("stat");
                    for (var prop in hStats[stat]) {
                      de.setAttribute(prop, hStats[stat][prop]);
                    }

                    doc.firstChild.appendChild(de);
                  }
                  var l = (new XMLSerializer()).serializeToString(doc);
                  //console.debug("Submit stats:",l);
                  CommunicationManager.sendStats(
                    function () {
                      application.statusPage.backlogStatsDate(utils.getISODate());
                      var ktdCount = keysToDelete.length;
                      var keyToDeleteRange = IDBKeyRange.bound(keysToDelete[0], keysToDelete[keysToDelete.length - 1]);
                      Logger.removeSpecificStats(function () {
                        v_BacklogStatsCount -= ktdCount;
                        if (v_BacklogStatsCount < 0) {
                          v_BacklogStatsCount = 0;
                        }
                        application.statusPage.backlogStatsCount(v_BacklogStatsCount);
                        if (v_BacklogStats) {
                          setTimeout(sendDailyStats, 30000);
                        }
                        if (!v_callbackCalled) {
                          v_callbackCalled = true;
                          if (successCb) {
                            successCb();
                          }
                        }
                      }, failureCb, keyToDeleteRange);
                    },
                    failureCb,
                    l
                  );
                } else {
                  v_BacklogStats = false;
                  if (successCb) {
                    successCb();
                    return;
                  }
                }
              }
            };
          },
          function () {
            if (failureCb) {
              failureCb(event);
            }
          }
        );
      };

      if (DeviceManager.Config.getOptions().hasOwnProperty('aggregationLevel')) {
        var aggrlevel = DeviceManager.Config.getOptions().aggregationLevel;
        switch (aggrlevel) {
          case Logger.AGGREGATIONLEVEL_HOURLY:
            sendHourlyStats();
            break;
          case Logger.AGGREGATIONLEVEL_DAILY:
            sendDailyStats();
            break;
          default:
            sendStats();
            break;
        }
      } else {
        sendStats();
      }
    };

    /**
     * Send details about any issues encountered back to the CMS.
     * @param {function} cbSuccess Function to call on success.
     * @param {function} cbFailure Function to call on failure.
     * @param {Object} fault The fault to report. See below.
     * <br>fault.code {Error.FaultCodes} The error code,
     * <br>fault.reason {String} The reason of the fault,
     * <br>fault.date {Date} The date the incident occured (dateTime string, Format Y-m-d H:i:s),
     * <br>fault.expires {Date} The date the fault expires (dateTime string, Format Y-m-d H:i:s),
     * <br>fault.layoutId {Number=} The ID of the affected Layout (integer),
     * <br>fault.regionId {Number=} The ID of the affected Region (integer),
     * <br>fault.widgetId {Number=} The ID of the affected Widget (integer),
     * <br>fault.scheduleId {Number=} The ID of the affected Schedule event (integer),
     * <br>fault.mediaId {Number=} The ID of the affected Media (integer)
     * <br>fault.onlyOnce {Boolean=} Should only send once per connection interval;
     * @returns undefined
     * @version 3.0.300
     * @since 3.0.300
     */
    this.cmsFault = function (cbSuccess, cbFailure, fault) {

      if (DeviceManager.Config.getCMSVersion() < 6) {
        var log = {category: "error", message: fault.reason};

        if (fault.scheduleID) {
          log.scheduleID = fault.scheduleID;
        }

        if (fault.layoutID) {
          log.layoutID = fault.layoutID;
        }

        if (fault.mediaID) {
          log.mediaID = fault.mediaID;
        }

        if (fault.type) {
          log.type = fault.type;
        }

        if (fault.method) {
          log.method = fault.method;
        }

        if (fault.thread) {
          log.thread = fault.thread;
        }

        if (fault.onlyOnce) {
          log.onlyOnce = fault.onlyOnce;
        }

        this.cmsLog(log, cbSuccess, cbFailure);
        return;

      }


      var v_fault = new faultEntry(fault.code, fault.reason, fault.date, fault.expires,
        fault.layoutId, fault.regionId, fault.widgetId, fault.scheduleId, fault.mediaId);

      v_fault.onlyOnce = fault.onlyOnce;

      //When a fault exists just change the expire date
      for (var i = 0; i < v_FaultsStorage.length; i++) {
        if (v_fault.equals(v_FaultsStorage[i])) {
          v_FaultsStorage[i].expires = v_fault.expires;
          if (cbSuccess) {
            cbSuccess();
          }
          return;
        }
      }
      v_FaultsStorage.push(v_fault);
      application.statusPage.backlogFaultsCount(v_FaultsStorage.length);
      application.statusPage.addLogMessage(v_fault);
      if (cbSuccess) {
        cbSuccess();
      }
    };

    /**
     * Submit the faults to the cms.
     * <br>This sunction is based on fire and forget logic.
     * <br>Calling successCb could take from 60 seconds to 30 minutes.
     * @param {Function} successCb Function to call on success.
     * @param {Function} failureCb Function to call on failure.
     * @returns None
     * @version 1.8.14
     * @since 1.8.0
     */
    this.submitFaults = function (successCb, failureCb) {
      if (DeviceManager.Config.getCMSVersion() < 6) {
        Logger.submitLog(successCb, failureCb);
        return;
      }

      //Handle the 429 response appropriately
      if (v_FaultsRetryTime > (new Date()).valueOf()) {
        return;
      }

      CommunicationManager.reportFaults(
        function () {
          v_FaultsStorage = [];
          application.statusPage.backlogFaultsCount(v_FaultsStorage.length);
          application.statusPage.backlogFaultsDate(utils.getISODate());
          if (successCb) {
            successCb();
          }
        },
        function (e) {
          if (e.code === 429) {
            var n = Number.parseInt(e.retryAfter);
            if (Number.isSafeInteger(n)) {
              v_FaultsRetryTime = (new Date()).valueOf() + n * 1000;
              console.log('Retry sending faults after ' + n + ' seconds!');
              setTimeout(
                function () {
                  Logger.submitFaults(successCb, failureCb);
                },
                n * 1100
              );
            }
          } else if (failureCb) {
            failureCb(e);
          }
        },
        v_FaultsStorage
      );
    };

    /**
     * Get all log keys from database.
     * @param {function} cbSuccess Function to call on success.<br>
     * Pass array of log keys.
     * @param {function} cbFailure Function to call on failure.<br>
     * Pass error object
     * @param {Number} maxCount The maximum number of log keys to return.
     * @returns {undefined}
     */
    this.getAllLogKeys = function (cbSuccess, cbFailure, maxCount) {
      var transaction = v_db.transaction([v_nameObjStoreLogs], "readwrite");
      var keysRequest = transaction.objectStore(v_nameObjStoreLogs).getAllKeys(undefined, maxCount);

      keysRequest.onerror = function (event) {
        console.error(event);
        if (cbFailure) {
          cbFailure(event);
        }
      };
      keysRequest.onsuccess = function (event) {
        //console.log(event.target.result);
        cbSuccess(event.target.result);
      };
    };

    /**
     * Get all keys from database.
     * @param {function} cbSuccess Function to call on success.<br>
     * Pass array of keys.
     * @param {function} cbFailure Function to call on failure.<br>
     * Pass error object
     * @param {Number} maxCount The maximum number of keys to return.
     * @returns {undefined}
     */
    this.getAllStatKeys = function (cbSuccess, cbFailure, maxCount) {
      var transaction = v_db.transaction([v_nameObjStoreStats], "readwrite");
      var keysRequest = transaction.objectStore(v_nameObjStoreStats).getAllKeys(undefined, maxCount);

      keysRequest.onerror = function (event) {
        console.error(event);
        if (cbFailure) {
          cbFailure(event);
        }
      };
      keysRequest.onsuccess = function (event) {
        //console.log(event.target.result);
        cbSuccess(event.target.result);
      };
    };

    /**
     * Get all keys from a specific objectStore of the database.
     * @param {function} cbSuccess Function to call on success.<br>
     * Pass array of faults keys.
     * @param {function} cbFailure Function to call on failure.<br>
     * Pass error object
     * @param {String} objectStoreName The name of the objectStore to query for the keys.
     * @param {Number} maxCount The maximum number of faults keys to return.
     * @returns {undefined}
     * @version 3.0.300
     * @since 3.0.300
     */
    this.getAllKeys = function (cbSuccess, cbFailure, objectStoreName, maxCount) {
      var transaction = v_db.transaction([objectStoreName], "readwrite");
      var keysRequest = transaction.objectStore(objectStoreName).getAllKeys(undefined, maxCount);

      keysRequest.onerror = function (event) {
        console.error(event);
        if (cbFailure) {
          cbFailure(event);
        }
      };
      keysRequest.onsuccess = function (event) {
        //console.log(event.target.result);
        cbSuccess(event.target.result);
      };
    };

    /**
     * Get all entries from database.
     * @param {function} cbSuccess Function to call on success.<br>
     * Pass array of entries.
     * @param {function} cbFailure Function to call on failure.<br>
     * Pass error object
     * @param {Number} maxCount The maximum number of entries to return.
     * @returns {undefined}
     * @version 2.0.205
     * @since 2.0.205
     */
    this.getAllStats = function (cbSuccess, cbFailure, maxCount) {
      var transaction = v_db.transaction([v_nameObjStoreStats], "readwrite");
      var request = transaction.objectStore(v_nameObjStoreStats).getAll(undefined, maxCount);
      request.onerror = function (event) {
        console.error(event);
        if (cbFailure) {
          cbFailure(event);
        }
      };
      request.onsuccess = function (event) {
        //console.log(event.target.result);
        cbSuccess(event.target.result);
      };
    };

    /**
     * Get all log entries from database.
     * @param {function} cbSuccess Function to call on success.<br>
     * Pass array of entries.
     * @param {function} cbFailure Function to call on failure.<br>
     * Pass error object
     * @param {Number} maxCount The maximum number of entries to return.
     * @returns {undefined}
     * @version 2.0.205
     * @since 2.0.205
     */
    this.getAllLogs = function (cbSuccess, cbFailure, maxCount) {
      var transaction = v_db.transaction([v_nameObjStoreLogs], "readwrite");
      var request = transaction.objectStore(v_nameObjStoreLogs).getAll(undefined, maxCount);
      request.onerror = function (event) {
        console.error(event);
        if (cbFailure) {
          cbFailure(event);
        }
      };
      request.onsuccess = function (event) {
        //console.log(event.target.result);
        cbSuccess(event.target.result);
      };
    };

    /**
     * Get all entries from a specific objectStore of the database.
     * @param {function} cbSuccess Function to call on success.<br>
     * Pass array of entries.
     * @param {function} cbFailure Function to call on failure.<br>
     * Pass error object
     * @param {String} objectStoreName The name of the objectStore to query for the entries.
     * @param {Number} maxCount The maximum number of entries to return.
     * @returns {undefined}
     * @version 2.0.205
     * @since 2.0.205
     */
    this.getAllEntries = function (cbSuccess, cbFailure, objectStoreName, maxCount) {
      var transaction = v_db.transaction([objectStoreName], "readwrite");
      var request = transaction.objectStore(objectStoreName).getAll(undefined, maxCount);
      request.onerror = function (event) {
        console.error(event);
        if (cbFailure) {
          cbFailure(event);
        }
      };
      request.onsuccess = function (event) {
        //console.log(event.target.result);
        cbSuccess(event.target.result);
      };
    };


    /**
     * Clear all logs from db.
     * @param {function} successCb Callback funtion after success.
     * @param {function} failureCb Callback funtion after failure.
     * @returns {undefined}.
     * @version 2.0.200
     * @since 2.0.200
     */
    this.clearLogs = function (successCb, failureCb) {
      var transaction = v_db.transaction([v_nameObjStoreLogs], "readwrite");
      var objectStore = transaction.objectStore(v_nameObjStoreLogs);
      var objectStoreRequest = objectStore.clear();
      objectStoreRequest.onsuccess = function (event) {
        application.statusPage.backlogLogsCount(0);
        v_BacklogLogsCount = 0;
        if (successCb) {
          successCb();
        }
      };
      objectStoreRequest.onerror = function (event) {
        if (failureCb) {
          failureCb(event);
        }
      };
    };

    /**
     * Clear all statistics from db.
     * @param {function} successCb Callback funtion after success.
     * @param {function} failureCb Callback funtion after failure.
     * @returns {undefined}.
     * @version 2.0.200
     * @since 2.0.200
     */
    this.clearStats = function (successCb, failureCb) {
      var transaction = v_db.transaction([v_nameObjStoreStats], "readwrite");
      var objectStore = transaction.objectStore(v_nameObjStoreStats);
      var objectStoreRequest = objectStore.clear();
      objectStoreRequest.onsuccess = function (event) {
        application.statusPage.backlogStatsCount(0);
        v_BacklogStatsCount = 0;
        if (successCb) {
          successCb();
        }
      };
      objectStoreRequest.onerror = function (event) {
        if (failureCb) {
          failureCb(event);
        }
      };
    };

    /**
     * Remove the Stats in keys
     * @param {function} successCb Callback function on success.
     * @param {function} failureCb Callback function on failure.
     * @param {IDBKeyRange} keyRangeValue Any key range from only(), lowerBound(), upperBound(), bound() methods.<br>
     * Be carefull, it needs the keys, not the index.
     * @returns {undefined}
     * @version 2.0.200
     * @since 2.0.200
     */
    this.removeSpecificStats = function (successCb, failureCb, keyRangeValue) {
      var transaction = v_db.transaction([v_nameObjStoreStats], "readwrite");
      var objectStore = transaction.objectStore(v_nameObjStoreStats);
      var request = objectStore.delete(keyRangeValue);
      request.onsuccess = function () {
        if (successCb) {
          successCb();
        }
      };
      request.onerror = function () {
        if (failureCb) {
          failureCb();
        }
      };
    };

    /**
     * Remove the Stats in keys
     * @param {function} successCb Callback function on success.

     * @param {function} failureCb Callback function on failure.
     * @param {IDBKeyRange} keyRangeValue Any key range from only(), lowerBound(), upperBound(), bound() methods.<br>
     * Be carefull, it needs the keys, not the index.
     * @returns {undefined}
     * @version 2.0.205
     * @since 2.0.205
     */
    this.removeSpecificLogs = function (successCb, failureCb, keyRangeValue) {
      var transaction = v_db.transaction([v_nameObjStoreLogs], "readwrite");
      var objectStore = transaction.objectStore(v_nameObjStoreLogs);
      var request = objectStore.delete(keyRangeValue);

      request.onsuccess = function () {
        //console.log(objStoreRequest);
        if (successCb) {
          successCb();
        }
      };
      request.onerror = function () {
        //console.log(objStoreRequest);
        if (failureCb) {
          failureCb();
        }
      };
    };

    /**
     * Remove a range of entries from the specific objectStore.
     * @param {function} successCb Callback function on success.
     * @param {function} failureCb Callback function on failure.
     * @param {String} objectStoreName The name of the objectStore to remove the entries.
     * @param {IDBKeyRange} keyRangeValue Any key range from only(), lowerBound(), upperBound(), bound() methods.<br>
     * Be carefull, it needs the keys, not the index.
     * @returns {undefined}
     * @version 2.0.205
     * @since 2.0.205
     */
    this.removeSpecificEntries = function (successCb, failureCb, objectStoreName, keyRangeValue) {
      var transaction = v_db.transaction([objectStoreName], "readwrite");
      var objectStore = transaction.objectStore(objectStoreName);
      var request = objectStore.delete(keyRangeValue);

      request.onsuccess = function () {
        //console.log(objStoreRequest);
        if (successCb) {
          successCb();
        }
      };
      request.onerror = function () {
        //console.log(objStoreRequest);
        if (failureCb) {
          failureCb();
        }
      };
    };

    /**
     * Count how many statistics has been logged but not send to CMS.
     * @param {function} successCb Callback function on successfull count of statistics.<br>
     * Returns the number of statistics.
     * @param {function} failureCb Callback function on failure.
     * @returns {undefined}
     * @version 2.0.205
     * @since 2.0.205
     */
    this.countStats = function (successCb, failureCb) {
      var transaction = v_db.transaction([v_nameObjStoreStats], "readwrite");
      var request = transaction.objectStore(v_nameObjStoreStats).count();
      transaction.onerror = function () {
        console.error('Error counting statistics', transaction.error);
        failureCb(transaction.error);
      };
      request.onsuccess = function () {
        console.info('Stats count', request.result);
        if (application.statusPage && application.statusPage.backlogStatsCount) {
          application.statusPage.backlogStatsCount(request.result);
        }
        v_BacklogStatsCount = request.result;
        if (successCb) {
          successCb(request.result);
        }
      };

      request.onerror = function (event) {
        if (failureCb) {
          failureCb(event);
        }
      };
    };

    /**
     * Count how many logs has been logged but not send to CMS.
     * @param {function} successCb Callback function on successfull count of logs.<br>
     * Returns the number of logs.
     * @param {function} failureCb Callback function on failure.
     * @returns {undefined}
     * @version 2.0.205
     * @since 2.0.205
     */
    this.countLogs = function (successCb, failureCb) {
      var transaction = v_db.transaction([v_nameObjStoreLogs], "readwrite");
      var request = transaction.objectStore(v_nameObjStoreLogs).count();
      transaction.onerror = function () {
        console.error('Error counting statistics', transaction.error);
        failureCb(transaction.error);
      };
      request.onsuccess = function () {
        //console.info('Stats count', request.result);
        if (application.statusPage && application.statusPage.backlogLogsCount) {
          application.statusPage.backlogLogsCount(request.result);
        }
        v_BacklogLogsCount = request.result;
        if (successCb) {
          successCb(request.result);
        }
      };

      request.onerror = function (event) {
        if (failureCb) {
          failureCb(event);
        }
      };
    };

    /**
     * Count how many entries are on the specific objectStore that has been logged but not send to CMS.
     * @param {function} successCb Callback function on successfull count of faults.<br>
     * Returns the number of faults.
     * @param {function} failureCb Callback function on failure.
     * @param {String} objectStoreName The name of the objectStore to remove the entries.
     * @returns {undefined}
     * @version 3.0.300
     * @since 3.0.300
     */
    this.countEntries = function (successCb, failureCb, objectStoreName) {
      var transaction = v_db.transaction([objectStoreName], "readwrite");
      var request = transaction.objectStore(objectStoreName).count();
      transaction.onerror = function () {
        console.error('Error counting entries', transaction.error);
        failureCb(transaction.error);
      };
      request.onsuccess = function () {
        if (successCb) {
          successCb(request.result);
        }
      };

      request.onerror = function (event) {
        if (failureCb) {
          failureCb(event);
        }
      };
    };


    /**
     * Initialize logger.
     * @param {type} cbSuccess
     * @param {type} cbFailure
     * @returns {undefined}
     * @version 1.8.15
     * @since 1.8.15
     */
    this.init = function (cbSuccess, cbFailure) {

      var retryCount = 3;
      var lastError = {};

      var tryInit = function () {
        retryCount--;
        if (retryCount > 0) {
          prepareDB(cbSuccess,
            function (errObj) {
              for (var propKey in errObj) {
                lastError[propKey] = errObj[propKey];
              }
              setTimeout(tryInit, 1000);
            });
        } else {
          if (cbFailure) {
            cbFailure(lastError);
          }
        }
      };
      tryInit();
    };

    /**
     * Get a reference to IndexedDB.
     * Useful for testing.
     * @returns {Object} Reference to IndexedDB instance.
     */
    this.getIndexedDBInstance = function () {
      return v_db;
    };
    // </editor-fold>
  };

  utils.const(Logger, 'AGGREGATIONLEVEL_INDIVIDUAL', 'Individual');
  utils.const(Logger, 'AGGREGATIONLEVEL_HOURLY', 'Hourly');
  utils.const(Logger, 'AGGREGATIONLEVEL_DAILY', 'Daily');

  /**
   * Error log level constant.
   */
  utils.const(Logger, 'LOGLEVEL_ERROR', 'error');

  /**
   * Audit log level constant.
   */
  utils.const(Logger, 'LOGLEVEL_AUDIT', 'audit');
  // </editor-fold>
}());
