/* global LayoutManager, utils, Logger */

(function () {
  "use strict";

  /**
   * Load and play an overlay audio of an element.<br>
   * @param {Object} media The audio information.
   * @param {Object} container Not Used.
   * @param callback
   * @param layoutid
   * @param scheduleid
   * @param scp
   * @returns {LayoutManager.audio} Return an audio object.
   */
  window.LayoutManager.audio = function (media, container, callback, layoutid, scheduleid, scp) {
    try {

      EventEmitter.call(this);

      var parent = this;
      parent.media = media;
      parent.container = container;
      var source = "file:///opt/usr/home/owner/content/Downloads/" + media.media.options[0].uri[0].jValue;
      var loop = 0;
      try {
        loop = parseInt(media.media.options[0].loop[0].jValue);
      } catch (error) {
        console.warn(error);
      }
      this.duration = parseInt(media.duration);
      var isPaused = false;
      var bufSource = null;
      this.statsStartTime = utils.getISODate();
      var duration = this.duration;
      var timeStarted = 0;
      var timePlayed = 0;


      //Event emmiters
      var widgetStarted = function () {
        var widgetstartedevt = new CustomEvent('widgetStarted', {
          detail: {}
        });
        parent.dispatchEvent(widgetstartedevt);
      };
      var widgetEnded = function () {
        var widgetendevt = new CustomEvent('widgetEnded', {
          detail: {}
        });
        parent.dispatchEvent(widgetendevt);
      };

      var prepareAudio = function (resolveCb, failureCb) {
        //Logger.log("Preload " + source, Logger.DEBUG);

        /* Create a buffer source for the specific file. */
        bufSource = LayoutManager.audioContext.createBufferSource(); // creates a sound source

        /* Create a gain node */
        var gain = LayoutManager.audioContext.createGain();

        /* Wire nodes */
        bufSource.connect(gain); // connect the source to the gain node for volume setting.
        gain.connect(LayoutManager.audioContext.destination); // connect the gain node to the context's destination (the speakers)

        if (loop) {
          if (Number(loop) === 1) {
            bufSource.loop = true;
          }
        }
        ;

        if (media.volume) {
          try {
            gain.gain.value = Number(media.volume) / 100;
          } catch (e) {
            gain.gain.value = 1;
          }
        }
        ;

        var request = new XMLHttpRequest();
        request.open('GET', source, true);

        request.responseType = 'arraybuffer';

        // Decode asynchronously
        request.onload = function () {
          //Logger.log("Loaded " + source, Logger.DEBUG);
          LayoutManager.audioContext.decodeAudioData(request.response,
            function (buffer) {
              //Logger.log("Decoded " + source, Logger.DEBUG);
              bufSource.buffer = buffer;
              if (parent.duration === 0) {
                getDuration(source, function (dur) {
                  parent.duration = dur;
                  duration = dur;
                  if (resolveCb) {
                    resolveCb();
                  }
                });
              } else {
                if (resolveCb) {
                  resolveCb();
                }
              }
            },
            function (evt) {
              //Logger.log("error", Logger.WARN, evt);
              if (failureCb) {
                failureCb(evt);
              }
            });
        };

        request.send();
      };


      var play = function () {
        bufSource.onended = function () {
          if (!isPaused) {
            bufSource.stop();
            bufSource = null;
            widgetEnded();
          }
        };
        bufSource.start(0, timePlayed);
        timeoutMediaInvoke(parent);
      };

      var stop = function () {
        if (bufSource) {//If source buffer is valid.
          bufSource.stop();
        }
      };

      this.timeoutMedia = function () {
        timeStarted = (new Date()).valueOf();

        parent.timeoutElement = setTimeout(function () {
          if (scheduleid && media.enableStat !== "0") {
            Logger.logStat({
              fromdt: parent.statsStartTime,
              todt: utils.getISODate(),
              scheduleid: scheduleid,
              layoutid: layoutid,
              mediaid: media.id
            });
          }
          stop();
          widgetEnded();
        }, parent.duration * 1000, layoutid, scp);
      };

      var timeoutMediaInvoke = this.timeoutMedia;

      function getDuration(src, cb) {
        var audio = new Audio(src);
        var internalLoadedMetadata = function () {
          audio.removeEventListener("loadedmetadata", internalLoadedMetadata);
          var d = audio.duration;
          audio = null;
          cb(d);
        };
        audio.addEventListener("loadedmetadata", internalLoadedMetadata);
      };

      prepareAudio(function (bfr) {
        //timeoutMediaInvoke(parent);
        play(bfr);
      }, null);

      this.stopMediaTimeout = function () {
        if (parent) {
          stop();
          clearTimeout(parent.timeoutElement);
        }
      };

      /**
       * Expire media immediately
       */
      this.expireMedia = function () {
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = null;
        widgetExpired();
      };

      /**
       * Extend media duration
       * @param {Number} dur Duration to extend in seconds
       */
      this.extendDuration = function (dur) {
        duration += dur * 1000;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };

      /**
       * Set media duration
       * @param {Number} dur Duration to set in seconds
       */
      this.setDuration = function (dur) {
        duration = dur * 1000;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };


      /**
       * Pause the audio widget. The audio element will hide it self!
       * Duration will be retained too!
       */
      this.pause = function () {
        if (!isPaused) {
          stop();
          //Calculate remain duration
          timePlayed += (new Date()).valueOf() - timeStarted;
          duration -= (new Date()).valueOf() - timeStarted;
          if (media.media.action) {
            media.media.action.forEach(function (action) {
              InteractiveManager.removeEventAction(action.jAttr.id);
            });
          }
          isPaused = true;
        }
      };

      /**
       * Resume the video widget. The video element will show it self!
       */
      this.resume = function () {
        if (isPaused) {
          prepareAudio(function (bfr) {
            play(bfr);
          }, null);
          isPaused = false;
        }
      };

    } catch (e) {
      console.error(e);
    }

  };
}());
