/* global DeviceManager, utils, ko, AppVersionInformation */
(function () {
  "use strict";

  /**
   * @classdesc Status page view model.
   * @type statusViewModel
   * @version 1.8.1
   * @since 1.8.1
   */
  window.statusViewModel = function _statusViewModel() {
    "use strict";
    /**
     * Formater of the layout type
     * @param {Object} target The target element that will have the formatted option.
     * @param {Object} options Object containing what type to hide.<br>
     *  options.hideDefault - Hides the default layout decorator
     *  options.hideOverlay - Hides the overlay layout decorator
     *  options.hideInterrupt - Hides the default layout decorator
     *  options.hideScheduledIndication - Hides the scheduled layout decorator
     * @returns {Object} The target object
     */
    ko.extenders.layoutFormater = function (target, options) {
      target.decorated = ko.computed({
        read: function () {
          var val = target();
          var ret = [];
          var cosms;

          if (val.length === 0) {
            return val;
          }

          if (val[0].id.toString().toLowerCase() == 'splash' || val[0].id == '0') {
            return ['Splash'];
          }

          for (var v in val) {

            if (!(options &&
              options.hideScheduledIndication &&
              options.hideScheduledIndication === true)) {
              cosms = (val[v].isScheduled) ? '' : '*';
            } else {
              cosms = '';
            }

            switch (val[v].type) {
              case 'default':
                if (!(options && options.hideDefault && options.hideDefault === true)) {
                  cosms += ' (D)';
                }
                break;
              case 'overlay':
                if (!(options && options.hideOverlay && options.hideOverlay === true)) {
                  cosms += ' (O)';
                }
                break;
              case 'interrupt':
                if (!(options && options.hideInterrupt && options.hideInterrupt === true)) {
                  cosms += ' (I)';
                }
                break;
              default:
                //Normal layouts don't have decorators
                break;
            }

            ret.push(val[v].id + cosms);
          }
          return ret;
        }
      });
      return target;
    };

    var self = this;

    var v_clearLogTimer = null;

    self.filterById = function (id) {
      if (typeof (id) === 'undefined') {
        return this.allLayoutsSet();
      } else {
        return ko.utils.arrayFilter(this.allLayoutsSet(), function (item) {
          return item.name === id;
        });
      }
    };

    self.filterByScheduled = function (isScheduled) {
      if (typeof (isScheduled) === 'undefined') {
        return this.allLayoutsSet();
      } else {
        return ko.utils.arrayFilter(this.allLayoutsSet(), function (item) {
          return item.isScheduled() === isScheduled;
        });
      }
    };

    self.filterByValid = function (isValid) {
      if (typeof (isValid) === 'undefined') {
        return this.allLayoutsSet();
      } else {
        return ko.utils.arrayFilter(this.allLayoutsSet(), function (item) {
          return item.isValid() === isValid;
        });
      }
    };

    self.filterByOverlay = function (isOverlay) {
      if (typeof (isOverlay) === 'undefined') {
        return this.allLayoutsSet();
      } else {
        return ko.utils.arrayFilter(this.allLayoutsSet(), function (item) {
          return item.isOverlay() === isOverlay;
        });
      }
    };

    /**
     * Create a layout status object.
     *
     * @param {string} name The name (id) of the layout.
     * @param {boolean} isDefault Flag for default layout.
     * @param {boolean} isValid Flag for valid layout.
     * @param {boolean} isScheduled Flag if layout is sceduled.
     * @param {boolean} isOverlay Flag if overlay.
     */
    self.createLayoutInfo = function (name, isDefault, isValid, isScheduled, isOverlay) {
      var valid;
      if (typeof (isValid) === "boolean") {
        valid = isValid;
      } else if (typeof (isValid) === 'undefined') {
        valid = typeof (isValid) === 'undefined' ? false : isValid;
      } else {
        valid = (isValid === 1) ? true : false;
      }
      isDefault = typeof (isDefault) === 'undefined' ? false : isDefault;
      isScheduled = typeof (isScheduled) === 'undefined' ? false : isScheduled;
      isOverlay = typeof (isOverlay) === 'undefined' ? false : isOverlay;

      var layoutInfo = {
        name: name,
        isDefault: ko.observable(isDefault),
        isValid: ko.observable(valid),
        isScheduled: ko.observable(isScheduled),
        isOverlay: ko.observable(isOverlay)
      };

      layoutInfo.toString = ko.computed(function () {
        var str = layoutInfo.name;
        if (!layoutInfo.isScheduled()) {
          str += "*";
        }
        if (layoutInfo.isDefault()) {
          str += " (D)";
        }
        if (layoutInfo.isOverlay()) {
          str += " (O)";
        }
        return str;
      });

      return layoutInfo;
    };

    self.appVersion = ko.observable(DeviceManager.Device.getPlatformInfo().clientVersion);
    self.appVerCode = ko.observable(DeviceManager.Device.getPlatformInfo().clientCode);
    self.cmsAddr = ko.observable(DeviceManager.Config.getURL());
    self.isDevVersion = ko.pureComputed(function () {
      return AppVersionInformation.isDevVersion;
    });

    self.patchVersion = ko.pureComputed(function () {
      return AppVersionInformation.patchVersion;
    });

    self.tizenVersion = ko.observable();
    self.deviceModel = ko.observable();
    self.firmwareVersion = ko.observable();
    self.hasWifi = ko.observable();
    self.wifiSignal = ko.observable();
    self.xmrAddr = ko.observable();
    self.lastXMRTimestamp = ko.observable();
    self.playerDateTime = ko.observable();
//		self.xmrAddrDispl = ko.pureComputed(function(){
//		return "("+self.xmrAddr()+")";
//		});
    self.downloadFileId = ko.observable();
    self.downloadStatus = ko.observable();
    self.downloadProgress = ko.computed(function () {
      var status = self.downloadStatus() ? self.downloadStatus() + '%' : '';
      var id = status ? ' - ' + self.downloadFileId() : self.downloadFileId();
      var progress = status && id ? '(' + status + id + ')' : '';
      return progress;
    }).extend({rateLimit: 250});
    self.isInsideDownloadWindow = ko.observable(false);
    self.downloadWindowStart = ko.observable();
    self.downloadWindowEnd = ko.observable();
    self.updateWindowStart = ko.observable();
    self.updateWindowEnd = ko.observable();
    self.downloadWindow = ko.computed(function () {
      if (self.downloadWindowStart() === self.downloadWindowEnd()) {
        return 'Always';
      }

      return self.downloadWindowStart() + ' - ' + self.downloadWindowEnd();
    });
    self.currentTime = ko.computed(function () {
      var time = utils.getISODate().substr(11, 5);
      self.isInsideDownloadWindow();
      return time;
    });
    self.memoryTotal = ko.observable();
    self.memoryFree = ko.observable();
    self.memoryUsed = ko.observable();

    self.spaceTotal = ko.observable();
    self.spaceFree = ko.observable();
    self.spaceUsed = ko.observable();

    self.spaceAvail = ko.pureComputed(function () {
      //jshint -W016
      return (((self.spaceFree() / self.spaceTotal()) * 100) | 0) + "%";
      //jshint +W016
    });

    self.displayName = ko.observable(DeviceManager.Config.getDisplayName());

    self.mainLicenseRemain = ko.observable();
    self.trialLicenseRemain = ko.observable();

    self.licenseValue = ko.computed(function () {
      if (self.mainLicenseRemain() > utils.getTimeStamp()) {
        return "Licensed.";
      } else if (self.trialLicenseRemain() > utils.getTimeStamp()) {
        var remain = new Date((self.trialLicenseRemain() - utils.getTimeStamp()) * 1000);
        return "Trial (Remain " + remain.getDate() + " days).";
      } else {
        return "No license.";
      }
    });

    self.cmsRegistered = ko.observable(false);
    self.currentLayout = ko.observable();
    self.screenSize = ko.observable();
    self.screenShotInterval = ko.observable("0");

    self.backlogStatsCount = ko.observable(0);
    self.backlogStatsDate = ko.observable();
    self.backlogLogsCount = ko.observable(0);
    self.backlogLogsDate = ko.observable();

    self.supportsFaults = ko.observable(DeviceManager.Config.getCMSVersion() >= 6);
    self.backlogFaultsCount = ko.observable(0);
    self.backlogFaultsDate = ko.observable();

    self.cmsRegInfo = ko.observable("Not registered.");
    self.cmsRegInfoTime = ko.observable();

    self.scheduleStatus = ko.observable();
    self.requiredFilesSkipped = ko.observable(false);
    //self.reqFilesStatus = ko.observable();
    self.downloadedFiles = ko.observable(0);
    self.totalFiles = ko.observable(0);
    self.remainingFiles = ko.observable(0); // Queued Number Connections
    self.failedXMDSConnections = ko.observable(0);

    self.nextLayout = ko.observable();

    /**
     * This contains all layouts that are set to the player.
     * Each layout shoule be an object like:
     * {
     *   id : {Number},
     *   isValid : {Boolean},
     *   isScheduled : {Boolean},
     *   type: {String} "default"|"normal or undefined"|"overlay"|"interrupt",
     * }
     */
    self.allLayouts = ko.observableArray().extend({rateLimit: 100, layoutFormater: ""});

    /* Invalid Layouts */
    self.validLayouts = ko.computed(function () {
      var a = [];
      ko.utils.arrayForEach(self.allLayouts(), function (item) {
        if (item.isValid) {
          a.push(item);
        }
      });

      return a;
    }).extend({
      rateLimit: 100, layoutFormater: {
        hideDefault: true,
        hideOverlay: true,
        hideInterrupt: true,
        hideScheduledIndication: true
      }
    });

    /* Invalid Layouts */
    self.invalidLayouts = ko.computed(function () {
      var a = [];
      ko.utils.arrayForEach(self.allLayouts(), function (item) {
        if (!item.isValid) {
          a.push(item);
        }
      });

      return a;
    }).extend({
      rateLimit: 100, layoutFormater: {
        hideDefault: true,
        hideOverlay: true,
        hideInterrupt: true,
        hideScheduledIndication: true
      }
    });

    /* Scheduled Layouts */
    self.scheduledLayouts = ko.observableArray();
    self.scheduledOverlayLayouts = ko.observableArray();

    self.scheduledLayouts_view = ko.computed(function () {
      var a = [];


      ko.utils.arrayForEach(self.scheduledLayouts(), function (item) {
        if (item.isScheduled) {
          a.push(item);
        }
      });
      ko.utils.arrayForEach(self.scheduledOverlayLayouts(), function (item) {
        if (item.isScheduled) {
          a.push(item);
        }
      });


      return a;
    }).extend({rateLimit: 100, layoutFormater: ""});

    self.setScheduledOverlayLayouts = function (arr) {
      var a = [];
      ko.utils.arrayForEach(self.allLayouts(), function (item) {
        if (item.type === "overlay") {
          item.isScheduled = false;
        }
      });

      var c = self.allLayouts();
      arr.forEach(function (el) {
        var obj = undefined;
        for (var i = 0; i < c.length; i++) {
          if (el.layoutId == c[i].id) {
            obj = c[i];
            break;
          }
        }
        if (obj) {
          obj.isScheduled = true;
          a.push(obj);
        }
        ;
      });
      self.scheduledOverlayLayouts(a);
      self.scheduledLayouts_view.notifySubscribers();
      self.allLayouts.notifySubscribers();
    };

    self.setScheduledLayouts = function (arr) {
      var a = [];
      ko.utils.arrayForEach(self.allLayouts(), function (item) {
        if (item.type !== "overlay") {
          item.isScheduled = false;
        }
      });

      var c = self.allLayouts();
      arr.forEach(function (el) {
        var obj = undefined;
        for (var i = 0; i < c.length; i++) {
          if (el.layoutId == c[i].id) {
            obj = c[i];
            break;
          }
        }
        if (obj) {
          obj.isScheduled = true;
          a.push(obj);
        }
        ;
      });

      self.scheduledLayouts(a);
      self.scheduledLayouts_view.notifySubscribers();
      self.allLayouts.notifySubscribers();
    };

    self.nextScheduleUpdate = ko.observable();
    self.blacklist = ko.observableArray();

    /**
     * Contains the last 5 log messages. Don't use to push logs, use {@see addLogMessage} for that.
     */
    self.logMessages = ko.observableArray();

    self.versionInformationCMS = ko.observable();

    self.addLogMessage = function (logEntry) {
      if (self.logMessages().length >= 5) {
        self.logMessages.shift();
      }
      self.logMessages.push(logEntry);
    };


    self.timelyClearLogs = function () {
      if (v_clearLogTimer !== null) {
        clearTimeout(v_clearLogTimer);
      }

      /* Automatic clear  */
      setTimeout(function () {
        self.logMessages.removeAll();
      }, 5 * 60 * 1000);
    };
    //<div>Image Cache Hit Rate: <span>0.25</span></div>
    //<div>Use Surface View for Video Rendering?: <span>true</span></div>

  };
}());